@@ -32,16 +32,13 @@
 #include "gretl_restrict.h"
 #include "gretl_func.h"
 #include "libset.h"
-#include "forecast.h"
 #include "cmd_private.h"
 #include "flow_control.h"
 #include "objstack.h"
 #include "gretl_xml.h"
-#include "gretl_panel.h"
-#include "usermat.h"
 #include "gretl_string_table.h"
+#include "gretl_bundle.h"
 #include "dbread.h"
-#include "boxplots.h"
 #include "gretl_www.h"
 #include "uservar.h"
 #include "csvdata.h"
@@ -215,13 +212,6 @@ static int cli_clear_data (CMD *cmd, DATASET *dset, MODEL *model)
     return err;
 }
 
-static void get_a_filename (char *fname)
-{
-    *fname = 0;
-
-    fgets(fname, MAXLEN - 1, stdin);
-}
-
 static const char *get_prompt (ExecState *s)
 {
     if (s->flags & DEBUG_EXEC) {
@@ -432,12 +422,11 @@ int main (int argc, char *argv[])
 #endif
 
     libgretl_init();
-#if 0
-    gretl_www_init(NULL, NULL, 0);
-#endif
 
-    logo(quiet); 
-    session_time(NULL);
+    logo(quiet);
+    if (!quiet) {
+	session_time(NULL);
+    }
 
     prn = gretl_print_new(GRETL_PRINT_STDOUT, &err);
     if (err) {
@@ -606,10 +595,14 @@ int main (int argc, char *argv[])
 	}
 
 	if (!state.in_comment) {
-	    err = maybe_get_input_line_continuation(line); 
-	    if (err) {
-		errmsg(err, prn);
-		break;
+	    if (cmd.context == FOREIGN) {
+		tailstrip(line);
+	    } else {
+		err = maybe_get_input_line_continuation(line); 
+		if (err) {
+		    errmsg(err, prn);
+		    break;
+		}
 	    }
 	} 
 
@@ -752,8 +745,8 @@ static int cli_open_append (CMD *cmd, const char *line,
 	strcmp(newfile, datafile)) {
 	fprintf(stderr, _("Opening a new data file closes the "
 			  "present one.  Proceed? (y/n) "));
-	fgets(response, sizeof response, stdin);
-	if (*response != 'y' && *response != 'Y') {
+	if (fgets(response, sizeof response, stdin) != NULL && 
+	    *response != 'y' && *response != 'Y') {
 	    pprintf(prn, _("OK, staying with current data set\n"));
 	    return 0;
 	}
@@ -902,7 +895,7 @@ static int exec_line (ExecState *s, DATASET *dset)
     }
 
     if (cmd->ci < 0) {
-	/* there's nothing there */ 	
+	/* nothing there, comment, or masked by "if" */
 	return 0;
     }
 
@@ -955,12 +948,23 @@ static int exec_line (ExecState *s, DATASET *dset)
 
     case DELEET:
 	if (cmd->opt & OPT_D) {
+	    /* delete from database */
 	    err = db_delete_series_by_name(cmd->param, prn);
+	} else if (cmd->opt & OPT_T) {
+	    /* delete all by type (not series) */
+	    err = gretl_cmd_exec(s, dset);
 	} else if (*cmd->param != '\0') {
+	    /* got a named non-series variable */
 	    err = gretl_delete_var_by_name(cmd->param, prn);
-	} else if (get_list_by_name(cmd->extra)) {
-	    err = user_var_delete_by_name(cmd->extra, prn);
+	} else if (*cmd->extra != '\0') {
+	    /* "extra" means we got "list <listname> delete" */
+	    if (get_list_by_name(cmd->extra)) {
+		err = user_var_delete_by_name(cmd->extra, prn);
+	    } else {
+		err = E_UNKVAR;
+	    }
 	} else {
+	    /* list of series? */
 	    int nv = 0;
 
 	    err = dataset_drop_listed_variables(cmd->list, dset, 
@@ -1021,10 +1025,11 @@ static int exec_line (ExecState *s, DATASET *dset)
 	}
 
 	printf(_("type a filename to store output (enter to quit): "));
-	get_a_filename(outfile);
-	top_n_tail(outfile, 0, NULL);
-
-	if (*outfile != 0 && *outfile != '\n' && *outfile != '\r' 
+	*outfile = '\0';
+	if (fgets(outfile, sizeof outfile, stdin)) {
+	    top_n_tail(outfile, 0, NULL);
+	}
+	if (*outfile != '\0' && *outfile != '\n' && *outfile != '\r' 
 	    && strcmp(outfile, "q")) {
 	    const char *udir = gretl_workdir();
 
@@ -1050,17 +1055,17 @@ static int exec_line (ExecState *s, DATASET *dset)
 	    err = getopenfile(line, runfile, OPT_S);
 	}
 	if (err) { 
-	    pputs(prn, _("Command is malformed\n"));
+	    errmsg(err, prn);
 	    break;
 	}
+	if (gretl_messages_on()) {
+	    pprintf(prn, " %s\n", runfile);
+	}
 	if (cmd->ci == INCLUDE && gretl_is_xml_file(runfile)) {
-	    err = load_user_XML_file(runfile);
+	    err = load_user_XML_file(runfile, prn);
 	    if (err) {
 		pprintf(prn, _("Error reading %s\n"), runfile);
 	    } else {
-		if (gretl_messages_on()) {
-		    pprintf(prn, " %s\n", runfile);
-		}
 		pprintf(cmdprn, "include \"%s\"\n", runfile);
 	    }
 	    break;
@@ -1078,9 +1083,6 @@ static int exec_line (ExecState *s, DATASET *dset)
 	    err = 1;
 	    fb = pop_input_file();
 	} else {
-	    if (gretl_messages_on()) {
-		pprintf(prn, " %s\n", runfile);
-	    }	    
 	    gretl_set_current_dir(runfile);
 	    strcpy(s->runfile, runfile);
 	    if (cmd->ci == INCLUDE) {
