@@ -605,6 +605,7 @@ static void dist_graph (int d, double *parms)
 {
     char pline[128];
     int ids[ID_MAX] = {0};
+    gchar *comment = NULL;
     gchar *title = NULL;
     FILE *fp;
     int alt = 0, err = 0;
@@ -619,15 +620,17 @@ static void dist_graph (int d, double *parms)
     print_keypos_string(GP_KEY_RIGHT_TOP, fp);
     fputs("set parametric\n", fp);
 
+    title = dist_graph_title(d, parms);
+
     gretl_push_c_numeric_locale();
 
     range_from_dist(d, parms, alt, fp);
 
     /* header */
     fprintf(fp, "# literal lines = %d\n", n_literal_lines(d, PLOT_PROB_DIST));
-    title = dist_comment_line(d, parms);
-    fprintf(fp, "%s\n", title);
-    g_free(title);
+    comment = dist_comment_line(d, parms);
+    fprintf(fp, "%s\n", comment);
+    g_free(comment);
 
     /* required variables and formulae */
     switch (d) {
@@ -661,15 +664,14 @@ static void dist_graph (int d, double *parms)
 	fprintf(fp, "%s\n", F_POIS);
 	break;
     case WEIBULL_DIST:
-	fprintf(fp, "shp1=%g\n", parms[0]);
-	fprintf(fp, "scl1=%g\n", parms[1]);
+	fprintf(fp, "shp1=%f\n", parms[0]);
+	fprintf(fp, "scl1=%f\n", parms[1]);
 	fprintf(fp, "%s\n", F_WEIB);
 	break;
     }
 
     fprintf(fp, "plot \\\n");
 
-    title = dist_graph_title(d, parms);
     make_plot_line(pline, d, alt, ids);
 
     if (d == BINOMIAL_DIST || d == POISSON_DIST) {
@@ -807,9 +809,9 @@ static void revise_distribution_plot (png_plot *plot, int d, double *parms)
 	break;
     case WEIBULL_DIST:
 	k = ids[ID_SHP] + 1;
-	sprintf(v1, "shp%d=%g", k, parms[0]);
+	sprintf(v1, "shp%d=%f", k, parms[0]);
 	k = ids[ID_SCL] + 1;
-	sprintf(v2, "scl%d=%g", k, parms[1]);
+	sprintf(v2, "scl%d=%f", k, parms[1]);
 	break;
     }  
 
@@ -3063,6 +3065,23 @@ static void real_stats_calculator (int code, gpointer data)
     window_list_add(child->dlg, STAT_TABLE);
 }
 
+static gchar *maybe_trim_y_equals (gchar *s)
+{
+    gchar *p = s;
+
+    while (isspace(*p)) p++;
+
+    if (*p == 'y' || *p == 'Y') {
+	p++;
+	while (isspace(*p)) p++;
+	if (*p == '=') {
+	    s = p + 1;
+	}
+    }
+
+    return s;
+}
+
 /* for gnuplot: convert '^' to '**' for exponentiation */
 
 static gchar *formula_mod (const gchar *s)
@@ -3104,22 +3123,30 @@ struct curve_plotter {
 
 static void do_plot_curve (GtkWidget *w, struct curve_plotter *p)
 {
-    const gchar *s = gtk_entry_get_text(GTK_ENTRY(p->entry));
+    gchar *s1, *s0 = get_genr_string(p->entry, NULL);
     FILE *fp = NULL;
     int err = 0;
 
-    if (s == NULL || *s == '\0') {
+    if (s0 == NULL || *s0 == '\0') {
 	return;
     }
 
+    /* It's "natural", but not accepted on a gnuplot 'plot'
+       line, to type something like "y = x**2"; here we just
+       want the bit to the right of the equals sign.
+    */
+    s1 = maybe_trim_y_equals(s0);
+
     g_free(p->formula);
 
-    if (strchr(s, '^')) {
-	p->formula = formula_mod(s);
+    if (strchr(s1, '^')) {
+	p->formula = formula_mod(s1);
     } else {
-	p->formula = g_strdup(s);
+	p->formula = g_strdup(s1);
     }
 
+    g_free(s0);
+
     fp = get_plot_input_stream(PLOT_CURVE, &err);
     if (err) { 
 	return;
@@ -3153,7 +3180,7 @@ static void do_plot_curve (GtkWidget *w, struct curve_plotter *p)
 
 /* plot a curve specified via formula (no data required) */
 
-static void plot_curve (void)
+static void plot_a_curve (void)
 {
     static struct curve_plotter plotter;
     GtkWidget *dialog, *hbox, *vbox;
@@ -3387,7 +3414,7 @@ void stats_calculator (GtkAction *action, gpointer data)
 		     code == CALC_PLOT);
 
     if (code == CALC_PLOT) {
-	plot_curve();
+	plot_a_curve();
     } else {
 	real_stats_calculator(code, data);
     }
