@@ -216,7 +216,7 @@ static int expand_data_dialog (int src_pd, int targ_pd, int *interpol,
 
 	resp = radio_dialog("gretl", _("Adding a lower frequency series to a\n"
 				       "higher frequency dataset"),
-			    opts, 2, 0, 0, parent);
+			    opts, 2, 0, EXPAND, parent);
 	if (resp == 0) {
 	    *interpol = 1;
 	    resp = GRETL_YES;
@@ -1871,7 +1871,7 @@ static int get_target_in_home (char *targ, int code,
 			       const char *objname,
 			       const char *ext)
 {
-#ifdef OSX_BUILD
+#ifdef OS_OSX
     const char *savedir = gretl_app_support_dir();
 #else
     const char *savedir = gretl_dotdir();
@@ -1962,7 +1962,7 @@ static char *get_writable_target (int code, char *objname,
 	ext = ".tar.gz";
     }
 
-#ifdef OSX_BUILD
+#ifdef OS_OSX
     /* we prefer writing to ~/Library/Application Support
        rather than /Applications/Gretl.app 
     */
@@ -2010,10 +2010,10 @@ static int unpack_book_data (const char *fname)
 	*p = '\0';
     }
 
-    chdir(path);
-
-    if (errno != 0) {
-        gretl_errmsg_set_from_errno("chdir");
+    if (chdir(path) != 0) {
+	if (errno != 0) {
+	    gretl_errmsg_set_from_errno("chdir");
+	}
 	err = E_FOPEN;
     }
 
@@ -2242,15 +2242,16 @@ real_get_db_description (const char *fullname, const char *binname,
     if (fp != NULL) {
 	char tmp[DB_DESCRIP_LEN + 32] = {0};
 
-	fgets(tmp, sizeof tmp, fp);
-	fclose(fp);
-	if (*tmp == '#' && strlen(tmp) > 2) {
-	    char *s = tmp + 2;
+	if (fgets(tmp, sizeof tmp, fp) != NULL) {
+	    if (*tmp == '#' && strlen(tmp) > 2) {
+		char *s = tmp + 2;
 
-	    tailstrip(s);
-	    utf8_correct(s);
-	    descrip = g_strdup(s);
+		tailstrip(s);
+		utf8_correct(s);
+		descrip = g_strdup(s);
+	    }
 	}
+	fclose(fp);
     }
 
     return descrip;
@@ -2957,7 +2958,7 @@ gint populate_remote_data_pkg_list (windata_t *vwin)
     return err;
 }
 
-gint populate_dbfilelist (windata_t *vwin)
+gint populate_dbfilelist (windata_t *vwin, int *pndb)
 {
     GtkListStore *store;
     GtkTreeIter iter;
@@ -2989,6 +2990,10 @@ gint populate_dbfilelist (windata_t *vwin)
 	presort_treelist(vwin);
     }
 
+    if (pndb != NULL) {
+	*pndb = ndb;
+    }
+
     return err;
 }
 
