@@ -15,7 +15,7 @@ void pkg_info_from_server (GtkWidget *w, windata_t *vwin);
 
 int unzip_package_file (const char *zipname, const char *path);
 
-gint populate_dbfilelist (windata_t *vwin);
+gint populate_dbfilelist (windata_t *vwin, int *pndb);
 
 void set_db_dir_callback (windata_t *vwin, char *path);
 
