@@ -188,7 +188,7 @@ static int get_title_from_descfile (file_collection *coll)
     fp = gretl_fopen(test, "r");
 
     if (fp != NULL && fgets(line, sizeof line, fp) != NULL) {
-	gretl_chopstr(line);
+	gretl_strstrip(line);
 	if (sscanf(line, "# %23[^:]", title) == 1) {
 	    coll->title = gretl_strdup(title);
 	    if (coll->title == NULL) {
@@ -527,7 +527,7 @@ static int build_file_collections (void)
 
 	n += seek_file_collections(gretl_home(), DATA_SEARCH, &err);
 	n += seek_file_collections(gretl_home(), SCRIPT_SEARCH, &err);
-#ifdef OSX_BUILD
+#ifdef OS_OSX
 	n += seek_file_collections(gretl_app_support_dir(), DATA_SEARCH, &err);
 	n += seek_file_collections(gretl_app_support_dir(), SCRIPT_SEARCH, &err);
 #else
@@ -755,7 +755,7 @@ static gint enter_opens_file (GtkWidget *w, GdkEventKey *key,
 
 static int gui_load_user_functions (const char *fname)
 {
-    int err = load_function_package_by_filename(fname);
+    int err = load_function_package_by_filename(fname, NULL);
 
     if (err) {
 	gui_errmsg(err);
@@ -1255,9 +1255,7 @@ static void build_funcfiles_popup (windata_t *vwin)
 static gboolean 
 funcfiles_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer data)
 {
-    GdkModifierType mods = widget_get_pointer_mask(w);
-
-    if (RIGHT_CLICK(mods)) {
+    if (right_click(event)) {
 	windata_t *vwin = (windata_t *) data;
 
 	if (vwin->popup != NULL) {
@@ -1712,7 +1710,7 @@ void display_files (int role, gpointer data)
 	err = populate_notebook_filelists(vwin, filebox, role);
     } else if (role == FUNC_FILES) {
 	err = populate_filelist(vwin, &fresp);
-	if (fresp.col1_width > 15) {
+	if (!err && fresp.col1_width > 15) {
 	    /* widen the file box if need be */
 	    gint w, h;
 
@@ -1720,6 +1718,15 @@ void display_files (int role, gpointer data)
 	    w += 100;
 	    gtk_widget_set_size_request(filebox, w, h);
 	}
+    } else if (role == NATIVE_DB) {
+	gint w, h, ndb = 0;
+
+	err = populate_filelist(vwin, &ndb);
+	if (!err && ndb > 12) {
+	    gtk_widget_get_size_request(filebox, &w, &h);
+	    h += 100;
+	    gtk_widget_set_size_request(filebox, w, h);
+	}	    
     } else {
 	err = populate_filelist(vwin, NULL);
     }
@@ -2100,7 +2107,7 @@ void maybe_update_func_files_window (int role)
 gint populate_filelist (windata_t *vwin, gpointer p)
 {
     if (vwin->role == NATIVE_DB) {
-	return populate_dbfilelist(vwin);
+	return populate_dbfilelist(vwin, p);
     } else if (vwin->role == REMOTE_DB) {
 	return populate_remote_db_list(vwin);
     } else if (vwin->role == REMOTE_FUNC_FILES) {
