@@ -24,6 +24,7 @@
 #include "filters.h"
 #include "libset.h"
 #include "plotspec.h"
+#include "uservar.h"
 #include "usermat.h"
 #include "cmdstack.h"
 #include "ssheet.h"
@@ -250,7 +251,7 @@ static void filter_make_varlabel (filter_info *finfo, int v, int i)
 	sprintf(s, "fracdiff(%s, %g)", finfo->vname, finfo->lambda);
     }
 
-    strcpy(VARLABEL(dataset, v), s);
+    series_set_label(dataset, v, s);
 }
 
 static void check_bk_limits1 (GtkWidget *s1, GtkWidget *s2)
@@ -1143,11 +1144,11 @@ static int save_filtered_var (filter_info *finfo, double *x, int i,
     int err = 0;
 
     if (v == dataset->v) {
-	err = dataset_add_allocated_series(x, dataset);
+	err = dataset_add_allocated_series(dataset, x);
     } else {
 	free(dataset->Z[v]);
 	dataset->Z[v] = x;
-	set_var_discrete(dataset, v, 0);
+	series_set_discrete(dataset, v, 0);
     }
 
     if (!err) {
