@@ -58,4 +58,10 @@ int download_addon (const char *pkgname, char **local_path);
 char *installed_addon_status_string (const char *path,
 				     const char *svstr);
 
+int revise_package_status (const gchar *pkgname,
+			   const gchar *label,
+			   const gchar *mpath,
+			   int uses_subdir,
+			   int maybe_edit);
+
 #endif /* FNCALL_H */
