@@ -27,6 +27,8 @@
 #include "winstack.h"
 #include "selector.h"
 #include "textutil.h"
+#include "gretl_bundle.h"
+#include "fncall.h"
 #include "fnsave.h"
 
 #include <libxml/xmlmemory.h>
@@ -41,6 +43,12 @@
 #define PKG_DEBUG 0
 #define NENTRIES 4
 
+enum {
+    NO_WINDOW,
+    MAIN_WINDOW,
+    MODEL_WINDOW
+};
+
 typedef struct function_info_ function_info;
 typedef struct login_info_ login_info;
 
@@ -55,6 +63,12 @@ struct function_info_ {
     GtkWidget *saveas;     /* "Save as" button in dialog */
     GtkWidget *validate;   /* Validate button */
     GtkWidget *popup;      /* popup menu */
+    GtkWidget *extra;      /* extra properties child dialog */
+    GtkWidget *maintree;   /* extra properties child dialog */
+    GtkWidget *modeltree;  /* extra properties child dialog */
+    GtkWidget *currtree;   /* currently displayed menu treeview */
+    GtkWidget *alttree;    /* currently undisplayed menu treeview */
+    GtkWidget *treewin;    /* scrolled window to hold menu trees */
     windata_t *samplewin;  /* window for editing sample script */
     GList *codewins;       /* list of windows editing function code */
     fnpkg *pkg;            /* pointer to package being edited */
@@ -67,8 +81,14 @@ struct function_info_ {
     gchar *help;           /* package help text */
     char **pubnames;       /* names of public functions */
     char **privnames;      /* names of private functions */
+    char **specials;       /* names of special functions */
     int n_pub;             /* number of public functions */
     int n_priv;            /* number of private functions */
+    int n_special;         /* (max) number of special functions */
+    gboolean uses_subdir;  /* the package has its own subdir (0/1) */
+    gchar *menupath;       /* path for menu attachment, if any */
+    gchar *menulabel;      /* label for menu attachment, if any */
+    int menuwin;           /* code for none/main/model window */
     char *active;          /* name of 'active' function */
     FuncDataReq dreq;      /* data requirement of package */
     int minver;            /* minimum gretl version, package */
@@ -90,15 +110,26 @@ struct login_info_ {
 static int validate_package_file (const char *fname,
 				  int verbose);
 
+static void finfo_set_menuwin (function_info *finfo);
+
 function_info *finfo_new (void)
 {
     function_info *finfo;
+    int ns = UFUN_ROLE_MAX - 1;
 
     finfo = mymalloc(sizeof *finfo);
     if (finfo == NULL) {
 	return NULL;
     }
 
+    finfo->specials = strings_array_new(ns);
+    if (finfo->specials == NULL) {
+	free(finfo);
+	return NULL;
+    }
+
+    finfo->n_special = ns;
+
     finfo->pkg = NULL;
     finfo->fname = NULL;
     finfo->author = NULL;
@@ -106,6 +137,13 @@ function_info *finfo_new (void)
     finfo->date = NULL;
     finfo->pkgdesc = NULL;
     finfo->sample = NULL;
+    finfo->menupath = NULL;
+    finfo->menulabel = NULL;
+    finfo->menuwin = 0;
+
+    finfo->currtree = NULL;
+    finfo->alttree = NULL;
+    finfo->treewin = NULL;
 
     finfo->upload = FALSE;
     finfo->modified = FALSE;
@@ -115,6 +153,7 @@ function_info *finfo_new (void)
     finfo->codewins = NULL;
     finfo->codesel = NULL;
     finfo->popup = NULL;
+    finfo->extra = NULL;
 
     finfo->help = NULL;
     finfo->pubnames = NULL;
@@ -125,6 +164,7 @@ function_info *finfo_new (void)
 
     finfo->dreq = 0;
     finfo->minver = 10804;
+    finfo->uses_subdir = 0;
 
     return finfo;
 }
@@ -159,6 +199,8 @@ static void finfo_free (function_info *finfo)
     g_free(finfo->pkgdesc);
     g_free(finfo->sample);
     g_free(finfo->help);
+    g_free(finfo->menupath);
+    g_free(finfo->menulabel);
 
     if (finfo->pubnames != NULL) {
 	strings_array_free(finfo->pubnames, finfo->n_pub);
@@ -168,6 +210,10 @@ static void finfo_free (function_info *finfo)
 	strings_array_free(finfo->privnames, finfo->n_priv);
     }
 
+    if (finfo->specials != NULL) {
+	strings_array_free(finfo->specials, finfo->n_special);
+    }    
+
     if (finfo->samplewin != NULL) {
 	gtk_widget_destroy(finfo->samplewin->main);
     }
@@ -1040,6 +1086,589 @@ static void toggle_upload (GtkToggleButton *b, function_info *finfo)
     finfo->upload = gtk_toggle_button_get_active(b);
 }
 
+static gchar *get_model_menu_string (const char *s)
+{
+    gchar *ret;
+
+    if (!strcmp(s, "Analysis")) {
+	ret = g_strdup(_("_Analysis"));
+	gretl_delchar('_', ret);
+    } else {
+	ret = g_strdup(_(s));
+    }
+
+    return ret;
+}
+
+static GtkTreeStore *make_menu_attachment_tree (function_info *finfo,
+						GtkTreePath **ppath,
+						int modelwin)
+{
+    const char *main_items = 
+	"0 Tools\n"
+	"0 Data\n"
+	"0 View\n"
+	"1 GraphVars\n"
+	"2 MultiPlots\n"
+	"0 Add\n"
+	"0 Sample\n"
+	"0 Variable\n"
+	"1 URTests\n"
+	"1 Filter\n"
+	"0 Model\n"
+	"1 ivreg\n"  
+	"1 LinearModels\n"
+	"1 LimdepModels\n"
+	"2 logit\n"
+	"2 probit\n"
+	"1 TSModels\n"
+	"1 PanelModels\n"
+	"1 RobustModels\n";
+    const char *model_items = 
+	"0 Tests\n"
+	"0 Save\n"
+	"0 Graphs\n"
+	"0 Analysis\n";
+    const char *leaders[] = {
+	"MAINWIN",
+	"MODELWIN"
+    };
+    const char *leader;
+    GtkTreeStore *store;
+    GtkTreeIter iter;
+    GtkTreeIter parents[2];
+    GtkTreeIter *iterp;
+    gchar *path, *ustr;
+    const char *s;
+    char words[3][16];
+    char *word;
+    int level;
+
+    store = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+
+    leader = modelwin ? leaders[1] : leaders[0];
+    s = modelwin ? model_items : main_items;
+
+    while (*s) {
+	level = atoi(s);
+	word = words[level];
+	s += 2;
+	sscanf(s, "%s", word);
+	if (modelwin) {
+	    ustr = get_model_menu_string(word);
+	} else {
+	    ustr = get_user_menu_string(word);
+	}
+	if (level == 0) {
+	    path = g_strdup_printf("%s/%s", leader, words[0]);
+	    gtk_tree_store_append(store, &parents[0], NULL);
+	    iterp = &parents[0];
+	} else if (level == 1) {
+	    path = g_strdup_printf("%s/%s/%s", leader, words[0], 
+				   words[1]);
+	    gtk_tree_store_append(store, &parents[1], &parents[0]);
+	    iterp = &parents[1];
+	} else {
+	    path = g_strdup_printf("%s/%s/%s/%s", leader, words[0], 
+				   words[1], words[2]); 
+	    gtk_tree_store_append(store, &iter, &parents[1]);
+	    iterp = &iter;
+	}
+	gtk_tree_store_set(store, iterp, 0, ustr, 1, path, -1);
+	if (finfo->menupath != NULL && !strcmp(path, finfo->menupath)) {
+	    /* record the path of pre-selected menu entry */
+	    *ppath = gtk_tree_model_get_path(GTK_TREE_MODEL(store), iterp);
+	}	
+	g_free(path);
+	g_free(ustr);
+	s += strlen(word) + 1;
+    }
+
+    return store;
+}
+
+static GtkWidget *add_menu_navigator (GtkWidget *holder, 
+				      function_info *finfo,
+				      int modelwin)
+{
+    GtkTreeStore *store;
+    GtkWidget *view;
+    GtkCellRenderer *renderer;
+    GtkTreeViewColumn *column;
+    GtkTreeSelection *select;
+    GtkTreePath *path = NULL;
+
+    store = make_menu_attachment_tree(finfo, &path, modelwin);
+    if (store == NULL) {
+	return NULL;
+    }
+
+    view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
+    g_object_set(view, "enable-tree-lines", TRUE, NULL);
+    g_object_ref(G_OBJECT(view));
+
+    renderer = gtk_cell_renderer_text_new();
+    column = gtk_tree_view_column_new_with_attributes("",
+						      renderer,
+						      "text", 0,
+						      NULL);
+    gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
+
+    select = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
+    gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
+
+    if (path != NULL) {
+	gtk_tree_view_expand_to_path(GTK_TREE_VIEW(view), path);
+	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(view), path,
+				     NULL, FALSE, 0, 0);
+	gtk_tree_selection_select_path(select, path);
+	gtk_tree_path_free(path);
+    }
+
+    if (finfo->treewin == NULL) {
+	GtkWidget *sw;
+
+	sw = finfo->treewin = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
+				       GTK_POLICY_AUTOMATIC, 
+				       GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
+					    GTK_SHADOW_IN);
+	gtk_container_add(GTK_CONTAINER(holder), sw);
+	gtk_widget_set_size_request(sw, 150, -1);
+    }
+
+    if ((modelwin == 0 && finfo->menuwin != MODEL_WINDOW) ||
+	(modelwin == 1 && finfo->menuwin == MODEL_WINDOW)) {
+	gtk_container_add(GTK_CONTAINER(finfo->treewin), view);
+	finfo->currtree = view;
+    } else {
+	finfo->alttree = view;
+    }
+
+    gtk_tree_view_columns_autosize(GTK_TREE_VIEW(view));
+
+    return view;
+}
+
+static void switch_menu_view (GtkComboBox *combo,
+			      function_info *finfo) 
+{
+    int w = gtk_combo_box_get_active(combo);
+    GtkWidget *sw = finfo->treewin;
+
+    if (w == NO_WINDOW) {
+	if (finfo->currtree != NULL) {
+	    gtk_widget_set_sensitive(finfo->currtree, FALSE);
+	}
+    } else if (w == MAIN_WINDOW) {
+	if (finfo->currtree == finfo->modeltree) {
+	    gtk_container_remove(GTK_CONTAINER(sw), finfo->modeltree);
+	    finfo->alttree = finfo->modeltree;
+	    gtk_widget_show(finfo->maintree);
+	    gtk_container_add(GTK_CONTAINER(sw), finfo->maintree);
+	    finfo->currtree = finfo->maintree;
+	}
+	gtk_widget_set_sensitive(finfo->currtree, TRUE);
+    } else if (w == MODEL_WINDOW) {
+	if (finfo->currtree == finfo->maintree) {
+	    gtk_container_remove(GTK_CONTAINER(sw), finfo->maintree);
+	    finfo->alttree = finfo->maintree;
+	    gtk_widget_show(finfo->modeltree);
+	    gtk_container_add(GTK_CONTAINER(sw), finfo->modeltree);
+	    finfo->currtree = finfo->modeltree;
+	}
+	gtk_widget_set_sensitive(finfo->currtree, TRUE);
+    }	
+}
+
+static void add_menu_attach_top (GtkWidget *holder,
+				 function_info *finfo)
+{
+    GtkWidget *w, *hbox, *entry;
+    GtkWidget *combo;
+
+    /* menu label entry */
+    hbox = gtk_hbox_new(FALSE, 5);
+    w = gtk_label_new(_("Label"));
+    gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
+    entry = gtk_entry_new();
+    gtk_entry_set_max_length(GTK_ENTRY(entry), 36);
+    gtk_entry_set_width_chars(GTK_ENTRY(entry), 32);
+    if (finfo->menulabel != NULL) {
+	gtk_entry_set_text(GTK_ENTRY(entry), finfo->menulabel);
+    }
+    gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 5);
+    g_object_set_data(G_OBJECT(finfo->extra), "label-entry", entry);
+    gtk_box_pack_start(GTK_BOX(holder), hbox, FALSE, FALSE, 5);
+
+    /* menu attachment combo */
+    hbox = gtk_hbox_new(FALSE, 5);
+    w = gtk_label_new(_("Window"));
+    gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
+    combo = gtk_combo_box_text_new();
+    combo_box_append_text(combo, _("none"));    
+    combo_box_append_text(combo, _("main window"));    
+    combo_box_append_text(combo, _("model window"));    
+    gtk_box_pack_start(GTK_BOX(hbox), combo, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(holder), hbox, FALSE, FALSE, 5);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combo), finfo->menuwin);
+    g_signal_connect(G_OBJECT(combo), "changed",
+		     G_CALLBACK(switch_menu_view), finfo);
+}
+
+static int process_menu_attachment (function_info *finfo)
+{
+    GtkTreeSelection *selection;
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    GtkWidget *view, *entry;
+    gchar *label;
+    int changed = 0;
+
+    view = finfo->currtree;
+
+    if (!gtk_widget_is_sensitive(view)) {
+	/* no menu attachment at present */
+	if (finfo->menupath != NULL) {
+	    g_free(finfo->menupath);
+	    finfo->menupath = NULL;
+	    changed = 1;
+	}
+	finfo->menuwin = NO_WINDOW;
+	goto finish;
+    }
+
+    entry = g_object_get_data(G_OBJECT(finfo->extra), "label-entry");
+    label = entry_box_get_trimmed_text(entry);
+
+    if (label == NULL || *label == '\0') {
+	if (finfo->menulabel != NULL) {
+	    g_free(finfo->menulabel);
+	    finfo->menulabel = NULL;
+	    changed = 1;
+	}
+    } else if (finfo->menulabel == NULL || strcmp(finfo->menulabel, label)) {
+	g_free(finfo->menulabel);
+	finfo->menulabel = label;
+	label = NULL;
+	changed = 1;
+    }
+
+    g_free(label);
+
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
+
+    if (!gtk_tree_selection_get_selected(selection, &model, &iter)) {
+	if (finfo->menupath != NULL) {
+	    g_free(finfo->menupath);
+	    finfo->menupath = NULL;
+	    changed = 1;
+	}
+    } else {
+	gchar *newpath = NULL;
+
+	gtk_tree_model_get(model, &iter, 1, &newpath, -1);
+	if (finfo->menupath == NULL || strcmp(finfo->menupath, newpath)) {
+	    g_free(finfo->menupath);
+	    finfo->menupath = newpath;
+	    newpath = NULL;
+	    changed = 1;
+	}
+	g_free(newpath);
+    }
+
+    finfo_set_menuwin(finfo);
+
+ finish:
+
+    if (changed) {
+	infobox(_("To update the menu attachment, you should\n"
+		  "(a) save this package, and (b) restart gretl."));
+    }
+
+    return changed;
+}
+
+static int process_special_functions (function_info *finfo)
+{
+    GtkWidget **c_array;
+    const char *oldfun;
+    gchar *newfun;
+    int n_changed = 0;
+    int i, err = 0;
+
+    c_array = g_object_get_data(G_OBJECT(finfo->extra), "combo-array");
+
+    /* For each special function slot, check to see if the 
+       currently selected function differs from what was
+       present originally, and if so update the record in
+       @finfo->specials. The changes are not yet saved to
+       the function package itself.
+    */
+
+    for (i=0; i<finfo->n_special && !err; i++) {
+	if (gtk_widget_is_sensitive(c_array[i])) {
+	    int newnull = 0, oldnull = 0, changed = 0;
+
+	    /* retrieve and check the selected name */
+	    newfun = combo_box_get_active_text(GTK_COMBO_BOX(c_array[i]));
+	    newnull = (newfun == NULL || *newfun == '\0' || 
+		       !strcmp(newfun, "none"));
+
+	    /* retrieve and check what was there before */
+	    oldfun = finfo->specials[i];
+	    oldnull = (oldfun == NULL || *oldfun == '\0' || 
+		       !strcmp(oldfun, "none"));
+
+	    if (oldnull && !newnull) {
+		changed = 1;
+	    } else if (!oldnull && newnull) {
+		changed = 1;
+	    } else if (!oldnull && !newnull) {
+		changed = strcmp(newfun, oldfun);
+	    } 
+	    if (changed) {
+		free(finfo->specials[i]);
+		finfo->specials[i] = gretl_strdup(newfun);
+		n_changed++;
+	    }
+	    g_free(newfun);
+	}
+    }
+
+    return n_changed;
+}
+
+static void extra_properties_callback (GtkWidget *w, function_info *finfo)
+{
+    GtkWidget *notebook;
+    gint page;
+    int changed;
+
+    notebook = g_object_get_data(G_OBJECT(finfo->extra), "book");
+    page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
+    
+    if (page == 0) {
+	changed = process_special_functions(finfo);
+    } else {
+	changed = process_menu_attachment(finfo);
+    }
+
+    if (changed) {
+	finfo_set_modified(finfo, TRUE);
+    }    
+}
+
+/* Prevent the user from assigning a given function to more
+   then one special role: when a selection is changed, if
+   the given function is already selected for a different
+   role, deselect it in that role.
+*/
+
+static void special_changed_callback (GtkComboBox *this, 
+				      function_info *finfo)
+{
+    GtkWidget **c_array;
+    GtkWidget *other;
+    gchar *s0, *si;
+    int i, dup = 0;
+
+    s0 = combo_box_get_active_text(this);
+    c_array = g_object_get_data(G_OBJECT(finfo->extra), "combo-array");
+
+    for (i=0; i<finfo->n_special && !dup; i++) {
+	other = c_array[i];
+	if (other != GTK_WIDGET(this)) {
+	    si = combo_box_get_active_text(GTK_COMBO_BOX(other));
+	    if (!strcmp(si, s0)) {
+		/* switch to "none" */
+		gtk_combo_box_set_active(GTK_COMBO_BOX(other), 0);
+		dup = 1;
+	    }
+	    g_free(si);
+	}
+    }
+
+    g_free(s0);
+}
+
+static void finfo_extra_help (GtkWidget *w, function_info *finfo)
+{
+    GtkWidget *notebook;
+    gint page;
+
+    notebook = g_object_get_data(G_OBJECT(finfo->extra), "book");
+    page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
+    
+    if (page == 0) {
+	show_gui_help(GUI_FUNCS);
+    } else {
+	show_gui_help(MENU_ATTACH);
+    }
+}
+
+static void unref_trees (GtkWidget *w, function_info *finfo)
+{
+    g_object_unref(G_OBJECT(finfo->maintree));
+    g_object_unref(G_OBJECT(finfo->modeltree));
+}
+
+#define must_be_private(r) (r == UFUN_GUI_PRECHECK)
+#define must_be_public(r) (r != UFUN_GUI_PRECHECK)
+
+/* The following function supports two "notebook" tabs: one allows the
+   user to select functions in the package for the various "special"
+   package roles (e.g. gui-main, bundle-print). The other allows for
+   selection of a menu attachment point and GUI label.
+*/
+
+static void extra_properties_dialog (GtkWidget *w, function_info *finfo)
+{
+    GtkWidget *dlg, *combo, *table;
+    GtkWidget *tmp, *vbox, *hbox;
+    GtkWidget **combo_array;
+    GtkWidget *notebook;
+    const char *funname;
+    const char *key;
+    const char *special;
+    int nfuns, i, j;
+
+    if (finfo->extra != NULL) {
+	gtk_window_present(GTK_WINDOW(finfo->extra));
+	return;
+    }
+
+    finfo->maintree = finfo->modeltree = NULL;
+    finfo->currtree = finfo->alttree = NULL;
+    finfo->treewin = NULL;
+
+    dlg = gretl_dialog_new(_("gretl: extra properties"), finfo->dlg,
+			   GRETL_DLG_BLOCK | GRETL_DLG_RESIZE);
+    finfo->extra = dlg;
+    g_signal_connect(G_OBJECT(dlg), "destroy",
+		     G_CALLBACK(gtk_widget_destroyed), &finfo->extra);
+    g_signal_connect(G_OBJECT(dlg), "destroy",
+		     G_CALLBACK(unref_trees), finfo);
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
+
+    notebook = gtk_notebook_new();
+    gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
+    g_object_set_data(G_OBJECT(dlg), "book", notebook);
+
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
+    tmp = gtk_label_new(_("Special functions"));
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, tmp);  
+
+    table = gtk_table_new(finfo->n_special, 2, TRUE);
+    gtk_table_set_row_spacings(GTK_TABLE(table), 5);
+    gtk_table_set_col_spacings(GTK_TABLE(table), 5);
+
+    nfuns = finfo->n_priv + finfo->n_pub;
+    combo_array = g_malloc(finfo->n_special * sizeof *combo_array);
+    g_object_set_data_full(G_OBJECT(dlg), "combo-array", 
+			   combo_array, g_free);
+
+    /* For each "special" function role, test the functions
+       in finfo->pkg to see if they qualify as candidates for
+       that role; if so, add them to the combo selector.
+    */
+
+    for (i=0; i<finfo->n_special; i++) {
+	int err, n_cands = 0;
+	int selected = 0;
+	int role = i + 1;
+
+	key = package_role_get_key(role);
+	special = finfo->specials[i];
+	combo = gtk_combo_box_text_new();
+	combo_box_append_text(combo, "none");
+
+	for (j=0; j<nfuns; j++) {
+	    if (j < finfo->n_priv) {
+		if (must_be_public(role)) {
+		    continue;
+		}
+		funname = finfo->privnames[j];
+	    } else {
+		if (must_be_private(role)) {
+		    continue;
+		}		
+		funname = finfo->pubnames[j - finfo->n_priv];
+	    }
+	    /* test for validity in this role */
+	    err = function_set_package_role(funname, finfo->pkg,
+					    key, OPT_T, NULL);
+	    if (!err) {
+		combo_box_append_text(combo, funname);
+		if (special != NULL && !selected && !strcmp(special, funname)) {
+		    selected = n_cands + 1;
+		}
+		n_cands++;
+	    }
+	}
+
+	tmp = gtk_label_new(key);
+	gtk_misc_set_alignment(GTK_MISC(tmp), 1, 0.5);
+	gtk_table_attach_defaults(GTK_TABLE(table), tmp, 0, 1, i, i+1);
+	gtk_table_attach_defaults(GTK_TABLE(table), combo, 1, 2, i, i+1);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), selected);
+	if (n_cands == 0) {
+	    gtk_widget_set_sensitive(combo, FALSE);
+	} else {
+	    g_signal_connect(G_OBJECT(combo), "changed", 
+			     G_CALLBACK(special_changed_callback), 
+			     finfo);
+	}
+	combo_array[i] = combo;
+    }
+
+    hbox = gtk_hbox_new(FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, FALSE, 5);
+
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
+    tmp = gtk_label_new(_("Menu attachment"));
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, tmp);
+
+    add_menu_attach_top(vbox, finfo);
+
+    finfo->maintree = add_menu_navigator(vbox, finfo, 0);
+    finfo->modeltree = add_menu_navigator(vbox, finfo, 1);
+
+    if (finfo->menuwin == NO_WINDOW) {
+	gtk_widget_set_sensitive(finfo->currtree, FALSE);
+    }
+
+    hbox = gtk_dialog_get_action_area(GTK_DIALOG(dlg));
+
+    /* Apply button */
+    tmp = apply_button(hbox);
+    g_signal_connect(G_OBJECT(tmp), "clicked", 
+		     G_CALLBACK(extra_properties_callback), finfo);
+    gtk_widget_grab_default(tmp);
+    gtk_widget_show(tmp);
+
+    /* Close button */
+    tmp = close_button(hbox);
+    g_signal_connect(G_OBJECT(tmp), "clicked", 
+                     G_CALLBACK(delete_widget), dlg);
+
+    /* Help button */
+    tmp = gtk_button_new_from_stock(GTK_STOCK_HELP);
+    gtk_container_add(GTK_CONTAINER(hbox), tmp);
+    gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(hbox),
+				       tmp, TRUE);
+    g_signal_connect(G_OBJECT(tmp), "clicked", 
+		     G_CALLBACK(finfo_extra_help),
+		     finfo);
+
+    gtk_widget_show_all(dlg);
+}
+
 /* Dialog for editing function package.  The user can get here in
    either of two ways: after selecting functions to put into a
    newly created package, or upon selecting an existing package
@@ -1186,6 +1815,14 @@ static void finfo_dialog (function_info *finfo)
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(add_remove_callback), finfo);
 
+#if 0 /* work in progress */
+    /* extra package properties button */
+    button = gtk_button_new_with_label(_("Extra properties"));
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 5);
+    g_signal_connect(G_OBJECT(button), "clicked", 
+		     G_CALLBACK(extra_properties_dialog), finfo);
+#endif
+
     /* write spec file button */
     button = gtk_button_new_with_label(_("Write spec file"));
     gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 5);
@@ -1512,6 +2149,23 @@ static int dont_overwrite_pkg (const char *fname)
     return ret;
 }
 
+static int pkg_save_special_functions (function_info *finfo)
+{
+    const char *key;
+    int i, err = 0;
+
+    for (i=0; i<finfo->n_special && !err; i++) {
+	key = package_role_get_key(i+1);
+	err = function_set_package_role(finfo->specials[i], 
+					finfo->pkg,
+					key,
+					OPT_NONE,
+					NULL);
+    }
+
+    return err;
+}
+
 /* callback from file selector when saving a function package, or
    directly from the package editor if using the package's
    existing filename */
@@ -1567,12 +2221,18 @@ int save_function_package (const char *fname, gpointer p)
 					      "sample-script", finfo->sample,
 					      "data-requirement", finfo->dreq,
 					      "min-version", finfo->minver,
+					      "menu-attachment", finfo->menupath,
+					      "label", finfo->menulabel,
 					      NULL);
 	if (err) {
 	    fprintf(stderr, "function_package_set_properties: err = %d\n", err);
 	}
     }
 
+    if (!err) {
+	pkg_save_special_functions(finfo);
+    }
+
     if (!err) {
 	err = function_package_write_file(finfo->pkg);
 	if (err) {
@@ -1583,10 +2243,10 @@ int save_function_package (const char *fname, gpointer p)
     if (err) {
 	gui_errmsg(err);
     } else {
-	const char *pname = function_package_get_name(finfo->pkg);
+	const char *pkgname = function_package_get_name(finfo->pkg);
 	gchar *title;
 
-	title = g_strdup_printf("gretl: %s", pname);
+	title = g_strdup_printf("gretl: %s", pkgname);
 	gtk_window_set_title(GTK_WINDOW(finfo->dlg), title);
 	g_free(title);
 	finfo_set_modified(finfo, FALSE);
@@ -1596,6 +2256,13 @@ int save_function_package (const char *fname, gpointer p)
 	if (finfo->upload) {
 	    do_pkg_upload(finfo, fname);
 	}
+	if (1 || finfo->menuwin != NO_WINDOW) {
+	    revise_package_status(pkgname, 
+				  finfo->menulabel,
+				  finfo->menupath,
+				  finfo->uses_subdir,
+				  1);
+	}
     }
 
     return err;
@@ -1666,6 +2333,20 @@ static void maybe_print (PRN *prn, const char *key,
 
 int save_function_package_spec (const char *fname, gpointer p)
 {
+    const char *extra_keys[] = {
+	GUI_MAIN,
+	"label",
+	"menu-attachment",
+	BUNDLE_PRINT,
+	BUNDLE_PLOT,
+	BUNDLE_TEST,
+	BUNDLE_FCAST,
+	BUNDLE_EXTRA,
+	GUI_PRECHECK,
+	NULL
+    };
+    gchar *strval;
+    int intval;
     function_info *finfo = p;
     PRN *prn;
     const char *reqstr = NULL;
@@ -1704,6 +2385,27 @@ int save_function_package_spec (const char *fname, gpointer p)
 	pprintf(prn, "data-requirement = %s\n", reqstr);
     }
 
+    for (i=0; extra_keys[i] != NULL; i++) {
+	function_package_get_properties(finfo->pkg, extra_keys[i],
+					&strval, NULL);
+	if (strval != NULL) {
+	    if (*strval != '\0') {
+		pprintf(prn, "%s = %s\n", extra_keys[i], strval);
+	    }
+	    g_free(strval);
+	    strval = NULL;
+	}
+    }
+
+    function_package_get_properties(finfo->pkg, "model-requirement",
+				    &intval, NULL);
+    if (intval > 0) {
+	reqstr = gretl_command_word(intval);
+	if (*reqstr != '\0') {
+	    pprintf(prn, "model-requirement = %s\n", reqstr);
+	}
+    }
+
     /* public interface names */
     pputs(prn, "public = ");
     len = 9;
@@ -1834,6 +2536,34 @@ void revise_function_package (void *p)
     free(privlist);
 }
 
+static void finfo_set_menuwin (function_info *finfo)
+{
+    if (finfo->menupath == NULL) {
+	finfo->menuwin = NO_WINDOW;
+    } else if (!strncmp(finfo->menupath, "MAINWIN", 7)) {
+	finfo->menuwin = MAIN_WINDOW;
+    } else if (!strncmp(finfo->menupath, "MODELWIN", 8)) {
+	finfo->menuwin = MODEL_WINDOW;
+    } else {
+	finfo->menuwin = NO_WINDOW;
+    }
+}
+
+static int finfo_set_special_names (function_info *finfo)
+{
+    const char *key;
+    int i, err = 0;
+
+    for (i=0; i<finfo->n_special && !err; i++) {
+	key = package_role_get_key(i+1);
+	err = function_package_get_properties(finfo->pkg, key,
+					      &finfo->specials[i],
+					      NULL);
+    }
+
+    return err;
+}
+
 void edit_function_package (const char *fname)
 {
     function_info *finfo;
@@ -1867,12 +2597,23 @@ void edit_function_package (const char *fname)
 					  "sample-script", &finfo->sample,
 					  "data-requirement", &finfo->dreq,
 					  "min-version", &finfo->minver,
+					  "menu-attachment", &finfo->menupath,
+					  "label", &finfo->menulabel,
+					  "lives-in-subdir", &finfo->uses_subdir,
 					  NULL);
 
     if (!err && publist != NULL) {
 	err = finfo_set_function_names(finfo, publist, privlist);
     }
 
+    if (!err) {
+	err = finfo_set_special_names(finfo);
+    }
+
+    if (!err) {
+	finfo_set_menuwin(finfo);
+    }
+
 #if PKG_DEBUG
     printlist(publist, "publist");
     printlist(privlist, "privlist");
