@@ -73,9 +73,11 @@ void filter_gnuplot_file (int ttype, int latin, int mono,
 
 void run_gp_script (gchar *buf);
 
-void launch_gnuplot_interactive (void);
+void launch_gnuplot_interactive (const char *plotfile);
 
 int dump_plot_buffer (const char *buf, const char *fname,
 		      int addpause);
 
+int get_graph_scale (int i, double *s);
+
 #endif /* GPT_CONTROL_H */
