@@ -417,10 +417,6 @@ static int gp_make_outfile (const char *gfname, int i, double scale)
 
     if (gpage.term == GP_TERM_EPS || pdfterm != GP_PDF_CAIRO) {
 	latin = iso_latin_version();
-	if (latin == 9 && !gnuplot_has_latin5()) {
-	    /* what we gonna do? */
-	    latin = 1;
-	}
 	fprintf(fq, "set encoding iso_8859_%d\n", latin);
     }
 
@@ -623,7 +619,7 @@ static int real_display_gpage (void)
 
 #if defined(G_OS_WIN32)
     err = win32_open_file(fname);
-#elif defined(OSX_BUILD)
+#elif defined(OS_OSX)
     err = osx_open_file(fname);
 #else
     viewer = (gpage.term == GP_TERM_PDF)? "viewpdf" : "viewps";
