@@ -48,6 +48,7 @@
 #include "selector.h"
 #include "guiprint.h"
 #include "tabwin.h"
+#include "gpt_control.h"
 
 #include <dirent.h>
 
@@ -79,6 +80,7 @@ static gboolean main_popup_handler (GtkWidget *w, GdkEventButton *event,
 				    gpointer data);
 static GtkWidget *make_main_menu (void);
 static void start_R_callback (void);
+static void start_gnuplot_callback (void);
 static void auto_store (void);
 static void restore_sample_callback (void);
 static void show_sample_callback (void);
@@ -165,7 +167,7 @@ char calculator[MAXSTR] = "calc.exe";
 char latex[MAXSTR] = "pdflatex.exe";
 char viewdvi[MAXSTR] = "windvi.exe";
 char Rcommand[MAXSTR] = "RGui.exe";
-#elif defined(OSX_BUILD)
+#elif defined(OS_OSX)
 char calculator[MAXSTR] = "/Applications/Calculator.app/Contents/MacOS/Calculator";
 char latex[MAXSTR] = "pdflatex";
 char viewdvi[MAXSTR] = "xdvi";
@@ -271,6 +273,8 @@ static int script_type (const char *fname)
 	return EDIT_OX;
     } else if (has_suffix(fname, ".m")) {
 	return EDIT_OCTAVE;
+    } else if (has_suffix(fname, ".py")) {
+	return EDIT_PYTHON;
     } else {
 	return 0;
     }
@@ -325,7 +329,7 @@ static void real_nls_init (void)
     bind_textdomain_codeset(PACKAGE, "UTF-8");
 }
 
-#elif defined(OSX_BUILD) 
+#elif defined(OS_OSX) 
 
 static void real_nls_init (void)
 {
@@ -703,7 +707,7 @@ int main (int argc, char **argv)
     gtk_main();
 
     /* clean up before exiting */
-    free_session();
+    free_session(OPT_NONE);
 
     destroy_working_model(model);
 
@@ -761,12 +765,12 @@ static void mdata_avoid_zero (GtkTreeView *view, gpointer p)
 
 /* keystrokes recognized in the main gretl window */
 
-static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
+static gint catch_mdata_key (GtkWidget *w, GdkEventKey *event, 
+			     windata_t *vwin)
 {
-    GdkModifierType mods = widget_get_pointer_mask(w);
-    int Ctrl = (mods & GDK_CONTROL_MASK);
-    int Alt = (mods & GDK_MOD1_MASK);
-    int k = key->keyval;
+    int Ctrl = (event->state & GDK_CONTROL_MASK);
+    int Alt = (event->state & GDK_MOD1_MASK);
+    int k = event->keyval;
 
     if (Ctrl && k == GDK_v) {
 	/* Ctrl-V for paste */
@@ -1417,6 +1421,7 @@ GtkActionEntry main_entries[] = {
     { "GnuplotScript", NULL, N_("gnuplot script"), NULL, NULL, G_CALLBACK(new_script_callback) },
     { "RScript", NULL, N_("R script"), NULL, NULL, G_CALLBACK(new_script_callback) },
     { "OctaveScript", NULL, N_("Octave script"), NULL, NULL, G_CALLBACK(new_script_callback) },
+    { "PyScript", NULL, N_("Python script"), NULL, NULL, G_CALLBACK(new_script_callback) },
     { "OxScript", NULL, N_("Ox program"), NULL, NULL, G_CALLBACK(new_script_callback) },
 
     { "SessionFiles", NULL, N_("_Session files"), NULL, NULL, NULL },
@@ -1451,6 +1456,7 @@ GtkActionEntry main_entries[] = {
     { "SetSeed", NULL, N_("_Seed for random numbers"), NULL, NULL, G_CALLBACK(rand_seed_dialog) },
     { "CommandLog", NULL, N_("_Command log"), NULL, NULL, G_CALLBACK(view_command_log) },
     { "ShowConsole", NULL, N_("_Gretl console"), NULL, NULL, G_CALLBACK(gretl_console) },
+    { "Gnuplot", NULL, N_("_Gnuplot"), NULL, NULL, G_CALLBACK(start_gnuplot_callback) },
     { "StartR", NULL, N_("Start GNU _R"), NULL, NULL, G_CALLBACK(start_R_callback) },
     { "NistTest", NULL, N_("_NIST test suite"), NULL, NULL, NULL },
     { "NistBasic", NULL, N_("_Basic"), NULL, NULL, G_CALLBACK(do_nistcheck) },
@@ -1524,6 +1530,7 @@ GtkActionEntry main_entries[] = {
     { "PeriodDums", NULL, N_("_Periodic dummies"), NULL, NULL, G_CALLBACK(add_dummies) },
     { "UnitDums", NULL, N_("_Unit dummies"), NULL, NULL, G_CALLBACK(add_dummies) },
     { "TimeDums", NULL, N_("_Time dummies"), NULL, NULL, G_CALLBACK(add_dummies) },
+    { "RangeDum", NULL, N_("_Observation range dummy"), NULL, NULL, G_CALLBACK(range_dummy_dialog) },
     { "dummify", NULL, N_("Dummies for selected _discrete variables"), NULL, NULL, 
       G_CALLBACK(logs_etc_callback) },
     { "NewMatrix", NULL, N_("_Define matrix..."), NULL, NULL, G_CALLBACK(new_matrix_callback) },
@@ -1596,9 +1603,7 @@ GtkActionEntry main_entries[] = {
     { "mpols", NULL, N_("High _precision OLS..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "anova", NULL, N_("ANOVA..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "TSModels", NULL, N_("_Time series"), NULL, NULL, NULL },
-    { "CORC", NULL, N_("_Cochrane-Orcutt..."), NULL, NULL, G_CALLBACK(model_callback) },
-    { "HILU", NULL, N_("_Hildreth-Lu..."), NULL, NULL, G_CALLBACK(model_callback) },
-    { "PWE", NULL, N_("_Prais-Winsten..."), NULL, NULL, G_CALLBACK(model_callback) },
+    { "ar1", NULL, N_("_AR(1)..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "ar", NULL, N_("_Autoregressive estimation..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "arima", NULL, N_("ARI_MA..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "garch", NULL, N_("_GARCH..."), NULL, NULL, G_CALLBACK(model_callback) },
@@ -1614,7 +1619,7 @@ GtkActionEntry main_entries[] = {
     { "PANEL_B", NULL, N_("_Between model..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "dpanel", NULL, N_("_Dynamic panel model..."), NULL, NULL, G_CALLBACK(model_callback) },
 
-    { "NonlinearModels", NULL, N_("_Nonlinear models"), NULL, NULL, NULL },
+    { "LimdepModels", NULL, N_("_Limited dependent variable"), NULL, NULL, NULL },
     { "logit", NULL, N_("_Logit"), NULL, NULL, NULL }, 
     { "blogit", NULL, N_("_Binary..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "ologit", NULL, N_("_Ordered..."), NULL, NULL, G_CALLBACK(model_callback) }, 
@@ -1628,7 +1633,6 @@ GtkActionEntry main_entries[] = {
     { "countmod", NULL, N_("_Count data..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "duration", NULL, N_("_Duration data..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "logistic", NULL, N_("Lo_gistic..."), NULL, NULL, G_CALLBACK(model_callback) }, 
-    { "nls", NULL, N_("_Nonlinear Least Squares..."), NULL, NULL, G_CALLBACK(gretl_callback) }, 
     { "intreg", NULL, N_("_Interval regression..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "RobustModels", NULL, N_("_Robust estimation"), NULL, NULL, NULL },
     { "lad", NULL, N_("Least _Absolute Deviation..."), NULL, NULL, G_CALLBACK(model_callback) }, 
@@ -1636,6 +1640,7 @@ GtkActionEntry main_entries[] = {
     { "spearman", NULL, N_("_Rank correlation..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "loess", NULL, N_("_Loess..."), NULL, NULL, G_CALLBACK(selector_callback) }, 
     { "nadarwat", NULL, N_("_Nadaraya-Watson..."), NULL, NULL, G_CALLBACK(selector_callback) },
+    { "nls", NULL, N_("_Nonlinear Least Squares..."), NULL, NULL, G_CALLBACK(gretl_callback) }, 
     { "mle", NULL, N_("_Maximum likelihood..."), NULL, NULL, G_CALLBACK(gretl_callback) }, 
     { "gmm", NULL, N_("_GMM..."), NULL, NULL, G_CALLBACK(gretl_callback) }, 
     { "system", NULL, N_("_Simultaneous equations..."), NULL, NULL, G_CALLBACK(gretl_callback) }, 
@@ -1652,11 +1657,20 @@ GtkActionEntry main_entries[] = {
     { "About", GTK_STOCK_ABOUT, N_("_About gretl"), NULL, NULL, G_CALLBACK(about_dialog) }
 };
 
+/* Given an "internal" menu path, as in gretlmain.xml (with one or
+   two slash-separated components), return its user-visible
+   counterpart, translated.
+*/
+
 gchar *get_user_menu_string (const gchar *mpath)
 {
     gchar *ret = NULL;
     gchar **S;
 
+    if (mpath == NULL) {
+	return NULL;
+    }
+
     if (!strncmp(mpath, "/menubar/", 9)) {
 	mpath += 8;
     } else if (!strncmp(mpath, "MAINWIN/", 8)) {
@@ -1664,10 +1678,13 @@ gchar *get_user_menu_string (const gchar *mpath)
     } else if (*mpath == '/') {
 	mpath += 1;
     } else if (!strncmp(mpath, "MODELWIN/", 9)) {
+	/* this function is main-window only */
 	return NULL;
     }
 
+#if 0
     fprintf(stderr, "get_user_menu_string: mpath='%s'\n", mpath);
+#endif
 
     S = g_strsplit(mpath, "/", 0);
 
@@ -1690,9 +1707,9 @@ gchar *get_user_menu_string (const gchar *mpath)
 	    }
 	}
 	if (p2 != NULL) {
-	    ret = g_strdup_printf("/%s/%s", p1, p2);
+	    ret = g_strdup_printf("%s/%s", _(p1), _(p2));
 	} else if (p1 != NULL) {
-	    ret = g_strdup_printf("/%s", p1);
+	    ret = g_strdup_printf("%s", _(p1));
 	}
     }
 	
@@ -1812,11 +1829,12 @@ static GtkWidget *make_main_menu (void)
 	    } else {
 		g_signal_connect(G_OBJECT(dataitem), "activate",
 				 G_CALLBACK(check_var_labels_state), mdata);
-		g_free(main_ui);
 	    }
 	}
     }
 
+    g_free(main_ui);
+
     return menu;
 }
 
@@ -1936,6 +1954,11 @@ static void start_R_callback (void)
     start_R(NULL, 1, 1);
 }
 
+static void start_gnuplot_callback (void)
+{
+    launch_gnuplot_interactive(NULL);
+}
+
 #ifndef G_OS_WIN32
 
 int gretl_fork (const char *progvar, const char *arg)
@@ -1945,12 +1968,12 @@ int gretl_fork (const char *progvar, const char *arg)
     GError *err = NULL;
     gboolean run;
 
-#ifdef OSX_BUILD
+#ifdef OS_OSX
     if (!strcmp(progvar, "calculator")) {
 	prog = calculator;
     } else if (!strcmp(progvar, "viewdvi")) {
 	prog = viewdvi;
-    } 
+    }
 #else
     if (!strcmp(progvar, "Browser")) {
 	prog = Browser;
@@ -1962,7 +1985,7 @@ int gretl_fork (const char *progvar, const char *arg)
 	prog = viewpdf;
     } else if (!strcmp(progvar, "viewps")) {
 	prog = viewps;
-    }    
+    }
 #endif
 
     if (prog == NULL) {
@@ -2177,9 +2200,7 @@ int mdata_active_var (void)
 static gboolean 
 main_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer data)
 {
-    GdkModifierType mods = widget_get_pointer_mask(w);
-
-    if (RIGHT_CLICK(mods)) {
+    if (right_click(event)) {
 	/* ignore all but right-clicks */
 	int selcount = vwin_selection_count(mdata, NULL);
 
