@@ -25,6 +25,7 @@
 				r == EDIT_R ||	    \
 				r == EDIT_OX ||     \
                                 r == EDIT_OCTAVE || \
+				r == EDIT_PYTHON || \
 				r == EDIT_X12A)
 
 #define vwin_editing_buffer(r) (r == EDIT_HEADER || \
@@ -128,7 +129,7 @@ int gui_validate_varname_strict (const char *name, GretlType t);
 
 gretlopt get_tex_eqn_opt (void);
 
-gint popup_menu_handler (GtkWidget *widget, GdkEvent *event,
+gint popup_menu_handler (GtkWidget *widget, GdkEventButton *event,
 			 gpointer data);
 
 void add_popup_item (const gchar *label, GtkWidget *menu,
@@ -150,9 +151,7 @@ int gretl_file_get_contents (const gchar *fname, gchar **contents, gsize *size);
 
 void start_R (const char *buf, int send_data, int interactive);
 
-void run_ox_script (gchar *buf);
-
-void run_octave_script (gchar *buf);
+void run_foreign_script (gchar *buf, int lang);
 
 const char *print_today (void);
 
