@@ -145,22 +145,18 @@ static struct gui_help_item gui_help_items[] = {
     { LOESS,          "loess" },
     { NADARWAT,       "nadarwat" },
     { CLUSTER,        "cluster" },
+    { GUI_FUNCS,      "gui-funcs" },
+    { MENU_ATTACH,    "menu-attach" },
     { -1,             NULL },
 };
 
 enum {
-    COMPAT_CORC = GUI_CMD_MAX + 1,
-    COMPAT_FCASTERR,
-    COMPAT_HILU,
-    COMPAT_PWE
+    COMPAT_FCASTERR = GUI_CMD_MAX + 1
 };
 
 static struct gui_help_item compat_help_items[] = {
     { GUI_CMD_MAX,     "nothing" },
-    { COMPAT_CORC,     "corc" },
     { COMPAT_FCASTERR, "fcasterr" },
-    { COMPAT_HILU,     "hilu" },
-    { COMPAT_PWE,      "pwe" },
     { -1,              NULL }
 };
 
@@ -748,10 +744,12 @@ void notify_string_not_found (GtkWidget *entry)
                           r == CLI_HELP_EN || \
                           r == FUNCS_HELP)
 
-static gboolean finder_key_handler (GtkEntry *entry, GdkEventKey *key,
+static gboolean finder_key_handler (GtkEntry *entry, GdkEventKey *event,
 				    windata_t *vwin)
 {
-    if (key->keyval == GDK_Tab && help_index_ok(vwin->role) &&
+    guint keyval = event->keyval;
+
+    if (keyval == GDK_Tab && help_index_ok(vwin->role) &&
 	vwin->active_var == 0) {
 	/* tab-completion in help index mode */
 	const gchar *s = gtk_entry_get_text(entry);
@@ -772,15 +770,10 @@ static gboolean finder_key_handler (GtkEntry *entry, GdkEventKey *key,
 
 	    return TRUE;
 	}
-    } else if (key->keyval == GDK_g) {
+    } else if (keyval == GDK_g && (event->state & GDK_CONTROL_MASK)) {
 	/* Ctrl-G: repeat search */
-	GdkModifierType mods = 
-	    widget_get_pointer_mask(GTK_WIDGET(entry));
-
-	if (mods & GDK_CONTROL_MASK) {
-	    vwin_finder_callback(entry, vwin);
-	    return TRUE;
-	}
+	vwin_finder_callback(entry, vwin);
+	return TRUE;
     }
 
     return FALSE;
@@ -1473,7 +1466,7 @@ void text_find (gpointer unused, gpointer data)
 	gtk_editable_select_region(GTK_EDITABLE(vwin->finder), 
 				   0, -1);
     } else {
-	find_string_dialog(find_in_text, data);
+	find_string_dialog(find_in_text, vwin);
     }
 }
 
@@ -1486,7 +1479,7 @@ void listbox_find (gpointer unused, gpointer data)
 	gtk_editable_select_region(GTK_EDITABLE(vwin->finder), 
 				   0, -1);
     } else {
-	find_string_dialog(find_in_listbox, data);
+	find_string_dialog(find_in_listbox, vwin);
     }
 }
 
@@ -1817,7 +1810,7 @@ static void find_string_dialog (void (*findfunc)(), windata_t *vwin)
 	return;
     }
 
-    find_dialog = gretl_dialog_new(_("gretl: find"), NULL, 0);
+    find_dialog = gretl_dialog_new(_("gretl: find"), vwin->main, 0);
     g_object_set_data(G_OBJECT(find_dialog), "windat", vwin);
     parent_find(find_dialog, vwin);
 
@@ -1975,7 +1968,7 @@ static int get_x12a_doc_path (char *path, const char *fname)
 	    *path = '\0';
 	}
 
-#if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
+#if !defined(G_OS_WIN32) && !defined(OS_OSX)
 	if (!ret) {
 	    /* using gretl x12a package? */
 	    sprintf(path, "/opt/x12arima/docs/%s", fname);
@@ -2089,7 +2082,7 @@ void gretl_show_pdf (const char *fname)
 {
 #if defined(G_OS_WIN32)
     win32_open_file(fname);
-#elif defined(OSX_BUILD)
+#elif defined(OS_OSX)
     osx_open_file(fname);
 #else
     gretl_fork("viewpdf", fname);
