@@ -31,7 +31,7 @@ int winfork (char *cmdline, const char *dir, int wshow,
 	     DWORD flags);
 #endif
 
-#ifdef OSX_BUILD
+#ifdef OS_OSX
 int osx_open_file (const char *path);
 
 int osx_open_url (const char *url);
@@ -129,6 +129,8 @@ int do_add_omit (selector *sr);
 
 int do_VAR_omit (selector *sr);
 
+void VAR_omit_auto (GtkAction *action, gpointer p);
+
 int do_confidence_region (selector *sr);
 
 void do_modtest (GtkAction *action, gpointer p);
@@ -201,6 +203,8 @@ void do_selector_genr (GtkWidget *w, dialog_t *dlg);
 
 void do_fncall_genr (GtkWidget *w, dialog_t *dlg);
 
+void do_range_dummy_genr (const gchar *buf);
+
 void do_global_setmiss (GtkWidget *w, dialog_t *dlg);
 
 void do_variable_setmiss (GtkWidget *w, dialog_t *dlg);
