@@ -28,6 +28,7 @@
 #include "textbuf.h"
 #include "ssheet.h"
 #include "selector.h"
+#include "gpt_control.h"
 
 #include "libset.h"
 #include "texprint.h"
@@ -105,6 +106,7 @@ static char hc_panel[9] = "Arellano";
 static char hc_garch[5] = "QML";
 
 static int lcnumeric = 1;
+static double graph_scale = 1.0;
 
 #ifdef G_OS_WIN32
 extern int use_wimp;
@@ -115,17 +117,18 @@ char midiplayer[MAXSTR] = "timidity -ig";
 #endif
 
 typedef enum {
-    USERSET  = 1 << 0,  /* user-lavel variable */
+    USERSET  = 1 << 0,  /* user-level variable */
     BOOLSET  = 1 << 1,  /* boolean value (user) */
     INTSET   = 1 << 2,  /* integer value (user) */
-    LISTSET  = 1 << 3,  /* user string, from fixed menu */
-    RADIOSET = 1 << 4,  /* user int, from fixed menu */
-    INVISET  = 1 << 5,  /* not visible in Preferences dialog */
-    FIXSET   = 1 << 6,  /* setting fixed by admin (Windows network use) */
-    MACHSET  = 1 << 7,  /* "local machine" setting */
-    BROWSER  = 1 << 8,  /* wants "Browse" button */
-    RESTART  = 1 << 9,  /* needs program restart to take effect */
-    GOTSET   = 1 << 10  /* dynamic: already found a setting */
+    FLOATSET = 1 << 3,  /* floating point value (user) */
+    LISTSET  = 1 << 4,  /* user selection from fixed menu */
+    RADIOSET = 1 << 5,  /* user int, from fixed menu */
+    INVISET  = 1 << 6,  /* not visible in Preferences dialog */
+    FIXSET   = 1 << 7,  /* setting fixed by admin (Windows network use) */
+    MACHSET  = 1 << 8,  /* "local machine" setting */
+    BROWSER  = 1 << 9,  /* wants "Browse" button */
+    RESTART  = 1 << 10, /* needs program restart to take effect */
+    GOTSET   = 1 << 11  /* dynamic: already found a setting */
 } rcflags;
 
 typedef struct {
@@ -151,8 +154,10 @@ RCVAR rc_vars[] = {
       MACHSET | BROWSER, sizeof paths.gretldir, TAB_MAIN, NULL },
     { "userdir", N_("User's gretl directory"), NULL, paths.workdir, 
       INVISET, sizeof paths.workdir, TAB_MAIN, NULL },
+#if 0 /* not working anyway */
     { "updater", N_("Tell me about gretl updates"), NULL, &updater, 
       BOOLSET, 0, TAB_MAIN, NULL },
+#endif
 #ifndef G_OS_WIN32
     { "winsize", N_("Remember main window size"), NULL, &winsize, 
       BOOLSET, 0, TAB_MAIN, NULL },
@@ -165,7 +170,7 @@ RCVAR rc_vars[] = {
     { "wimp", N_("Emulate Windows look"), NULL, &use_wimp, 	 
       BOOLSET | RESTART, 0, TAB_MAIN, NULL }, 	 
 #endif
-#if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
+#if !defined(G_OS_WIN32) && !defined(OS_OSX)
     { "browser", N_("Web browser"), NULL, Browser, 
       MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
 #endif
@@ -189,6 +194,8 @@ RCVAR rc_vars[] = {
     { "langpref", N_("Language preference"), NULL, langpref, 
       LISTSET | RESTART, 32, TAB_MAIN, NULL },
 #endif
+    { "graph_scale", N_("Default graph scale"), NULL, &graph_scale,
+      LISTSET | FLOATSET, 0, TAB_MAIN, NULL },
 #ifndef G_OS_WIN32 
     { "gnuplot", N_("Command to launch gnuplot"), NULL, paths.gnuplot, 
       MACHSET | BROWSER, MAXLEN, TAB_PROGS, NULL },
@@ -196,14 +203,14 @@ RCVAR rc_vars[] = {
     { "Rcommand", N_("Command to launch GNU R"), NULL, Rcommand, 
       MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
 #ifdef G_OS_WIN32 
-    { "Rbin", "Path to Rterm.exe", NULL, paths.rbinpath, 
-      MACHSET | BROWSER, MAXSTR, TAB_NONE, NULL },
+    { "Rbin", N_("Path to Rterm.exe"), NULL, paths.rbinpath, 
+      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL }, /* was TAB_NONE? */
 #endif
     { "latex", N_("Command to compile TeX files"), NULL, latex, 
       MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
     { "viewdvi", N_("Command to view DVI files"), NULL, viewdvi, 
       MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
-#if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
+#if !defined(G_OS_WIN32) && !defined(OS_OSX)
     { "viewps", N_("Command to view postscript files"), NULL, viewps, 
       MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
     { "viewpdf", N_("Command to view PDF files"), NULL, viewpdf, 
@@ -231,6 +238,10 @@ RCVAR rc_vars[] = {
       MACHSET | BROWSER, sizeof paths.oxlpath, TAB_PROGS, NULL},
     { "octave", N_("Path to octave executable"), NULL, paths.octpath, 
       MACHSET | BROWSER, sizeof paths.octpath, TAB_PROGS, NULL},
+    { "stata", N_("Path to Stata executable"), NULL, paths.statapath, 
+      MACHSET | BROWSER, sizeof paths.statapath, TAB_PROGS, NULL},
+    { "python", N_("Path to Python executable"), NULL, paths.pypath, 
+      MACHSET | BROWSER, sizeof paths.statapath, TAB_PROGS, NULL},
     { "dbhost", N_("Database server name"), NULL, paths.dbhost, 
       USERSET, sizeof paths.dbhost, TAB_NET, NULL },
     { "dbproxy", N_("HTTP proxy"), NULL, http_proxy, 
@@ -314,6 +325,8 @@ int autoicon_on (void)
 	return autoicon;
     } else if (n_user_matrices() > 0) {
 	return autoicon;
+    } else if (n_user_bundles() > 0) {
+	return autoicon;
     } else {
 	return 0;
     }
@@ -569,7 +582,7 @@ static const char *get_reg_base (const char *key)
 }
 #endif
 
-#ifdef OSX_BUILD
+#ifdef OS_OSX
 static int alt_ok (const char *prog)
 {
     char *p, test[MAXSTR];
@@ -624,7 +637,7 @@ static void set_tramo_status (void)
 	const char *tramo = gretl_tramo();
 
 	ok = check_for_prog(tramo);
-# ifdef OSX_BUILD
+# ifdef OS_OSX
 	if (!ok) {
 	    ok = alt_ok(tramo);
 	}
@@ -667,7 +680,7 @@ static void set_x12a_status (void)
 	const char *x12a = gretl_x12_arima();
 
 	ok = check_for_prog(x12a);
-# ifdef OSX_BUILD    
+# ifdef OS_OSX    
 	if (!ok) {
 	    ok = alt_ok(x12a);
 	}
@@ -824,9 +837,9 @@ static void root_check (void)
     if (getuid() == 0) {
 	int resp;
 
-	resp = yes_no_dialog ("gretl", _("You seem to be running gretl " 
-			      "as root.  Do you really want to do this?"), 
-			      0);
+	resp = yes_no_dialog("gretl", _("You seem to be running gretl " 
+					"as root.  Do you really want to do this?"), 
+			     0);
 	if (resp == GRETL_NO) {
 	    exit(EXIT_FAILURE);
 	}
@@ -892,7 +905,8 @@ int options_dialog (int page, const char *varname, GtkWidget *parent)
 	return 0;
     }
 
-    dialog = gretl_dialog_new(_("gretl: options"), parent, GRETL_DLG_BLOCK);
+    dialog = gretl_dialog_new(_("gretl: options"), parent, 
+			      GRETL_DLG_RESIZE | GRETL_DLG_BLOCK);
 #if GTK_MAJOR_VERSION < 3
     gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
 #endif
@@ -1087,7 +1101,7 @@ static const char **get_list_setting_strings (void *var, int *n)
     } else if (var == hc_garch) {
 	strs = garch_strs;
 	*n = sizeof garch_strs / sizeof garch_strs[0];
-    } 
+    }
 
     return strs;
 }
@@ -1159,7 +1173,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 {
     GtkWidget *b_table = NULL, *s_table = NULL;
     GtkWidget *l_table = NULL;
-    GtkWidget *box, *w = NULL;
+    GtkWidget *vbox, *w = NULL;
     int s_len = 1, b_len = 0, l_len = 1;
     int s_cols, b_cols = 0, l_cols = 0;
     int b_col = 0;
@@ -1167,13 +1181,12 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
     int n_bool = 0;
     int n_browse = 0;
     int n_list = 0;
-    int langs = 0;
     RCVAR *rc;
     int i;
    
-    box = gtk_vbox_new(FALSE, 0);
-    gtk_container_set_border_width(GTK_CONTAINER(box), 10);
-    gtk_widget_show(box);
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
+    gtk_widget_show(vbox);
 
     if (tab == TAB_MAIN) {
 	w = gtk_label_new(_("General"));
@@ -1188,7 +1201,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
     }
     
     gtk_widget_show(w);
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box, w);   
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, w);   
 
     get_table_sizes(tab, &n_str, &n_bool, &n_browse, &n_list);
 
@@ -1200,7 +1213,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	l_table = gtk_table_new(l_len, l_cols, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(l_table), 5);
 	gtk_table_set_col_spacings(GTK_TABLE(l_table), 5);
-	gtk_box_pack_start(GTK_BOX(box), l_table, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), l_table, FALSE, FALSE, 0);
 	gtk_widget_show(l_table);
     }	
 
@@ -1208,7 +1221,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	s_table = gtk_table_new(s_len, s_cols, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(s_table), 5);
 	gtk_table_set_col_spacings(GTK_TABLE(s_table), 5);
-	gtk_box_pack_start(GTK_BOX(box), s_table, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), s_table, FALSE, FALSE, 0);
 	gtk_widget_show(s_table);
     }
 
@@ -1217,20 +1230,22 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	b_table = gtk_table_new(1, b_cols, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(b_table), 5);
 	gtk_table_set_col_spacings(GTK_TABLE(b_table), 5);
-	gtk_box_pack_start(GTK_BOX(box), b_table, FALSE, FALSE, 10);
+	gtk_box_pack_start(GTK_BOX(vbox), b_table, FALSE, FALSE, 10);
 	gtk_widget_show(b_table);
     }
 
     if (tab != TAB_VCV && n_list > 0) {
-	/* non-VCV tab: language list comes last */
+	/* non-VCV tab -- list entries come last, and we
+	   use an hbox to pack them left */
+	GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
+
 	l_cols = 2;
-#ifdef ENABLE_NLS
-	langs = 1;
-#endif
 	l_table = gtk_table_new(l_len, l_cols, FALSE);
-	gtk_table_set_row_spacings(GTK_TABLE(l_table), 5);
-	gtk_table_set_col_spacings(GTK_TABLE(l_table), 5);
-	gtk_box_pack_start(GTK_BOX(box), l_table, FALSE, FALSE, 0);
+	gtk_table_set_row_spacings(GTK_TABLE(l_table), 10);
+	gtk_table_set_col_spacings(GTK_TABLE(l_table), 10);
+	gtk_box_pack_start(GTK_BOX(hbox), l_table, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+	gtk_widget_show(hbox);
 	gtk_widget_show(l_table);
     }
 
@@ -1385,28 +1400,53 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 				 rc->var);
 	    }
 	} else if (rc->flags & LISTSET) {
+	    int langs = rc->var == langpref;
 	    int j, active = 0;
 
 	    l_len++;
 
 	    gtk_table_resize(GTK_TABLE(l_table), l_len, l_cols);
 	    w = gtk_label_new(_(rc->description));
-	    if (langs) {
-		gtk_table_attach(GTK_TABLE(l_table), w, 
-				 0, 1, l_len - 1, l_len,
-				 0, 0, 0, 0);
+	    if (tab == TAB_MAIN) {
+		gtk_misc_set_alignment(GTK_MISC(w), 0.0, 0.5);
 	    } else {		
 		gtk_misc_set_alignment(GTK_MISC(w), 1.0, 0.5);
-		gtk_table_attach_defaults(GTK_TABLE(l_table), 
-					  w, 0, 1, l_len - 1, l_len);
 	    } 
+	    gtk_table_attach_defaults(GTK_TABLE(l_table), 
+				      w, 0, 1, l_len - 1, l_len);
 	    gtk_widget_show(w);
 
 	    rc->widget = gtk_combo_box_text_new();
-	    gtk_table_attach(GTK_TABLE(l_table), rc->widget, 
-			     1, 2, l_len - 1, l_len,
-			     0, 0, 0, 0);
-	    if (langs) {
+
+	    if (tab == TAB_MAIN) {
+		GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
+
+		gtk_box_pack_start(GTK_BOX(hbox), rc->widget, FALSE, FALSE, 5);
+		gtk_table_attach(GTK_TABLE(l_table), hbox, 
+				 1, 2, l_len - 1, l_len,
+				 GTK_EXPAND | GTK_FILL, 0, 0, 0);
+		gtk_widget_show(hbox);
+	    } else {
+		gtk_table_attach(GTK_TABLE(l_table), rc->widget, 
+				 1, 2, l_len - 1, l_len,
+				 0, 0, 0, 0);
+	    }		
+
+	    if (rc->flags & FLOATSET) {
+		/* special: graph scale */
+		double scale, *xvar = (double *) rc->var;
+		char numstr[4];
+
+		j = 0;
+		while (get_graph_scale(j, &scale)) {
+		    sprintf(numstr, "%.1f", scale);
+		    combo_box_append_text(rc->widget, numstr);
+		    if (scale == *xvar) {
+			active = j;
+		    }
+		    j++;
+		}
+	    } else if (langs) {
 		char *strvar = (char *) rc->var;
 		const char *str;
 
@@ -1511,6 +1551,11 @@ static void set_gp_colors (void)
     }
 }
 
+static void set_gp_scale (void)
+{
+    gnuplot_png_set_default_scale(graph_scale);
+}
+
 #if defined(HAVE_TRAMO) || defined(HAVE_X12A)
 
 static void maybe_revise_tramo_x12a_status (void)
@@ -1560,6 +1605,20 @@ static void rcvar_set_string (RCVAR *rcvar, const char *sval, int *changed)
     }
 }
 
+static void rcvar_set_double (RCVAR *rcvar, const char *sval, int *changed)
+{
+    double *xvar = (double *) rcvar->var;
+
+    if (sval != NULL && *sval != '\0') {
+	double xval = atof(sval);
+
+	if (xval != *xvar) {
+	    flag_changed(rcvar, changed);
+	    *xvar = xval;
+	}
+    }
+}
+
 /* register and react to changes from Preferences dialog */
 
 static void apply_changes (GtkWidget *widget, gpointer data) 
@@ -1596,7 +1655,11 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 		} else {
 		    gchar *str = combo_box_get_active_text(box);
 
-		    rcvar_set_string(rcvar, str, &changed);
+		    if (rcvar->flags & FLOATSET) {
+			rcvar_set_double(rcvar, str, &changed);
+		    } else {
+			rcvar_set_string(rcvar, str, &changed);
+		    }
 		    g_free(str);
 		}
 	    }
@@ -1616,6 +1679,9 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 	use_proxy = 0;
     }
 
+    /* update scale factor for PNG plots */
+    gnuplot_png_set_default_scale(graph_scale);
+
     write_rc(); /* note: calls gretl_update_paths */
 
     /* register these settings for the current session using 
@@ -1673,6 +1739,10 @@ int write_rc (void)
 	    fprintf(fp, "%s = %s\n", rcvar->key, val);
 	} else if (rcvar->flags & INTSET) {
 	    fprintf(fp, "%s = %d\n", rcvar->key, *(int *) rcvar->var);
+	} else if (rcvar->flags & FLOATSET) {
+	    gretl_push_c_numeric_locale();
+	    fprintf(fp, "%s = %g\n", rcvar->key, *(double *) rcvar->var);
+	    gretl_pop_c_numeric_locale();
 	} else {
 	    strvar = (char *) rcvar->var;
 	    fprintf(fp, "%s = %s\n", rcvar->key, strvar);
@@ -1711,6 +1781,15 @@ static void str_to_int (const char *s, void *b)
     }
 }
 
+static void str_to_double (const char *s, void *b)
+{
+    double *xvar = (double *) b;
+
+    if (s != NULL && *s != '\0') {
+	*xvar = dot_atof(s);
+    }
+}
+
 static int dir_exists (const char *dname, FILE *fp)
 {
     DIR *test;
@@ -1731,7 +1810,7 @@ static int dir_exists (const char *dname, FILE *fp)
     return ok;
 }
 
-#if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
+#if !defined(G_OS_WIN32) && !defined(OS_OSX)
 
 static void maybe_fix_viewpdf (void)
 {
@@ -1779,6 +1858,7 @@ static int common_read_rc_setup (void)
     libset_set_bool(SHELL_OK, shellok);
     libset_set_bool(USE_CWD, usecwd);
     set_gp_colors();
+    set_gp_scale();
 
     set_xsect_hccme(hc_xsect);
     set_tseries_hccme(hc_tseri);
@@ -1795,7 +1875,7 @@ static int common_read_rc_setup (void)
     gretl_www_init(paths.dbhost, http_proxy, use_proxy);
     set_tex_use_pdf(latex);
 
-#if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
+#if !defined(G_OS_WIN32) && !defined(OS_OSX)
     maybe_fix_viewpdf();
 #endif
 
@@ -1837,6 +1917,8 @@ static void find_and_set_rc_var (const char *key, const char *val)
 		    str_to_boolvar(val, rcvar->var);
 		} else if (rcvar->flags & INTSET) {
 		    str_to_int(val, rcvar->var);
+		} else if (rcvar->flags & FLOATSET) {
+		    str_to_double(val, rcvar->var);
 		} else {
 		    strvar = (char *) rcvar->var;
 		    *strvar = '\0';
@@ -1878,7 +1960,7 @@ static int get_network_settings (void)
 
 	    if (sscanf(p, "%31s", key) == 1) {
 		strcpy(linevar, p + strlen(key) + 3); 
-		gretl_chopstr(linevar); 
+		gretl_strstrip(linevar); 
 		gotvar = 0;
 		for (j=0; rc_vars[j].key != NULL; j++) {
 		    if (!strcmp(key, rc_vars[j].key)) {
@@ -1947,7 +2029,7 @@ static void win32_read_gretlrc (void)
 	    /* note: don't take gretldir from here */
 	    if (strcmp(key, "gretldir")) {
 		strcpy(linevar, line + strlen(key) + 3); 
-		gretl_chopstr(linevar); 
+		gretl_strstrip(linevar); 
 		if (*linevar != '\0') {
 		    find_and_set_rc_var(key, linevar);
 		}
@@ -2075,7 +2157,7 @@ static int read_gretlrc (void)
 	    if (sscanf(line, "%31s", key) == 1) {
 		*linevar = '\0';
 		strncat(linevar, line + strlen(key) + 3, MAXLEN-1); 
-		gretl_chopstr(linevar); 
+		gretl_strstrip(linevar); 
 		if (*linevar != '\0') {
 		    find_and_set_rc_var(key, linevar);
 		}
