@@ -1406,7 +1406,7 @@ static void update_scalars_from_sheet (Spreadsheet *sheet)
 	if (vname != NULL && *vname != '\0') {
 	    x = (*val == '\0')? NADBL : atof(val);
 	    if (gretl_is_scalar(vname)) {
-		if (x != gretl_scalar_get_value(vname)) {
+		if (x != gretl_scalar_get_value(vname, NULL)) {
 		    gretl_scalar_set_value(vname, x);
 		    sheet_set_modified(sheet, TRUE);
 		}
@@ -2389,10 +2389,10 @@ static gint catch_spreadsheet_key (GtkWidget *view, GdkEventKey *key,
     return FALSE;
 }
 
-static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
+static gint catch_spreadsheet_click (GtkWidget *view, 
+				     GdkEventButton *event,
 				     Spreadsheet *sheet)
 {   
-    GdkModifierType mods; 
     gint ret = FALSE;
 
 #if CELLDEBUG
@@ -2403,23 +2403,19 @@ static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
 	return FALSE;
     }
 
-    mods = widget_get_pointer_mask(view);
-
     if (sheet->matrix == NULL && !editing_scalars(sheet) && 
-	(RIGHT_CLICK(mods))) {
-	GdkEventButton *bevent = (GdkEventButton *) event;
+	(right_click(event))) {
 
 	if (sheet->popup == NULL) {
 	    build_sheet_popup(sheet);
 	}
 
 	gtk_menu_popup(GTK_MENU(sheet->popup), NULL, NULL, NULL, NULL,
-		       bevent->button, bevent->time);
+		       event->button, event->time);
 	return TRUE;
     }	    
 	
-    if (mods & GDK_BUTTON1_MASK) {
-	GdkEventButton *bevent = (GdkEventButton *) event;
+    if (event->button == 1) {
 	GtkTreePath *path = NULL, *oldpath = NULL;
 	GtkTreeViewColumn *column = NULL, *oldcol = NULL;	
 
@@ -2446,8 +2442,8 @@ static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
 	}
 
 	gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(sheet->view),
-				      (gint) bevent->x, 
-				      (gint) bevent->y,
+				      (gint) event->x, 
+				      (gint) event->y,
 				      &path, &column,
 				      NULL, NULL);
 
