@@ -54,7 +54,8 @@ enum {
 
 #define gui_help(r) (r == GUI_HELP || r == GUI_HELP_EN)
 #define foreign_script_role(r) (r == EDIT_GP || r == EDIT_R || \
-				r == EDIT_OX || r == EDIT_OCTAVE)
+				r == EDIT_OX || r == EDIT_OCTAVE || \
+				r == EDIT_PYTHON)
 
 /* globals accessed in settings.c */
 int tabwidth = 4;
@@ -419,6 +420,8 @@ static void sourceview_apply_language (windata_t *vwin)
 	id = "cpp";
     } else if (vwin->role == EDIT_OCTAVE) {
 	id = "octave";
+    } else if (vwin->role == EDIT_PYTHON) {
+	id = "python";
     } else {
 	id = "gretl";
     }
@@ -426,7 +429,18 @@ static void sourceview_apply_language (windata_t *vwin)
     lang = gtk_source_language_manager_get_language(lm, id);
 
     if (lang == NULL) {
+	const gchar * const *S = gtk_source_language_manager_get_search_path(lm);
+
 	fprintf(stderr, "*** gtksourceview: lang is NULL for id='%s'\n", id);
+	if (S != NULL) {
+	    fprintf(stderr, "the gtksourceview search path:\n");
+	    while (*S != NULL) {
+		fprintf(stderr, " %s\n", *S);
+		S++;
+	    }
+	} else {
+	    fprintf(stderr, "the gtksourceview search path is NULL!\n");
+	}
     } else {
 	gtk_source_buffer_set_language(vwin->sbuf, lang);
     }
@@ -581,16 +595,16 @@ static void set_source_tabs (GtkWidget *w, int cw)
 */
 
 static gint 
-script_key_handler (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
+script_key_handler (GtkWidget *w, GdkEventKey *event, windata_t *vwin)
 {
-    GdkModifierType mods = widget_get_pointer_mask(w);
+    guint keyval = event->keyval;
     gboolean ret = FALSE;
 
-    if (mods & GDK_CONTROL_MASK) {
-	if (key->keyval == GDK_r)  {
+    if (event->state & GDK_CONTROL_MASK) {
+	if (keyval == GDK_r)  {
 	    do_run_script(w, vwin);
 	    ret = TRUE;
-	} else if (key->keyval == GDK_Return) {
+	} else if (keyval == GDK_Return) {
 	    gchar *str = textview_get_current_line_with_newline(w);
 
 	    if (str != NULL) {
@@ -602,14 +616,14 @@ script_key_handler (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 	    ret = TRUE;
 	}
     } else {
-	if (key->keyval == GDK_F1) {
+	if (keyval == GDK_F1) {
 	    set_window_help_active(vwin);
 	    interactive_script_help(NULL, NULL, vwin);
 	} else if (script_editing(vwin->role)) {    
-	    if (key->keyval == GDK_Return) {
+	    if (keyval == GDK_Return) {
 		ret = script_electric_enter(vwin);
-	    } else if (tabkey(key->keyval)) {
-		ret = script_tab_handler(vwin, mods);
+	    } else if (tabkey(keyval)) {
+		ret = script_tab_handler(vwin, event->state);
 	    }
 	}
     } 
@@ -1414,7 +1428,7 @@ cmdref_motion_notify (GtkWidget *w, GdkEventMotion *event)
     gtk_text_view_window_to_buffer_coords(view, GTK_TEXT_WINDOW_WIDGET,
 					  event->x, event->y, &x, &y);
     set_cursor_if_appropriate(view, x, y);
-    widget_get_pointer_mask(w);
+    // widget_get_pointer_mask(w); /* ?? */
 
     return FALSE;
 }
@@ -1650,9 +1664,7 @@ static GtkWidget *build_help_popup (windata_t *hwin)
 gboolean 
 help_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p)
 {
-    GdkModifierType mods = widget_get_pointer_mask(w);
-
-    if (RIGHT_CLICK(mods)) {
+    if (right_click(event)) {
 	windata_t *hwin = (windata_t *) p;
 
 	if (hwin->active_var == 0 && pop_backpage(w) == 0) {
@@ -1936,6 +1948,7 @@ static void normalize_indent (GtkTextBuffer *tbuf,
     char word[9], line[1024];
     const char *ins;
     int incomment = 0;
+    int inforeign = 0;
     int i, nsp;
 
     if (buf == NULL) {
@@ -1963,7 +1976,18 @@ static void normalize_indent (GtkTextBuffer *tbuf,
 	if (!incomment) {
 	    *word = '\0';
 	    textbuf_get_cmdword(ins, word);
-	    adjust_indent(word, &this_indent, &next_indent);
+	    if (!strcmp(word, "foreign")) {
+		inforeign = 1;
+	    } else if (inforeign) {
+		if (!strncmp(ins, "end foreign", 11)) {
+		    inforeign = 0;
+		} else {
+		    gtk_text_buffer_insert(tbuf, start, line, -1);
+		    continue;
+		}
+	    } else {
+		adjust_indent(word, &this_indent, &next_indent);
+	    }
 	}
 	nsp = this_indent * tabwidth;
 	if (incomment == 2) {
@@ -1978,6 +2002,37 @@ static void normalize_indent (GtkTextBuffer *tbuf,
     bufgets_finalize(buf); 
 }
 
+static int in_foreign_land (GtkWidget *text_widget)
+{
+    GtkTextBuffer *tbuf;
+    GtkTextIter start, end;
+    gchar *buf;
+    char *s, line[1024];
+    int inforeign = 0;
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_widget));
+    gtk_text_buffer_get_start_iter(tbuf, &start);
+    gtk_text_buffer_get_iter_at_mark(tbuf, &end, 
+				     gtk_text_buffer_get_insert(tbuf));
+    buf = gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
+
+    bufgets_init(buf);
+
+    while (bufgets(line, sizeof line, buf)) {
+	s = line + strspn(line, " \t");
+	if (!strncmp(s, "foreign", 7)) {
+	    inforeign = 1;
+	} else if (!strncmp(s, "end foreign", 7)) {
+	    inforeign = 0;
+	}
+    }
+
+    bufgets_finalize(buf);
+    g_free(buf);
+
+    return inforeign;
+}
+
 static void auto_indent_script (GtkWidget *w, windata_t *vwin)
 {
     GtkTextBuffer *tbuf;
@@ -2384,7 +2439,7 @@ static gboolean script_electric_enter (windata_t *vwin)
     int targsp = 0;
     gboolean ret = FALSE;
 
-    if (!smarttab) {
+    if (!smarttab || in_foreign_land(vwin->text)) {
 	return FALSE;
     }
 
@@ -2481,7 +2536,7 @@ static gboolean script_electric_enter (windata_t *vwin)
     return ret;
 }
 
-/* handler for the user pressing the Tab key when editing a script */
+/* handler for the user pressing the Tab key when editing a gretl script */
 
 static gboolean script_tab_handler (windata_t *vwin, GdkModifierType mods)
 {
@@ -2490,6 +2545,10 @@ static gboolean script_tab_handler (windata_t *vwin, GdkModifierType mods)
 
     g_return_val_if_fail(GTK_IS_TEXT_VIEW(vwin->text), FALSE);
 
+    if (in_foreign_land(vwin->text)) {
+	return FALSE;
+    }
+
     if (smarttab && !(mods & GDK_SHIFT_MASK)) {
 	if (maybe_insert_smart_tab(vwin)) {
 	    return TRUE;
@@ -2657,9 +2716,7 @@ static gboolean destroy_textbit (GtkWidget **pw, struct textbit *tc)
 static gboolean 
 script_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p)
 {
-    GdkModifierType mods = widget_get_pointer_mask(w);
-
-    if (RIGHT_CLICK(mods)) {
+    if (right_click(event)) {
 	windata_t *vwin = (windata_t *) p;
 	struct textbit *tc = NULL;
 
