@@ -103,7 +103,7 @@ static int day_in_year (int day, int month, int year)
 
 long epoch_day_from_ymd (int y, int m, int d)
 {
-    long temp;
+    long ret;
 
     if (y < 0 || m < 0 || d < 0) {
 	return -1;
@@ -113,30 +113,33 @@ long epoch_day_from_ymd (int y, int m, int d)
 	return -1;
     }
 
-    temp = (long)(y - 1) * 365 + leap_years_since_year_1(y - 1)
+    ret = (long)(y - 1) * 365 + leap_years_since_year_1(y - 1)
 	+ day_in_year(d, m, y);
 
-    return temp;
+    return ret;
 }
 
 /**
  * get_epoch_day:
  * @date: string representation of calendar date, in form
- * YY[YY]/MM/DD.
+ * YY[YY]-MM-DD.
  * 
  * Returns: the epoch day number, or -1 on failure.
  */
 
 long get_epoch_day (const char *date)
 {
-    long temp;
-    int year, month, day;
+    int nf, year, month, day;
+    long ret;
 
-    if (sscanf(date, "%d/%d/%d", &year, &month, &day) != 3) {
-	return -1;
+    nf = sscanf(date, YMD_READ_FMT, &year, &month, &day);
+
+    if (nf != 3) {
+	/* backward compatibility: try slashes instead */
+	nf = sscanf(date, "%d/%d/%d", &year, &month, &day);
     }
 
-    if (year < 0 || month < 0 || day < 0) {
+    if (nf != 3 || year < 0 || month < 0 || day < 0) {
 	return -1;
     }
 
@@ -148,10 +151,10 @@ long get_epoch_day (const char *date)
 	year = FOUR_DIGIT_YEAR(year);
     }
 
-    temp = (long)(year - 1) * 365 + leap_years_since_year_1(year - 1)
+    ret = (long)(year - 1) * 365 + leap_years_since_year_1(year - 1)
 	+ day_in_year(day, month, year);
 
-    return temp;
+    return ret;
 }
 
 /**
@@ -252,9 +255,9 @@ void calendar_date_string (char *str, int t, const DATASET *dset)
     day = rem - modays;
 
     if (strlen(dset->stobs) == 8) {
-	sprintf(str, "%02d/%02d/%02d", yr % 100, mo, day);
+	sprintf(str, YMD_WRITE_Y2_FMT, yr % 100, mo, day);
     } else {
-	sprintf(str, "%04d/%02d/%02d", yr, mo, day);
+	sprintf(str, YMD_WRITE_Y4_FMT, yr, mo, day);
     }
 }
 
@@ -358,7 +361,7 @@ int MS_excel_date_string (char *date, int mst, int pd, int d1904)
 
 	sprintf(date, "%d:%d", yr, qtr);
     } else {
-	sprintf(date, "%04d/%02d/%02d", yr, mo, day);
+	sprintf(date, YMD_WRITE_Y4_FMT, yr, mo, day);
     }
 
     return 0;
@@ -378,15 +381,23 @@ double get_dec_date (const char *date)
     long ed0, edn, edt;
     double dyr, frac;
 
-    if (sscanf(date, "%d/%d/%d", &yr, &mo, &day) != 3) {
+    if (sscanf(date, YMD_READ_FMT, &yr, &mo, &day) != 3) {
 	return NADBL;
     }
 
+#if USE_ISO_8601
+    edt = get_epoch_day(date);
+    sprintf(tmp, "%04d-01-01", yr);
+    ed0 = get_epoch_day(tmp);
+    sprintf(tmp, "%04d-12-31", yr);
+    edn = get_epoch_day(tmp);
+#else
     edt = get_epoch_day(date);
     sprintf(tmp, "%04d/01/01", yr);
     ed0 = get_epoch_day(tmp);
     sprintf(tmp, "%04d/12/31", yr);
     edn = get_epoch_day(tmp);
+#endif
 
     if (yr < 100) {
 	yr = FOUR_DIGIT_YEAR(yr);
@@ -469,7 +480,7 @@ int get_day_of_week (const char *date)
 {
     int yr, mo, day;
 
-    if (sscanf(date, "%d/%d/%d", &yr, &mo, &day) != 3) {
+    if (sscanf(date, YMD_READ_FMT, &yr, &mo, &day) != 3) {
 	return -1;
     }
 
