@@ -58,38 +58,6 @@ struct COMPARE {
     const int *testvars; /* list of variables added or omitted */
 };
 
-/* Critical values for Quandt likelihood ratio (break) test:
-   columns contain the 10%, 5% and 1% values respectively,
-   for the given number of restrictions, q, and 15% trimming.  
-   Taken from Stock and Watson, Introduction to Econometrics
-   (Addison-Wesley, 2003), based on Andrews.
-*/
-
-#define QLR_QMAX 20
-
-static double QLR_critvals[QLR_QMAX][3] = {
-    { 7.12, 8.68, 12.16 }, /*  1 */
-    { 5.00, 5.86,  7.78 }, /*  2 */
-    { 4.09, 4.71,  6.02 }, /*  3 */
-    { 3.59, 4.09,  5.12 }, /*  4 */
-    { 3.26, 3.66,  4.53 }, /*  5 */
-    { 3.02, 3.37,  4.12 }, /*  6 */
-    { 2.84, 3.15,  3.82 }, /*  7 */
-    { 2.69, 2.98,  3.57 }, /*  8 */
-    { 2.58, 2.84,  3.38 }, /*  9 */
-    { 2.48, 2.71,  3.23 }, /* 10 */
-    { 2.40, 2.62,  3.09 }, /* 11 */
-    { 2.33, 2.54,  2.97 }, /* 12 */
-    { 2.27, 2.46,  2.87 }, /* 13 */
-    { 2.21, 2.40,  2.78 }, /* 14 */
-    { 2.16, 2.34,  2.71 }, /* 15 */
-    { 2.12, 2.29,  2.64 }, /* 16 */
-    { 2.08, 2.25,  2.58 }, /* 17 */
-    { 2.05, 2.20,  2.53 }, /* 18 */
-    { 2.01, 2.17,  2.48 }, /* 19 */
-    { 1.99, 2.13,  2.43 }  /* 20 */
-};
-
 /* Given a list of variables, check them against the independent
    variables included in a model, and construct a mask with 1s in
    positions where there is a match, 0s otherwise.  If the test list
@@ -290,22 +258,38 @@ static int add_diffvars_to_test (ModelTest *test, const int *list,
     return err;
 }
 
-static void print_add_omit_null (const int *list, const DATASET *dset,
-				 PRN *prn)
+void print_add_omit_null (const int *list, const DATASET *dset,
+			  gretlopt opt, PRN *prn)
 {
-    if (list[0] == 1) {
+    int nl = list == NULL ? 0 : list[0];
+
+    if (nl == 1 && !(opt & OPT_S)) {
+	/* single parameter, not in system context */
 	pputs(prn, "\n  ");
 	pprintf(prn, _("Null hypothesis: the regression parameter is zero for %s"), 
 		dset->varname[list[1]]);
 	pputc(prn, '\n');
+    } else if (nl == 0) {
+	/* VAR omit specials */
+	if ((opt & OPT_E) && (opt & OPT_T)) {
+	    pprintf(prn, "\n  %s: %s\n", _("Null hypothesis"),
+		    _("no seasonal effects or trend"));
+	} else if (opt & OPT_E) {
+	    pprintf(prn, "\n  %s: %s\n", _("Null hypothesis"),
+		    _("no seasonal effects"));
+	} else if (opt & OPT_T) {
+	    pprintf(prn, "\n  %s: %s\n", _("Null hypothesis"),
+		    _("no trend"));
+	}
     } else {
+	/* all other cases */
 	const char *vname;
 	int i, nc = 0;
 
 	pputs(prn, _("\n  Null hypothesis: the regression parameters are "
 		     "zero for the variables\n"));
-	pputs(prn, "    ");
 
+	pputs(prn, "    ");
 	for (i=1; i<=list[0]; i++) {
 	    vname = dset->varname[list[i]];
 	    nc += strlen(vname) + 2;
@@ -320,6 +304,14 @@ static void print_add_omit_null (const int *list, const DATASET *dset,
 	    }
 	}
 	pputc(prn, '\n');
+	if (opt & OPT_E) {
+	    /* seasonals */
+	    pprintf(prn, "    %s\n", _("seasonal dummies"));
+	}
+	if (opt & OPT_T) {
+	    /* trend */
+	    pputs(prn, "    time\n");
+	}	
     }
 }
 
@@ -356,7 +348,7 @@ static void print_compare (struct COMPARE *cmp,
 	pputc(prn, '\n');
     } 
 
-    print_add_omit_null(cmp->testvars, dset, prn);
+    print_add_omit_null(cmp->testvars, dset, OPT_NONE, prn);
 
     if (cmp->stat == GRETL_STAT_WALD_CHISQ) {
 	pprintf(prn, "  %s: %s(%d) = %g, %s %g\n",  _("Wald test"),
@@ -692,7 +684,7 @@ static MODEL replicate_estimator (const MODEL *orig, int *list,
     int order = 0;
     int first = 1;
 
-    gretl_model_init(&rep);
+    gretl_model_init(&rep, dset);
 
     /* recreate options and auxiliary vars, if required */
 
@@ -1087,7 +1079,7 @@ static MODEL LM_add_test (MODEL *pmod, DATASET *dset, int *list,
 
     err = add_residual_to_dataset(pmod, dset);
     if (err) {
-	gretl_model_init(&aux);
+	gretl_model_init(&aux, dset);
 	aux.errcode = err;
 	return aux;
     }
@@ -1401,7 +1393,7 @@ static MODEL auto_omit (MODEL *orig, const int *omitlist,
     int i, drop;
     int err = 0;
 
-    gretl_model_init(&omod);
+    gretl_model_init(&omod, dset);
 
     tmplist = gretl_list_copy(orig->list);
     if (tmplist == NULL) {
@@ -1679,7 +1671,7 @@ double get_DW_pvalue_for_model (const MODEL *pmod, DATASET *dset,
 	return NADBL;
     }
 
-    gretl_model_init(&dwmod);
+    gretl_model_init(&dwmod, dset);
 
     /* impose the sample range used for the original model */ 
     impose_model_smpl(pmod, dset);
@@ -1748,7 +1740,7 @@ int reset_test (MODEL *pmod, DATASET *dset,
     use_square = !(opt & OPT_C); /* not cubes-only */
     use_cube = !(opt & OPT_R);   /* not squares-only */
 
-    gretl_model_init(&aux);
+    gretl_model_init(&aux, dset);
 
     if (opt & OPT_R) {
 	addv = 1;
@@ -1988,7 +1980,7 @@ static int ivreg_autocorr_test (MODEL *pmod, int order,
     /* impose original sample range */
     impose_model_smpl(pmod, dset);
 
-    gretl_model_init(&aux);
+    gretl_model_init(&aux, dset);
 
     if (order <= 0) {
 	order = dset->pd;
@@ -2195,7 +2187,7 @@ int autocorr_test (MODEL *pmod, int order, DATASET *dset,
 #endif
     }
 
-    gretl_model_init(&aux);
+    gretl_model_init(&aux, dset);
 
     if (order <= 0) {
 	order = dset->pd;
@@ -2418,35 +2410,8 @@ make_chow_list (const MODEL *pmod, DATASET *dset,
     return chowlist;
 }
 
-static double QLR_get_critval (double Fmax, int dfn, int *a, int *approx)
-{
-    double crit = 0.0;
-    int i, j = dfn - 1;
-
-    *approx = 0;
-
-    if (j >= QLR_QMAX) {
-	j = QLR_QMAX - 1;
-	*approx = 1;
-    } 
-
-    for (i=2; i>=0; i--) {
-	if (Fmax > QLR_critvals[j][i]) {
-	    *a = (i == 2)? 1 : (i == 1)? 5 : 10;
-	    crit = QLR_critvals[j][i];
-	    break;
-	}
-    }
-
-    if (crit == 0.0) {
-	crit = QLR_critvals[j][0];
-    }
-
-    return crit;
-}
-
-static int QLR_graph (const double *Ft, int t1, int t2, 
-		      const DATASET *dset)
+static int QLR_graph (const double *testvec, int t1, int t2, 
+		      const DATASET *dset, int robust)
 {
     const double *x = gretl_plotx(dset);
     FILE *fp;
@@ -2463,9 +2428,10 @@ static int QLR_graph (const double *Ft, int t1, int t2,
 
     fprintf(fp, "plot \\\n"
 	    "'-' using 1:2 title '%s' w lines\n",
+	    robust ? _("Robust Wald test for break") : 
 	    _("Chow F-test for break"));
     for (t=t1; t<=t2; t++) {
-	fprintf(fp, "%g %g\n", x[t], Ft[t-t1]);
+	fprintf(fp, "%g %g\n", x[t], testvec[t-t1]);
     }
     fputs("e\n", fp);
 
@@ -2477,86 +2443,100 @@ static int QLR_graph (const double *Ft, int t1, int t2,
 }
 
 static void save_QLR_test (MODEL *pmod, const char *datestr,
-			   double Fmax, double crit, double alpha,
-			   int dfn, int dfd)
+			   double X2, double pval, int df)
 {
     ModelTest *test = model_test_new(GRETL_TEST_QLR);
 
     if (test != NULL) {
 	model_test_set_teststat(test, GRETL_STAT_SUP_WALD);
 	model_test_set_param(test, datestr);
-	model_test_set_value(test, Fmax);
-	model_test_set_crit_and_alpha(test, crit, alpha);
-	model_test_set_dfn(test, dfn);
-	model_test_set_dfd(test, dfd);
+	model_test_set_value(test, X2);
+	model_test_set_pvalue(test, pval);
+	model_test_set_dfn(test, df);
 	maybe_add_test_to_model(pmod, test);
     }	  
 }
 
+static double get_QLR_pval (double test, int df, int k1, int k2, 
+			    MODEL *pmod, PRN *prn)
+{
+    double (*qlr_asy_pvalue) (double, int, double);
+    void *handle;
+    double pi_1, pi_2, lam0;
+    double pval;
+
+    qlr_asy_pvalue = get_plugin_function("qlr_asy_pvalue", &handle);
+
+    if (qlr_asy_pvalue == NULL) {
+	fputs(I_("Couldn't load plugin function\n"), stderr);
+	return NADBL;
+    }
+
+    pi_1 = (k1 - pmod->t1 + 1) / (double) pmod->nobs; 
+    pi_2 = (k2 - pmod->t1 + 1) / (double) pmod->nobs;
+    lam0 = (pi_2*(1.0 - pi_1)) / (pi_1*(1.0 - pi_2));
+
+    pval = (*qlr_asy_pvalue) (test, df, lam0);
+
+    close_plugin(handle);
+
+    if (!na(pval)) {
+	pprintf(prn, _("Asymptotic p-value = %.4g for "
+		       "chi-square(%d) = %g"),
+		pval, df, test);
+	pputc(prn, '\n');
+#if 0
+	pprintf(prn, "(k1 = %d, k2 = %d, lambda0 = %g, pi0 = %g)\n", 
+		k1, k2, lam0, 1.0/(1 + sqrt(lam0)));
+#endif
+    }
+
+    return pval;
+}
+
 static void QLR_print_result (MODEL *pmod,
-			      double Fmax, int tmax, int dfn, int dfd,
-			      const DATASET *dset, gretlopt opt,
+			      double test, int tmax, 
+			      int k1, int k2, 
+			      int dfn, int dfd,
+			      const DATASET *dset, 
+			      gretlopt opt,
+			      int robust,
 			      PRN *prn)
 {
     char datestr[OBSLEN];
-    double crit = 0.0;
-    int a = 0, approx = 0;
+    double X2, pval;
 
     ntodate(datestr, tmax, dset);
 
     pputs(prn, _("Quandt likelihood ratio test for structural break at an "
 		 "unknown point,\nwith 15 percent trimming"));
     pputs(prn, ":\n\n");
-    pprintf(prn, _("The maximum F(%d, %d) = %g occurs "
-		   "at observation %s"), dfn, dfd, Fmax, datestr);
+    if (robust) {
+	pprintf(prn, _("The maximum Wald test = %g occurs "
+		       "at observation %s"), test, datestr);
+	X2 = test;
+    } else {
+	pprintf(prn, _("The maximum F(%d, %d) = %g occurs "
+		       "at observation %s"), dfn, dfd, test, datestr);
+	X2 = dfn * test;
+    }
     pputc(prn, '\n');
 
-    crit = QLR_get_critval(Fmax, dfn, &a, &approx);
+    pval = get_QLR_pval(X2, dfn, k1, k2, pmod, prn);
 
-    if (a > 0) {
-	pprintf(prn, _("Significant at the %d percent level "), a);
-	pprintf(prn, _("(%d%% critical value %s %.2f)"), a, 
-		(approx)? "<" : "=", crit);
-    } else {
-	if (approx) {
-	    pprintf(prn, _("10%% critical value for q = 20 is %.2f"),
-		    crit);
-	} else {
-	    pputs(prn, _("Not significant at the 10 percent level "));
-	    pprintf(prn, _("(10%% value = %.2f)"), crit);
-	}
-	a = 10;
-    }
-
-    pputs(prn, "\n\n");
-    pputs(prn, _("This statistic does not follow the standard "
-	  "F distribution;\ncritical values are from Stock and Watson "
-	  "(2003)."));
-    pputs(prn, "\n\n");
-	  
+    record_test_result(X2, pval, "QLR");
 
     if (opt & OPT_S) {
-	save_QLR_test(pmod, datestr, Fmax, crit, a / 100.0,
-		      dfn, dfd);
+	save_QLR_test(pmod, datestr, X2, pval, dfn);
     }
 }
 
-static double robust_chow_test (MODEL *pmod, const int *list,
-				int *err)
+static double robust_chow_test (MODEL *pmod, const int *testlist)
 {
-    double test = NADBL;
-    int *tlist;
+    double test = wald_omit_F(testlist, pmod);
 
-    tlist = gretl_list_diff_new(pmod->list, list, 2);
-
-    if (tlist == NULL) {
-	*err = E_ALLOC;
-    } else {
-	test = wald_omit_F(tlist, pmod);
-	if (!na(test)) {
-	    test *= tlist[0]; /* chi-square form */
-	}
-	free(tlist);
+    if (!na(test)) {
+	test *= testlist[0]; /* chi-square form */
     }
 
     return test;
@@ -2623,14 +2603,18 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
        original model was estimated */
     impose_model_smpl(pmod, dset);
 
-    gretl_model_init(&chow_mod);
+    gretl_model_init(&chow_mod, dset);
 
     if (QLR) {
-	/* 15 percent trimming */
-	split = pmod->t1 + 0.15 * pmod->nobs;
-	smax = pmod->t1 + 0.85 * pmod->nobs;
+	/* "15 percent trimming": exactly how this should be
+	   defined is debatable, but the following agrees
+	   with the strucchange package for R, when the
+	   Fstats() function is given "from=0.15, to=0.85".
+	*/
+	split = pmod->t1 + floor(0.15 * pmod->nobs) - 1;
+	smax = pmod->t1 + floor(0.85 * pmod->nobs) - 1;
     } else if (opt & OPT_D) {
-	/* Chow, using dummy */
+	/* Chow, using a predefined dummy */
 	dumv = chowparm;
     } else {
 	/* Chow, using break observation */
@@ -2647,36 +2631,57 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
 
     if (QLR) {
 	/* Quandt likelihood ratio */
-	double F, Fmax = 0.0;
-	double *Ft = NULL;
+	int robust = (pmod->opt & OPT_R);
+	gretlopt lsqopt = OPT_A;
+	double test, testmax = 0.0;
+	double *testvec = NULL;
+	int *testlist = NULL;
 	int dfn = 0, dfd = 0;
 	int tmax = 0;
 	int i, t;
 
 	if (gretl_in_gui_mode()) {
-	    Ft = malloc((smax - split + 1) * sizeof *Ft);
+	    testvec = malloc((smax - split + 1) * sizeof *testvec);
+	}
+
+	if (robust) {
+	    lsqopt |= OPT_R;
+	    testlist = gretl_list_diff_new(chowlist, pmod->list, 2);
+	    if (testlist == NULL) {
+		err = E_ALLOC;
+	    }
 	}
 	
-	for (t=split; t<=smax; t++) {
-	    chow_mod = lsq(chowlist, dset, OLS, OPT_A);
+	for (t=split; t<=smax && !err; t++) {
+	    chow_mod = lsq(chowlist, dset, OLS, lsqopt);
 	    if (chow_mod.errcode) {
 		err = chow_mod.errcode;
 		errmsg(err, prn);
 		break;
 	    }
 	    dfn = chow_mod.ncoeff - pmod->ncoeff;
-	    dfd = chow_mod.dfd;
-	    F = (pmod->ess - chow_mod.ess) * dfd / (chow_mod.ess * dfn);
-	    if (F > Fmax) {
-		tmax = t;
-		Fmax = F;
+	    if (robust) {
+		test = robust_chow_test(&chow_mod, testlist);
+#if 0
+		fprintf(stderr, "%d %g\n", t+1, test);
+#endif
+	    } else {
+		dfd = chow_mod.dfd;
+		test = (pmod->ess - chow_mod.ess) * dfd / (chow_mod.ess * dfn);
+#if 0
+		fprintf(stderr, "%d %g\n", t+1, test*dfn);
+#endif
 	    }
-	    if (Ft != NULL) {
-		Ft[t - split] = F;
+	    if (test > testmax) {
+		tmax = t;
+		testmax = test;
+	    }
+	    if (testvec != NULL) {
+		testvec[t - split] = test;
 	    }
 #if 0
-	    fprintf(stderr, "split at t=%d: F(%d,%d)=%g\n", t, 
-		    dfn, dfd, F);
+	    fprintf(stderr, "split at t=%d: F(%d,%d)=%g (X2=%g)\n", t, 
+		    dfn, dfd, F, test*dfn);
 	    fprintf(stderr, " pmod->ess = %g, chow_mod.ess = %g\n", 
 		    pmod->ess, chow_mod.ess);
 #endif
@@ -2687,23 +2692,28 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
 	}
 
 	if (!err) {
-	    QLR_print_result(pmod, Fmax, tmax, dfn, dfd, dset, opt, prn);
-	    record_test_result(Fmax, NADBL, "QLR");
-	    if (Ft != NULL) {
-		QLR_graph(Ft, split, smax, dset);
+	    QLR_print_result(pmod, testmax, tmax, split, smax, 
+			     dfn, dfd, dset, opt, robust, prn);
+	    if (testvec != NULL) {
+		QLR_graph(testvec, split, smax, dset, robust);
 	    }
 	}
 
-	if (Ft != NULL) {
-	    free(Ft);
-	}
+	free(testvec);
+	free(testlist);
     } else {
 	/* regular (or robust) Chow test */
 	int robust = (pmod->opt & OPT_R);
 	gretlopt lsqopt = OPT_A;
+	int *testlist = NULL;
 
 	if (robust) {
 	    lsqopt |= OPT_R;
+	    testlist = gretl_list_diff_new(chowlist, pmod->list, 2);
+	    if (testlist == NULL) {
+		err = E_ALLOC;
+		goto bailout;
+	    }
 	}
 
 	chow_mod = lsq(chowlist, dset, OLS, lsqopt);
@@ -2725,7 +2735,7 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
 	    }
 
 	    if (robust) {
-		test = robust_chow_test(&chow_mod, pmod->list, &err);
+		test = robust_chow_test(&chow_mod, testlist);
 		if (!na(test)) {
 		    pval = chisq_cdf_comp(dfn, test);
 		}
@@ -2773,6 +2783,7 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
 	    } 
 	}
 	clear_model(&chow_mod);
+	free(testlist);
     }
 
  bailout:
