@@ -719,7 +719,7 @@ static int compress_daily (DATASET *dset, int pd)
     int t, yr, mon, day;
 
     for (t=0; t<dset->n; t++) {
-	sscanf(dset->S[t], "%d/%d/%d", &yr, &mon, &day);
+	sscanf(dset->S[t], YMD_READ_FMT, &yr, &mon, &day);
 	if (pd == 1) {
 	    sprintf(dset->S[t], "%d", yr);
 	} else if (pd == 12) {
@@ -762,8 +762,8 @@ static void retransform_daily_dates (DATASET *dset)
     */
 
     for (t=0; t<dset->n; t++) {
-	sscanf(dset->S[t], "%d/%d/%d", &y, &d, &m);
-	sprintf(dset->S[t], "%d/%d/%d", d, m, y);
+	sscanf(dset->S[t], YMD_READ_FMT, &y, &d, &m);
+	sprintf(dset->S[t], YMD_WRITE_FMT, d, m, y);
     }
 }
 
@@ -784,7 +784,7 @@ static int transform_daily_dates (DATASET *dset, int dorder)
 	    n = sscanf(label, "%d%c%d%c%d", &mon, &s1, &day, &s2, &yr);
 	}
 	if (n == 5) {
-	    sprintf(label, "%02d/%02d/%02d", yr, mon, day);
+	    sprintf(label, YMD_WRITE_Y2_FMT, yr, mon, day);
 	} else {
 	    err = 1;
 	}
@@ -1490,12 +1490,31 @@ int import_obs_label (const char *s)
 	    !strcmp(tmp, "period"));    
 }
 
+static int join_wants_col_zero (csvdata *c, const char *s)
+{
+    const char *colname;
+    int i;
+
+    if (*s == '\0') {
+	return 0;
+    }
+
+    for (i=0; i<JOIN_MAXCOL; i++) {
+	colname = c->jspec->colnames[i];
+	if (colname != NULL && !strcmp(s, colname)) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 static void check_first_field (const char *line, csvdata *c, PRN *prn)
 {
     if (c->delim != ' ' && *line == c->delim) {
 	csv_set_blank_column(c);
     } else {
-	char field1[16];
+	char field1[OBSLEN];
 	int i = 0;
 
 	if (c->delim == ' ' && *line == ' ') line++;
@@ -1507,6 +1526,10 @@ static void check_first_field (const char *line, csvdata *c, PRN *prn)
 	field1[i] = '\0';
 	iso_to_ascii(field1);
 
+	if (joining(c) && join_wants_col_zero(c, field1)) {
+	    return;
+	}	
+
 	pprintf(prn, A_("   first field: '%s'\n"), field1);
 
 	if (import_obs_label(field1)) {
@@ -1691,7 +1714,7 @@ static int process_csv_obs (csvdata *c, int i, int t, int *miss_shown,
     } else if (csv_missval(c->str, i, t+1, miss_shown, prn)) {
 	c->dset->Z[i][t] = NADBL;
     } else {
-	c->dset->Z[i][t] = csv_atof(c, gretl_chopstr(c->str));
+	c->dset->Z[i][t] = csv_atof(c, gretl_strstrip(c->str));
     } 
 
     return err;
@@ -1947,9 +1970,10 @@ static int handle_join_varname (csvdata *c, int k, int *pj)
 #endif
 
     for (i=0; i<JOIN_MAXCOL; i++) {
+	/* find "wanted name" i */
 	colname = c->jspec->colnames[i];
 	if (colname == NULL || c->jspec->colnums[i] > 0) {
-	    /* not wanted, or already found */
+	    /* name not wanted, or already found */
 	    continue;
 	}
 	if (!strcmp(c->str, colname)) {
@@ -2085,6 +2109,7 @@ static int csv_varname_scan (csvdata *c, FILE *fp, PRN *prn, PRN *mprn)
 		break;
 	    }
 	}
+	fprintf(stderr, "numcount = %d\n", numcount);
 	err = E_DATA;
     }
 
@@ -2291,7 +2316,7 @@ static int csv_read_data (csvdata *c, FILE *fp, PRN *prn, PRN *mprn)
     pputs(mprn, A_("scanning for row labels and data...\n"));
     err = real_read_labels_and_data(c, fp, prn);
 
-    if (!err && csv_skip_column(c) && !rows_subset(c)) {
+    if (!err && csv_skip_column(c) && !rows_subset(c) && !joining(c)) {
 	c->markerpd = test_markers_for_dates(c->dset, &reversed,
 					     c->skipstr, prn);
 	if (reversed) {
@@ -2314,8 +2339,37 @@ static void print_csv_parsing_header (const char *fname, PRN *prn)
     }
 }
 
-static void csv_set_dataset_dimensions (csvdata *c)
+static int join_unique_columns (csvdata *c)
 {
+    const char **cnames = c->jspec->colnames;
+    int counted[JOIN_MAXCOL] = {0};
+    int i, j, ncols = 0;
+
+    for (i=0; i<JOIN_MAXCOL; i++) {
+	if (cnames[i] != NULL && counted[i] == 0) {
+	    counted[i] = 1;
+	    /* mark any duplicates as counted too */
+	    for (j=i+1; j<JOIN_MAXCOL; j++) {
+		if (cnames[j] != NULL && !strcmp(cnames[i], cnames[j])) {
+		    counted[j] = 1;
+		}
+	    }
+#if CDEBUG
+	    fprintf(stderr, "join_unique_columns: '%s'\n", cnames[i]);
+#endif
+	    ncols++;
+	}
+    }
+
+    return ncols;
+}
+
+static int csv_set_dataset_dimensions (csvdata *c)
+{
+    int err = 0;
+
+    c->dset->v = 0;
+
     if (rows_subset(c)) {
 	c->dset->n = n_from_row_mask(c);
     }
@@ -2326,22 +2380,28 @@ static void csv_set_dataset_dimensions (csvdata *c)
 	}
 	c->dset->v = c->ncols + 1;
     } else {
+	int cols_wanted, cols_present;
+
 	if (c->dset->n == 0) {
 	    c->dset->n = c->nrows - 1; /* allow for varnames row */
 	}
-	if (joining(c)) {
-	    int i;
 
-	    c->dset->v = 1;
-	    for (i=0; i<JOIN_MAXCOL; i++) {
-		if (c->jspec->colnames[i] != NULL) {
-		    c->dset->v += 1;
-		}
-	    }
+	cols_present = csv_skip_column(c) ? (c->ncols - 1) : c->ncols;
+
+	if (joining(c)) {
+	    cols_wanted = join_unique_columns(c);
 	} else if (cols_subset(c)) {
-	    c->dset->v = c->cols_list[0] + 1;
+	    cols_wanted = c->cols_list[0];
 	} else {
-	    c->dset->v = csv_skip_column(c) ? c->ncols : c->ncols + 1;
+	    cols_wanted = cols_present;
+	}
+
+	if (cols_wanted > cols_present) {
+	    gretl_errmsg_set(_("Invalid column specification"));
+	    err = E_DATA;
+	} else {
+	    /* allow for the constant */
+	    c->dset->v = cols_wanted + 1;
 	}
     }
 
@@ -2351,6 +2411,8 @@ static void csv_set_dataset_dimensions (csvdata *c)
 		c->dset->v, c->dset->n);
     }
 #endif
+
+    return err;
 }
 
 /*
@@ -2359,11 +2421,10 @@ static void csv_set_dataset_dimensions (csvdata *c)
  * @dset: dataset struct.
  * @cols: column specification.
  * @rows: row specification.
- * @keyspec: join key specification.
- * @cptr: optional location to grab CSV data struct.
+ * @join: specification pertaining to "join" command.
  * @opt: use OPT_N to force interpretation of data colums containing
- * strings as coded (non-numeric) values and not errors; for use of OPT_T see
- * the help for "append".
+ * strings as coded (non-numeric) values and not errors; for use of 
+ * OPT_T see the help for "append".
  * @prn: gretl printing struct (or NULL).
  * 
  * Open a Comma-Separated Values data file and read the data into
@@ -2491,7 +2552,11 @@ static int real_import_csv (const char *fname,
 	goto csv_bailout;
     }
 
-    csv_set_dataset_dimensions(c);
+    err = csv_set_dataset_dimensions(c);
+    if (err) {
+	err = E_DATA;
+	goto csv_bailout;
+    }	
 
     pprintf(mprn, A_("   number of variables: %d\n"), c->dset->v - 1);
     pprintf(mprn, A_("   number of non-blank lines: %d\n"), c->nrows);
@@ -2555,7 +2620,7 @@ static int real_import_csv (const char *fname,
 	    fseek(fp, datapos, SEEK_SET);
 	    err = csv_read_data(c, fp, prn, NULL);
 	}
-    }	
+    }
 
     if (popit) {
 	gretl_pop_c_numeric_locale();
@@ -2613,8 +2678,18 @@ static int real_import_csv (const char *fname,
 	for (i=1; i<c->dset->v; i++) {
 	    sprintf(c->dset->varname[i], "v%d", i);
 	}
-    } else if (fix_varname_duplicates(c->dset)) {
-	pputs(prn, A_("warning: some variable names were duplicated\n"));
+    } else {
+#if CDEBUG
+	int ii;
+
+	fprintf(stderr, "HERE, calling fix_varname_duplicates\n");
+	for (ii=0; ii<c->dset->v; ii++) {
+	    fprintf(stderr, " c->dset->varname[%d] = '%s'\n", ii, c->dset->varname[ii]);
+	}
+#endif	
+	if (fix_varname_duplicates(c->dset)) {
+	    pputs(prn, A_("warning: some variable names were duplicated\n"));
+	}
     }
 
     if (!joining(c)) {
@@ -2655,8 +2730,8 @@ static int real_import_csv (const char *fname,
  * @fname: name of CSV file.
  * @dset: dataset struct.
  * @opt: use OPT_N to force interpretation of data colums containing
- * strings as coded (non-numeric) values and not errors; for use of OPT_T see
- * the help for "append".
+ * strings as coded (non-numeric) values and not errors; for use of 
+ * OPT_T see the help for "append".
  * @prn: gretl printing struct (or NULL).
  * 
  * Open a Comma-Separated Values data file and read the data into
@@ -2718,6 +2793,14 @@ struct jr_row_ {
 
 typedef struct jr_row_ jr_row;
 
+struct obskey_ {
+    char *timefmt;
+    int keycol;
+    int m_means_q;
+};
+
+typedef struct obskey_ obskey;
+
 struct joiner_ {
     int n_rows;     /* number of rows in data table */
     int n_keys;     /* number of keys used (1 or 2) */
@@ -2727,13 +2810,13 @@ struct joiner_ {
     int *key_freq;  /* counts of occurrences of (primary) key values */
     int str_keys;   /* flag for string comparison of primary keys */
     int str_keys2;  /* flag for string comparison of secondary keys */
-    int auto_keys;  /* flag for extracting keys from obs markers */ 
     const int *l_keyno; /* for string comparison: list of ikey IDs in lhs dset */
     const int *r_keyno; /* for string comparison: list of okey IDs in rhs dset */
     AggrType aggr;    /* aggregation method for 1:n joining */
     int seqval;       /* aux. sequence number for aggregation */
     int auxcol;       /* aux. data column for aggregation */
     int valcol;       /* column of RHS dataset holding payload */
+    obskey auto_keys; /* struct to hold info on obs-based key(s) */
     DATASET *l_dset;  /* the main (left-hand) dataset */
     DATASET *r_dset;  /* the temporary CSV dataset */
 };
@@ -2753,8 +2836,6 @@ struct jr_filter_ {
 
 typedef struct jr_filter_ jr_filter;
 
-#define AUTO_KEYS 1 /* experimental */
-
 static void jr_filter_destroy (jr_filter *f)
 {
     if (f != NULL) {
@@ -2808,7 +2889,7 @@ static int join_row_wanted (DATASET *dset, int i,
 	}
 
 	if (sx == NULL || sy == NULL) {
-	    fprintf(stderr, "join: missing string in filtering\n");
+	    gretl_errmsg_set("join: missing string in filtering");
 	    *err = E_MISSDATA;
 	} else if (filter->op == B_EQ) {
 	    ret = (strcmp(sx, sy) == 0);
@@ -2828,7 +2909,7 @@ static int join_row_wanted (DATASET *dset, int i,
 	double y = filter->rhcol ? dset->Z[filter->rhcol][i] : filter->rhval;
 	
 	if (na(x) || na(y)) {
-	    fprintf(stderr, "join: found NAs in filtering\n");
+	    gretl_errmsg_set("join: found NAs in filtering");
 	    *err = E_MISSDATA;
 	} else if (filter->op == B_EQ) {
 	    ret = x == y;
@@ -2855,27 +2936,68 @@ static int join_row_wanted (DATASET *dset, int i,
     return ret;
 }
 
-/* Parse the obs string on row @i of the outer dataset and set the two
-   keys on row @j of the joiner struct, representing year and month or
+/* Parse the obs string on row @i of the outer dataset and set the
+   key(s) on row @j of the joiner struct, representing year and month or
    quarter. We get here only if we have verified that the obs strings
-   exist, and plausibly conform to the right pattern. The indices
-   @i and @j may not be equal if a filter is being used.
+   exist. The indices @i and @j may not be equal if a filter is being used.
 */
 
-static int read_outer_auto_keys (joiner *jr, int j,
-				 const DATASET *dset,
-				 int i)
+static int read_outer_auto_keys (joiner *jr, int j, int i)
 {
-    const char *s = dset->S[i];
-    int maj, min, err = 0;
-    char c;
+    char *tfmt = jr->auto_keys.timefmt;
+    int tcol = jr->auto_keys.keycol;
+    struct tm t = {0};
+    const char *s;
+    char *test;
+    int err = 0;
 
-    if (sscanf(s, "%d%c%d", &maj, &c, &min) == 3) {
-	jr->rows[j].n_keys = 2;
-	jr->rows[j].keyval = maj;
-	jr->rows[j].keyval2 = min;
+    if (tcol >= 0) {
+	/* using a specified string-valued column */
+	s = series_get_string_val(jr->r_dset, tcol, i);	
     } else {
+	/* using first-column observation strings */
+	s = jr->r_dset->S[i];
+    }
+
+    test = strptime(s, tfmt, &t);
+    if (test == NULL || *test != '\0') {
+	gretl_errmsg_sprintf("'%s' does not match the format '%s'", s, tfmt);
 	err = E_DATA;
+    } else if (calendar_data(jr->l_dset)) {
+	int y, m, d, eday;
+
+	y = t.tm_year + 1900;
+	m = t.tm_mon + 1;
+	d = t.tm_mday;
+	eday = epoch_day_from_ymd(y, m, d);
+	if (eday < 0) {
+	    gretl_errmsg_sprintf("'%s' is not a valid date", s);
+	    err = E_DATA;
+	} else {
+	    jr->rows[j].n_keys = 1;
+	    jr->rows[j].keyval = eday;
+	    jr->rows[j].keyval2 = 0;
+	}
+    } else {
+	int maj, min;
+
+	maj = t.tm_year + 1900;
+	min = t.tm_mon + 1;
+	if (jr->auto_keys.m_means_q) {
+	    /* using the gretl-specific "%q" conversion */
+	    if (min > 4) {
+		gretl_errmsg_sprintf("'%s' is not a valid date", s);
+		err = E_DATA;
+	    }
+	} else if (jr->l_dset->pd == 4) {
+	    /* map from month on right to quarter on left */
+	    min = (int) ceil(min / 3.0);
+	}
+	if (!err) {
+	    jr->rows[j].n_keys = 2;
+	    jr->rows[j].keyval = maj;
+	    jr->rows[j].keyval2 = min;
+	}
     }
 
     return err;
@@ -2911,22 +3033,28 @@ static int verify_filter (jr_filter *filter, int lhcol, int rhcol,
 /* get an integer key value from a double, checking for
    pathology */
 
-static int dtoi (double x, int *err)
+static int dtoi (double x, int obs, int *err)
 {
     if (xna(x) || fabs(x) > INT_MAX) {
-	*err = E_INVARG;
+	if (obs > 0) {
+	    gretl_errmsg_sprintf("join: invalid outer key value on row %d",
+				 obs);
+	}
+	*err = E_DATA;
 	return -1;
     } else {
 	return (int) trunc(x);
     }
 }
 
+#define using_auto_keys(j) (j->auto_keys.timefmt != NULL)
+
 static joiner *joiner_new (csvjoin *jspec, 
 			   DATASET *l_dset,
 			   jr_filter *filter,
 			   AggrType aggr,
 			   int seqval,
-			   int auto_keys,
+			   obskey *auto_keys,
 			   int *err)
 {
     joiner *jr = NULL;
@@ -2958,15 +3086,19 @@ static joiner *joiner_new (csvjoin *jspec,
 
     if (*err) {
 	return NULL;
-    }
+    }      
 
     if (filter != NULL) {
 	/* count the filtered rows */
-	for (i=0; i<r_dset->n; i++) {
+	for (i=0; i<r_dset->n && !*err; i++) {
 	    if (join_row_wanted(r_dset, i, filter, err)) {
 		nrows++;
 	    }
 	}
+	if (*err) {
+	    free(jr);
+	    return NULL;
+	}  	
     } else {
 	nrows = r_dset->n;
     }
@@ -2990,7 +3122,7 @@ static joiner *joiner_new (csvjoin *jspec,
 	jr->valcol = valcol;
 	jr->l_dset = l_dset;
 	jr->r_dset = r_dset;
-	jr->auto_keys = auto_keys;
+	jr->auto_keys = *auto_keys;
     }
 
     if (jr != NULL) {
@@ -3001,14 +3133,14 @@ static joiner *joiner_new (csvjoin *jspec,
 	for (i=0; i<r_dset->n && !*err; i++) {
 	    if (join_row_wanted(r_dset, i, filter, err)) {
 		/* the keys */
-		if (auto_keys) {
-		    *err = read_outer_auto_keys(jr, j, r_dset, i);
+		if (using_auto_keys(jr)) {
+		    *err = read_outer_auto_keys(jr, j, i);
 		} else if (keycol > 0) {
-		    jr->rows[j].keyval = dtoi(Z[keycol][i], err);
+		    jr->rows[j].keyval = dtoi(Z[keycol][i], i+1, err);
 		    if (key2col > 0) {
 			/* double key */
 			jr->rows[j].n_keys = 2;
-			jr->rows[j].keyval2 = dtoi(Z[key2col][i], err);
+			jr->rows[j].keyval2 = dtoi(Z[key2col][i], i+1, err);
 		    } else {
 			/* single key */
 			jr->rows[j].n_keys = 1;
@@ -3372,23 +3504,28 @@ static double maybe_adjust_string_code (series_table *rst,
     return rz;
 }
 
-static int get_inner_keys (joiner *jr, int i,
-			   const int *ikeyvars,
-			   int *pk1, int *pk2,
-			   int *missing)
+static int get_inner_key_values (joiner *jr, int i,
+				 const int *ikeyvars,
+				 int *pk1, int *pk2,
+				 int *missing)
 {
     DATASET *dset = jr->l_dset;
     int err = 0;
 
     *pk1 = *pk2 = 0;
 
-    if (jr->auto_keys) {
-	/* real-time data special: use the LHS obs info */
-	char obs[8];
+    if (using_auto_keys(jr)) {
+	/* using the LHS dataset obs info */
+	char obs[12];
 
 	ntodate(obs, i, dset);
-	*pk1 = atoi(obs);
-	*pk2 = atoi(obs + 5);
+	if (calendar_data(dset)) {
+	    *pk1 = get_epoch_day(obs);
+	} else {
+	    /* monthly or quarterly (FIXME others?) */
+	    *pk1 = atoi(obs);
+	    *pk2 = atoi(obs + 5);
+	}
     } else {
 	/* using regular LHS key series */
 	double dk1, dk2 = 0;
@@ -3401,9 +3538,9 @@ static int get_inner_keys (joiner *jr, int i,
 	if (xna(dk1) || xna(dk2)) {
 	    *missing = 1;
 	} else {
-	    k1 = dtoi(dk1, &err);
+	    k1 = dtoi(dk1, 0, &err);
 	    if (!err && jr->n_keys == 2) {
-		k2 = dtoi(dk2, &err);
+		k2 = dtoi(dk2, 0, &err);
 	    }
 	}
 	if (!err && !*missing) {
@@ -3480,7 +3617,7 @@ static int aggregate_data (joiner *jr, const int *ikeyvars, int v)
 	int missing = 0;
 	double z;
 
-	err = get_inner_keys(jr, i, ikeyvars, &key, &key2, &missing);
+	err = get_inner_key_values(jr, i, ikeyvars, &key, &key2, &missing);
 	if (err) {
 	    break;
 	} else if (missing) {
@@ -3617,7 +3754,7 @@ static jr_filter *make_join_filter (const char *s,
 	if (numeric_string(lhs)) {
 	    filter->lhval = dot_atof(lhs);
 	} else if (gretl_is_scalar(lhs)) {
-	    filter->lhval = gretl_scalar_get_value(lhs);
+	    filter->lhval = gretl_scalar_get_value(lhs, NULL);
 	} else {
 	    filter->lhname = lhs;
 	    lhs = NULL;
@@ -3625,7 +3762,7 @@ static jr_filter *make_join_filter (const char *s,
 	if (numeric_string(rhs)) {
 	    filter->rhval = dot_atof(rhs);
 	} else if (gretl_is_scalar(rhs)) {
-	    filter->rhval = gretl_scalar_get_value(rhs);
+	    filter->rhval = gretl_scalar_get_value(rhs, NULL);
 	} else {
 	    filter->rhname = rhs;
 	    rhs = NULL;
@@ -3668,15 +3805,21 @@ static int get_target_varnum (const char *vname,
    @s. If @s contains a comma, we accept a zero-length name on either
    the left or the right -- but not both -- as indicating that we
    should use the corresponding inner key name.
+
+   If opt contains OPT_K (--tkey) we assume there's just one 
+   column name here.
 */
 
 static int process_outer_key (const char *s, int n_keys, 
-			      char *name1, char *name2)
+			      char *name1, char *name2,
+			      gretlopt opt)
 {
     int n_okeys = 0;
     int err = 0;
 
-    if (strchr(s, ',') == NULL) {
+    if (opt & OPT_K) {
+	strncat(name1, s, CSVSTRLEN - 1);
+    } else if (strchr(s, ',') == NULL) {
 	/* just one outer key */
 	strncat(name1, s, CSVSTRLEN - 1);
 	n_okeys = 1;
@@ -3776,70 +3919,104 @@ static int join_data_type_check (joiner *jr, int targvar,
     return err;
 }
 
-#if AUTO_KEYS
+/* Handle the case where the user gave a "%q" conversion specifier
+   (which we take to mean quarter). We convert this to %m for use with
+   strptime(), but record that the fact that "month means quarter".
+*/
 
-static int subperiod_char (char c, int pd)
+static void check_for_quarterly (obskey *auto_keys)
 {
-    if (pd == 12 && (c == 'm' || c == 'M')) {
-	return 1;
-    } else if (pd == 4 && (c == 'q' || c == 'Q')) {
-	return 1;
-    } else {
-	return 0;
+    char *s = auto_keys->timefmt;
+    int i;
+
+    for (i=0; s[i]; i++) {
+	if (s[i] == '%' && s[i+1] == 'q' && (i == 0 || s[i-1] != '%')) {
+	    s[i+1] = 'm';
+	    auto_keys->m_means_q = 1;
+	}
     }
 }
 
-/* Real-time data, no keys supplied: can we figure out keys
-   using the time-series information on the left plus regimented
-   observation marker string on the right, of the form "1990q1" for
-   quarterly data or "1990m1" for monthly? Maybe.
-*/
+/* time-series data on the left, and no explicit keys supplied */
 
 static int auto_keys_check (const DATASET *l_dset,
-			    const DATASET *r_dset)
+			    const DATASET *r_dset,
+			    gretlopt opt,
+			    obskey *auto_keys,
+			    int *n_keys)
 {
-    const char *dig = "0123456789";
-    const char *s;
     int pd = l_dset->pd;
     int err = 0;
 
-    if (!dataset_is_time_series(l_dset) || l_dset->S != NULL) {
-	return E_DATA;
-    } else if (pd != 4 && pd != 12) {
-	return E_PDWRONG;
+    if (!dataset_is_time_series(l_dset)) {
+	/* On the left we need a time-series dataset */
+	err = E_DATA;
+	goto bailout;
     }
 
-    if (r_dset->S == NULL) {
+    if (r_dset->S == NULL && auto_keys->keycol < 0) {
+	/* On the right, we need either obs strings or a specified
+	   time column */
 	err = E_DATA;
-    } else {
-	int tmax = 12; /* arbitrary sample */
-	int n, t;
+	goto bailout;
+    }
 
-	if (tmax > r_dset->n) { 
-	    tmax = r_dset->n;
-	}	
+    if (opt & OPT_T) {
+	/* the user should have supplied a time-format spec */
+	const char *s = get_optval_string(JOIN, OPT_T);
 
-	for (t=0; t<tmax && !err; t++) {
-	    s = r_dset->S[t];
-	    n = strlen(s);
-	    if ((n == 6 || n == 7) && strspn(s, dig) == 4 && 
-		subperiod_char(s[4], pd) &&
-		strspn(s + 5, dig) == n - 5) {
+	if (s == NULL) {
+	    err = E_DATA;
+	} else {
+	    auto_keys->timefmt = gretl_strdup(s);
+	    if (auto_keys->timefmt == NULL) {
+		err = E_ALLOC;
 	    } else {
-		err = E_DATA;
+		if (pd == 4) {
+		    check_for_quarterly(auto_keys);
+		}
+		if (calendar_data(l_dset)) {
+		    *n_keys = 1;
+		} else {
+		    *n_keys = 2;
+		}
 	    }
 	}
+    } else {
+	/* ISO 8601 defaults */
+	if (calendar_data(l_dset)) {
+	    auto_keys->timefmt = gretl_strdup("%Y-%m-%d");
+	    if (auto_keys->timefmt == NULL) {
+		err = E_ALLOC;
+	    } else {
+		*n_keys = 1;
+	    }
+	} else if (pd == 12) {
+	    auto_keys->timefmt = gretl_strdup("%Y-%m");
+	    if (auto_keys->timefmt == NULL) {
+		err = E_ALLOC;
+	    } else {
+		*n_keys = 2;
+	    }	    
+	} else {
+	    err = E_PDWRONG;
+	}
     }
 
-#if CDEBUG
-    fprintf(stderr, "realtime_key_check: err = %d\n", err);
-#endif
+ bailout:
+    
+    /* we should flag an error here only if the user
+       explicitly requested use of this apparatus,
+       by giving --time=<format> (that is, OPT_T)
+       and/or --tkey=<colname> (OPT_K)
+    */
+    if (err && !(opt & (OPT_T | OPT_K))) {
+	err = 0;
+    }
 
     return err;
 }
 
-#endif /* AUTO_KEYS */
-
 /**
  * join_from_csv:
  * @fname: name of delimited text data file.
@@ -3868,7 +4045,7 @@ int join_from_csv (const char *fname,
 		   const int *ikeyvars,
 		   const char *okey,
 		   const char *filtstr,
-		   const char *data,
+		   const char *dataname,
 		   AggrType aggr,
 		   int seqval,
 		   const char *auxname,
@@ -3881,10 +4058,10 @@ int join_from_csv (const char *fname,
     int okeyvars[3] = {0, 0, 0};
     char okeyname1[CSVSTRLEN] = {0};
     char okeyname2[CSVSTRLEN] = {0};
+    obskey auto_keys = {0};
     int targvar, orig_v = dset->v;
     int str_keys = 0;
     int str_keys2 = 0;
-    int auto_keys = 0;
     int n_keys = 0;
     int err = 0;
 
@@ -3898,14 +4075,17 @@ int join_from_csv (const char *fname,
 	n_keys = ikeyvars[0];
     }
 
+    auto_keys.timefmt = NULL;
+    auto_keys.keycol = -1;
+
 #if CDEBUG
     fputs("*** join_from_csv:\n", stderr);
     fprintf(stderr, " filename = '%s'\n", fname);
     fprintf(stderr, " target series name = '%s'\n", varname);
     if (n_keys > 0) {
-	fprintf(stderr, " inner key series = %d\n", ikeyvars[1]);
+	fprintf(stderr, " inner key series ID = %d\n", ikeyvars[1]);
 	if (n_keys == 2) {
-	    fprintf(stderr, " second inner key series = %d\n", ikeyvars[2]);
+	    fprintf(stderr, " second inner key series ID = %d\n", ikeyvars[2]);
 	}
     }
     if (okey != NULL) {
@@ -3921,15 +4101,15 @@ int join_from_csv (const char *fname,
     if (filtstr != NULL) {
 	fprintf(stderr, " filter = '%s'\n", filtstr);
     }    
-    if (data != NULL) {
-	fprintf(stderr, " source data series = '%s'\n", data);
-    } else {
-	fprintf(stderr, " source data series = '%s' (from inner varname)\n", 
+    if (dataname != NULL) {
+	fprintf(stderr, " source data series = '%s'\n", dataname);
+    } else if (aggr != AGGR_COUNT) {
+	fprintf(stderr, " source data series: assuming '%s' (from inner varname)\n", 
 		varname);
     }
-    fprintf(stderr, " aggregation = %d\n", aggr);
+    fprintf(stderr, " aggregation method = %d\n", aggr);
     if (auxname != NULL) {
-	fprintf(stderr, " aggr auxiliary col = '%s'\n", auxname);
+	fprintf(stderr, " aggr auxiliary column = '%s'\n", auxname);
     }
 #endif
 
@@ -3941,10 +4121,10 @@ int join_from_csv (const char *fname,
     }
 
     if (!err && okey != NULL) {
-	err = process_outer_key(okey, n_keys, okeyname1, okeyname2);
+	err = process_outer_key(okey, n_keys, okeyname1, okeyname2, opt);
 	if (err) {
 	    fprintf(stderr, "join: error %d processing outer key(s)\n", err);
-	} 
+	}
     }
 
     /* Below: fill out the array of required column names,
@@ -3962,8 +4142,8 @@ int join_from_csv (const char *fname,
 
 	/* the data or "payload" column */
 	if (aggr != AGGR_COUNT) {
-	    if (data != NULL) {
-		jspec.colnames[JOIN_VAL] = data;
+	    if (dataname != NULL) {
+		jspec.colnames[JOIN_VAL] = dataname;
 	    } else {
 		jspec.colnames[JOIN_VAL] = varname;
 	    }
@@ -3995,15 +4175,14 @@ int join_from_csv (const char *fname,
     }
 
     if (!err && aggr != AGGR_COUNT) {
-	/* run some sanity tests on the payload */
+	/* run some sanity tests on the payload (there's no need
+	   for a payload with AGGR_COUNT)
+	*/
 	int valcol = jspec.colnums[JOIN_VAL];
 
 	if (valcol == 0) {
-	    if (data != NULL) {
-		fprintf(stderr, "join: data column '%s' was not found\n", 
-			jspec.colnames[1]);
-		err = E_UNKVAR;
-	    }
+	    gretl_errmsg_sprintf("Series not found, '%s'", jspec.colnames[JOIN_VAL]);
+	    err = E_UNKVAR;
 	} else if (aggr != AGGR_NONE && aggr != AGGR_SEQ) {
 	    if (series_has_string_table(jspec.c->dset, valcol)) {
 		/* maybe this should just be a warning? */
@@ -4014,6 +4193,14 @@ int join_from_csv (const char *fname,
 	}
     }
 
+    if (!err && auxname != NULL && jspec.colnums[JOIN_AUX] == 0) {
+	/* an auxiliary variable was specified for aggr but was
+	   not found */
+	fprintf(stderr, "join: auxiliary column '%s' was not found\n", 
+		jspec.colnames[JOIN_AUX]);
+	err = E_UNKVAR;
+    }	
+
     if (!err && jspec.colnames[0] != NULL) {
 	/* check that outer key was found in the right-hand-side
 	   file, and is conformable to the inner key 
@@ -4032,12 +4219,25 @@ int join_from_csv (const char *fname,
 	    (jspec.colnames[JOIN_KEY2] != NULL && okeyvars[2] == 0)) {
 	    fprintf(stderr, "join: error finding outer key columns\n");
 	    err = E_DATA;
+	} else if (opt & OPT_K) {
+	    /* time key on right: should be string */
+	    int rstr = series_has_string_table(jspec.c->dset, okeyvars[1]);
+	    
+	    if (!rstr) {
+		err = E_TYPES;
+	    } else {
+		auto_keys.keycol = okeyvars[1];
+	    }
 	} else {
 	    int lstr = series_has_string_table(dset, ikeyvars[1]);
 	    int rstr = series_has_string_table(jspec.c->dset, okeyvars[1]);
 
 	    if (lstr != rstr) {
-		fprintf(stderr, "key 1: numeric/string mismatch\n");
+		if (lstr) {
+		    fprintf(stderr, "key 1: string on left but not on right\n");
+		} else {
+		    fprintf(stderr, "key 1: string on right but not on left\n");
+		}
 		err = E_TYPES; 
 	    } else if (lstr) {
 		str_keys = 1;
@@ -4057,15 +4257,11 @@ int join_from_csv (const char *fname,
 	}
     }
 
-#if AUTO_KEYS
-    if (!err && (opt & OPT_R) && n_keys == 0) {
-	err = auto_keys_check(dset, jspec.c->dset);
-	if (!err) {
-	    auto_keys = 1;
-	    n_keys = 2;
-	}
+    if (!err && n_keys == 0 && 
+	(dataset_is_time_series(dset) || (opt & OPT_T))) {
+	err = auto_keys_check(dset, jspec.c->dset, opt, 
+			      &auto_keys, &n_keys);
     }
-#endif
 
     if (!err) {
 #if CDEBUG > 1
@@ -4075,7 +4271,7 @@ int join_from_csv (const char *fname,
 	printdata(NULL, NULL, jspec.c->dset, OPT_O, eprn);
 	gretl_print_destroy(eprn);
 #endif
-	jr = joiner_new(&jspec, dset, filter, aggr, seqval, auto_keys, &err);
+	jr = joiner_new(&jspec, dset, filter, aggr, seqval, &auto_keys, &err);
 	if (err) {
 	    fprintf(stderr, "join: error %d from joiner_new()\n", err);
 	}
@@ -4137,6 +4333,10 @@ int join_from_csv (const char *fname,
 	dataset_drop_last_variables(dset, dset->v - orig_v);
     }
 
+    if (auto_keys.timefmt != NULL) {
+	free(auto_keys.timefmt);
+    }
+
     csvdata_free(jspec.c);
     joiner_destroy(jr);
     jr_filter_destroy(filter);
