@@ -58,7 +58,7 @@ int join_from_csv (const char *fname,
 		   const int *ikeyvars,
 		   const char *okey,
 		   const char *filter,
-		   const char *data,
+		   const char *dataname,
 		   AggrType agg,
 		   int seqval,
 		   const char *auxname,
