@@ -97,12 +97,13 @@ static int skip_esl_comments (FILE *fp, const char *str)
 {
     char word[MAXLEN];  /* should be big enough to accommodate
 			   strings among the comments? */
+    int n = 0;
 
     *word = '\0';
 
     if (strncmp(str, STARTCOMMENT, 2) == 0) {
         while (strcmp(word, ENDCOMMENT)) {
-            fscanf(fp, "%s", word);
+            n += fscanf(fp, "%s", word);
         }
         return 0;
     } 
@@ -205,11 +206,12 @@ static int read_esl_data (FILE *fp, const DATASET *dset, int byvar)
 	    }
 	    if (dset->markers) {
 		*marker = '\0';
-		fscanf(fp, sformat, marker);
-		if (*marker == '"' || *marker == '\'') {
-		    strcpy(dset->S[t], marker + 1);
-		} else {
-		    strcpy(dset->S[t], marker);
+		if (fscanf(fp, sformat, marker) == 1) {
+		    if (*marker == '"' || *marker == '\'') {
+			strcpy(dset->S[t], marker + 1);
+		    } else {
+			strcpy(dset->S[t], marker);
+		    }
 		}
 	    }
 	    for (i=1; i<dset->v; i++) {
@@ -271,13 +273,13 @@ int check_varname (const char *varname)
     if (testchar != 'a') {
 	if (isprint((unsigned char) testchar)) {
 	    if (ret == VARNAME_FIRSTCHAR) {
-		gretl_errmsg_sprintf(_("First char of varname ('%c') is bad\n"
+		gretl_errmsg_sprintf(_("First char of varname '%s' is bad\n"
 				       "(first must be alphabetical)"), 
-				     (unsigned char) testchar);
+				     varname);
 	    } else {
-		gretl_errmsg_sprintf(_("Varname contains illegal character '%c'\n"
+		gretl_errmsg_sprintf(_("Varname '%s' contains illegal character '%c'\n"
 				       "Use only letters, digits and underscore"), 
-				     (unsigned char) testchar);
+				     varname, (unsigned char) testchar);
 	    }
 	} else {
 	    if (ret == VARNAME_FIRSTCHAR) {
@@ -299,7 +301,7 @@ static int read_esl_hdr (const char *hdrfile, DATASET *dset, int *byvar)
 {
     FILE *fp;
     char str[MAXLEN], byobs[6], option[8];
-    int n, i, descrip = 0;
+    int n, i = 0, descrip = 0;
     int err = 0;
 
     gretl_error_clear();
@@ -310,8 +312,9 @@ static int read_esl_hdr (const char *hdrfile, DATASET *dset, int *byvar)
 	return E_FOPEN;
     }
 
-    fscanf(fp, "%s", str);
-    i = skip_esl_comments(fp, str); 
+    if (fscanf(fp, "%s", str)) {
+	i = skip_esl_comments(fp, str);
+    } 
 
     /* find the number of variables */
 
@@ -344,17 +347,23 @@ static int read_esl_hdr (const char *hdrfile, DATASET *dset, int *byvar)
     rewind(fp);
 
     *str = '\0';
-    fscanf(fp, "%s", str);
-    if (skip_esl_comments(fp, str)) {
-	*dset->varname[i] = '\0';
-	strncat(dset->varname[i], str, VNAMELEN - 1);
-	err = check_varname(dset->varname[i++]);
+    if (fscanf(fp, "%s", str)) {
+	if (skip_esl_comments(fp, str)) {
+	    *dset->varname[i] = '\0';
+	    strncat(dset->varname[i], str, VNAMELEN - 1);
+	    err = check_varname(dset->varname[i++]);
+	} else {
+	    descrip = 1; /* comments were found */
+	}
     } else {
-	descrip = 1; /* comments were found */
+	err = E_DATA;
     }
 
     while (!err) {
-        fscanf(fp, "%s", str);
+        if (fscanf(fp, "%s", str) == 0) {
+	    err = E_DATA;
+	    break;
+	}
 	n = strlen(str);
 	if (str[n-1] != ';') {
 	    *dset->varname[i] = '\0';
@@ -370,14 +379,20 @@ static int read_esl_hdr (const char *hdrfile, DATASET *dset, int *byvar)
 	}
     }
 
+    if (!err) {
+	n = 0;
+	n += fscanf(fp, "%d", &dset->pd);
+	n += fscanf(fp, "%s", dset->stobs);
+	n += fscanf(fp, "%s", dset->endobs);
+	if (n != 3) {
+	    err = E_DATA;
+	}
+    }
+
     if (err) {
 	goto bailout;
     }
 
-    fscanf(fp, "%d", &dset->pd);
-    fscanf(fp, "%s", dset->stobs);
-    fscanf(fp, "%s", dset->endobs);
-
     colonize_obs(dset->stobs);
     colonize_obs(dset->endobs);
 
@@ -993,33 +1008,40 @@ static int write_esl_hdr (const char *hdrfile, const int *list,
     return 0;
 }
 
-static int alt_get_dec_places (double zmin, double zmax, int d)
+static int alt_get_dec_places (const double *x, int n, int d)
 {
     char *s, numstr[32];
-    int d1 = 0, d2 = 0;
+    int dmin = 100, dmax = 0;
+    int dt, t;
 
-    /* see if we get a (close to) common number of decimal places if we
+    /* see if we get (close to) a common number of decimal places if we
        print to precision @d without forcing the inclusion of trailing
        zeros
     */
 
     gretl_push_c_numeric_locale();
 
-    sprintf(numstr, "%.*g", d, zmin);
-    s = strrchr(numstr, '.');
-    if (s != NULL && strchr(numstr, 'e') == NULL) {
-	d1 = strlen(s) - 1;
-	sprintf(numstr, "%.*g", d, zmax);
+    for (t=0; t<n; t++) {
+	sprintf(numstr, "%.*g", d, x[t]);
 	s = strrchr(numstr, '.');
 	if (s != NULL && strchr(numstr, 'e') == NULL) {
-	    d2 = strlen(s) - 1;
+	    dt = strlen(s) - 1;
+	    if (dt < dmin) {
+		dmin = dt;
+	    } 
+	    if (dt > dmax) {
+		dmax = dt;
+	    }
+	} else {
+	    dmin = dmax = 0;
+	    break;
 	}
     }
 
     gretl_pop_c_numeric_locale();
 
-    if (d1 > 0 && d2 > 0 && abs(d1 - d2) < 2) {
-	return d1 > d2 ? d1 : d2;
+    if (dmin > 0 && dmax > 0 && dmax - dmin < 2) {
+	return dmax;
     } else {
 	return 0;
     }
@@ -1070,7 +1092,7 @@ int get_precision (const double *x, int n, int placemax)
 
     if (placemax >= 10 && placemax < 24) {
 	/* interpret @placemax as number of significant digits */
-	p = alt_get_dec_places(zmin, zmax, placemax);
+	p = alt_get_dec_places(x, n, placemax);
 	if (p > 0) {
 	    return p;
 	}
@@ -1152,19 +1174,24 @@ format_from_opt_or_name (gretlopt opt, const char *fname,
     return fmt;
 }
 
-static void date_maj_min (int t, const DATASET *dset, int *maj, int *min)
+void date_maj_min (int t, const DATASET *dset, int *maj, int *min)
 {
     char obs[OBSLEN];
-    char *s;
 
     ntodate(obs, t, dset);
 
-    *maj = atoi(obs);
-    s = strchr(obs, ':');
-    if (s != NULL && strlen(s) > 1) {
-	*min = atoi(s + 1);
-    } else {
-	*min = 1;
+    if (maj != NULL) {
+	*maj = atoi(obs);
+    }
+
+    if (min != NULL) {
+	char *s = strchr(obs, ':');
+
+	if (s != NULL && strlen(s) > 1) {
+	    *min = atoi(s + 1);
+	} else {
+	    *min = 1;
+	}
     }
 }
 
@@ -1270,8 +1297,6 @@ static void R_data_out (const DATASET *dset, const int *list,
 
 #define DEFAULT_CSV_DIGITS 12
 
-#define annual_data(p) (p->structure == TIME_SERIES && p->pd == 1)
-
 /**
  * write_data:
  * @fname: name of file to write.
@@ -1363,7 +1388,7 @@ int write_data (const char *fname, int *list, const DATASET *dset,
 
     csv_digits = libset_get_int(CSV_DIGITS);
 
-    if (csv_digits == 0) {
+    if (csv_digits <= 0) {
 	/* the user has not overriden the default */
 	pmax = malloc(l0 * sizeof *pmax);
 	if (pmax != NULL) {
@@ -1702,7 +1727,7 @@ static int import_esl (const char *datafile, char *auxfile,
     if (!err) {
 	/* print out basic info from the files read */
 	pprintf(prn, I_("periodicity: %d, maxobs: %d\n"
-			"observations range: %s-%s\n"), tmpset->pd, tmpset->n,
+			"observations range: %s to %s\n"), tmpset->pd, tmpset->n,
 		tmpset->stobs, tmpset->endobs);
 
 	pputs(prn, I_("\nReading "));
@@ -1865,7 +1890,7 @@ int open_nulldata (DATASET *dset, int data_status, int length,
     if (gretl_messages_on()) {
 	/* print out basic info */
 	pprintf(prn, A_("periodicity: %d, maxobs: %d\n"
-			"observations range: %s-%s\n"), 
+			"observations range: %s to %s\n"), 
 		dset->pd, dset->n, dset->stobs, dset->endobs);
     }
 
@@ -2277,6 +2302,15 @@ static int merge_data (DATASET *dset, DATASET *addset,
 		newvar = 1;
 		strcpy(dset->varname[v], addset->varname[i]);
 		copy_varinfo(dset->varinfo[v], addset->varinfo[i]);
+		if (series_has_string_table(addset, i) &&
+		    addset->n == dset->n && offset == 0 &&
+		    addobs == 0) {
+		    series_table *st;
+
+		    st = series_get_string_table(addset, i);
+		    series_attach_string_table(dset, v, st);
+		    series_attach_string_table(addset, i, NULL);
+		}
 	    } 
 
 	    if (dayspecial) {
@@ -2371,6 +2405,25 @@ static int check_imported_string (char *src, int i, size_t len)
     return err;
 }
 
+static int count_markers (FILE *fp, char *line, int linelen,
+			  char *marker)
+{
+    int n = 0;
+
+    while (fgets(line, linelen, fp)) {
+	if (sscanf(line, "%31[^\n\r]", marker) == 1) {
+	    g_strstrip(marker);
+	    if (*marker != '\0') {
+		n++;
+	    }
+	}
+    }
+
+    rewind(fp);
+
+    return n;
+}
+
 /**
  * add_obs_markers_from_file:
  * @dset: data information struct.
@@ -2390,6 +2443,7 @@ int add_obs_markers_from_file (DATASET *dset, const char *fname)
     char **S = NULL;
     FILE *fp;
     char line[128], marker[32];
+    int done = 0;
     int t, err = 0;
 
     fp = gretl_fopen(fname, "r");
@@ -2402,19 +2456,50 @@ int add_obs_markers_from_file (DATASET *dset, const char *fname)
 	fclose(fp);
 	return E_ALLOC;
     }
-    
-    for (t=0; t<dset->n && !err; t++) {
-	if (fgets(line, sizeof line, fp) == NULL) {
-	    gretl_errmsg_sprintf("Expected %d markers; found %d\n", 
-				 dset->n, t);
-	    err = E_DATA;
-	} else if (sscanf(line, "%31[^\n\r]", marker) != 1) {
-	    gretl_errmsg_sprintf("Couldn't read marker on line %d", t+1);
-	    err = E_DATA;
-	} else {
-	    g_strstrip(marker);
-	    strncat(S[t], marker, OBSLEN - 1);
-	    err = check_imported_string(S[t], t+1, OBSLEN);
+
+    if (dataset_is_panel(dset)) {
+	/* allow the case where we get just enough markers to
+	   label the cross-sectional units */
+	int nm = count_markers(fp, line, sizeof line, marker);
+	int N = dset->n / dset->pd; /* = number of units */
+
+	if (nm == N) {
+	    int T = dset->pd;
+	    int t, i = 0;
+
+	    while (fgets(line, sizeof line, fp) && !err) {
+		*marker = '\0';
+		if (sscanf(line, "%31[^\n\r]", marker) == 1) {
+		    g_strstrip(marker);
+		    strncat(S[i], marker, OBSLEN - 1);
+		    err = check_imported_string(S[i], i+1, OBSLEN);
+		    if (!err) {
+			/* copy to remaining observations */
+			for (t=1; t<T; t++) {
+			    strcpy(S[i+t], S[i]);
+			}
+		    }
+		    i += T;
+		}
+	    }
+	    done = 1;
+	}
+    }
+
+    if (!done) {
+	for (t=0; t<dset->n && !err; t++) {
+	    if (fgets(line, sizeof line, fp) == NULL) {
+		gretl_errmsg_sprintf("Expected %d markers; found %d\n", 
+				     dset->n, t);
+		err = E_DATA;
+	    } else if (sscanf(line, "%31[^\n\r]", marker) != 1) {
+		gretl_errmsg_sprintf("Couldn't read marker on line %d", t+1);
+		err = E_DATA;
+	    } else {
+		g_strstrip(marker);
+		strncat(S[t], marker, OBSLEN - 1);
+		err = check_imported_string(S[t], t+1, OBSLEN);
+	    }
 	}
     }
 
