@@ -170,7 +170,12 @@ void clear_datainfo (DATASET *dset, int code)
     if (dset->padmask != NULL) {
 	free(dset->padmask);
 	dset->padmask = NULL;
-    }	
+    }
+
+    if (dset->pantime != NULL) {
+	free(dset->pantime);
+	dset->pantime = NULL;
+    }    
 
     /* if this is not a sub-sample datainfo, free varnames, labels, etc. */
 
@@ -409,6 +414,7 @@ void datainfo_init (DATASET *dset)
     dset->submask = NULL;
     dset->restriction = NULL;
     dset->padmask = NULL;
+    dset->pantime = NULL;
 
     dset->auxiliary = 0;
 }
@@ -601,6 +607,7 @@ int start_new_Z (DATASET *dset, gretlopt opt)
     dset->submask = NULL;
     dset->restriction = NULL;
     dset->padmask = NULL;
+    dset->pantime = NULL;
     
     return 0;
 }
@@ -1722,7 +1729,6 @@ int dataset_drop_listed_variables (int *list,
     int oldv = dset->v;
     int *dlist = NULL;
     int free_dlist = 0;
-    int lastvar[2];
     int err = 0;
 
     if (dset->n == 0 || dset->v == 0) {
@@ -1745,10 +1751,8 @@ int dataset_drop_listed_variables (int *list,
 	}
 	free_dlist = 1;
     } else if (list[0] == 0) {
-	/* signal to drop last variable */
-	lastvar[0] = 1;
-	lastvar[1] = dset->v - 1;
-	dlist = lastvar;
+	/* no-op */
+	return 0;
     } else {
 	dlist = list;
     }
@@ -2356,7 +2360,7 @@ static int get_stack_param_val (const char *s, const DATASET *dset)
 	*vname = '\0';
 	strncat(vname, s, len);
 	if (gretl_is_scalar(vname)) {
-	    val = gretl_scalar_get_value(vname);
+	    val = gretl_scalar_get_value(vname, NULL);
 	} else {
 	    i = series_index(dset, vname);
 	    if (i < dset->v) {
@@ -3957,3 +3961,39 @@ int steal_string_table (DATASET *l_dset, int lvar,
 
     return 0;
 }
+
+int dataset_set_panel_time (DATASET *dset, const double *tvals)
+{
+    int err = 0;
+
+    if (tvals == NULL) {
+	free(dset->pantime);
+	dset->pantime = NULL;
+    } else {
+	/* minimal sanity check: no NaNs, strictly increasing
+	   values */
+	double ti;
+	int i;
+	
+	for (i=0; i<dset->pd && !err; i++) {
+	    ti = tvals[i];
+	    if (xna(ti) || (i > 0 && ti <= tvals[i-1])) {
+		err = E_DATA;
+	    }
+	}
+	if (!err) {
+	    free(dset->pantime); 
+	    dset->pantime = copyvec(tvals, dset->pd);
+	    if (dset->pantime == NULL) {
+		err = E_ALLOC;
+	    }
+	}
+    }
+
+    return err;
+}
+
+const double *dataset_get_panel_time (const DATASET *dset)
+{
+    return (dset == NULL)? NULL : dset->pantime;
+}
