@@ -223,6 +223,16 @@ typedef struct _series_table series_table;
 #define quarterly_or_monthly(p) (p != NULL && p->structure == TIME_SERIES && \
                                  (p->pd == 4 || p->pd == 12))
 
+/**
+ * annual_data:
+ * @p: pointer to data information struct.
+ *
+ * Attempt to determine whether a data set is an annual
+ * time series (1), or something else (0).
+ */
+#define annual_data(p) (p != NULL && p->structure == TIME_SERIES && \
+			p->pd == 1)
+
 /**
  * dataset_is_panel:
  * @p: pointer to data information struct.
@@ -372,7 +382,7 @@ void series_set_linewidth (DATASET *dset, int i, int w);
 int series_get_linewidth (const DATASET *dset, int i);
 
 int series_record_display_name (DATASET *dset, int i,
-			  const char *s); 
+				const char *s); 
 
 int series_record_label (DATASET *dset, int i,
 			 const char *s); 
@@ -472,4 +482,8 @@ const char **series_get_string_vals (const DATASET *dset, int i,
 int steal_string_table (DATASET *l_dset, int lvar,
 			DATASET *r_dset, int rvar);
 
+int dataset_set_panel_time (DATASET *dset, const double *tvals);
+
+const double *dataset_get_panel_time (const DATASET *dset);
+
 #endif /* DATASET_H */
