@@ -507,7 +507,10 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
 	    strcpy(sinfo->varname, sername);
 	}
 
-	fgets(s2, sizeof s2, fp);
+	if (fgets(s2, sizeof s2, fp) == NULL) {
+	    err = DB_PARSE_ERROR;
+	    break;
+	}
 
 	if (gotit) {
 	    get_native_series_comment(sinfo, s1);
@@ -533,7 +536,7 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
 
     fclose(fp);
 
-    if (!gotit) {
+    if (!err && !gotit) {
 	gretl_errmsg_sprintf(_("Series not found, '%s'"), series);
 	err = DB_NO_SUCH_SERIES;
     }
@@ -700,7 +703,7 @@ get_pcgive_series_info (const char *series, SERIESINFO *sinfo)
 			int rem = MAXLABEL - strlen(sinfo->descrip) - 1;
 
 			if (rem > 0) {
-			    gretl_chopstr(line);
+			    gretl_strstrip(line);
 			    strncat(sinfo->descrip, line + 1, rem);
 			}
 		    } else {
@@ -898,24 +901,37 @@ static RECNUM read_rats_directory (FILE *fp, const char *series_name,
     RATSDirect rdir;
     DATEINFO dinfo;
     RECNUM ret;
+    int nread;
     int i, err = 0;
 
     memset(rdir.series_name, 0, NAMELENGTH);
 
-    fread(&rdir.back_point, sizeof(RECNUM), 1, fp);
-    fread(&rdir.forward_point, sizeof(RECNUM), 1, fp);
-    fseek(fp, 4L, SEEK_CUR); /* skip two shorts */
-    fread(&rdir.first_data, sizeof(RECNUM), 1, fp);
-    fread(rdir.series_name, NAMELENGTH, 1, fp);  
-    rdir.series_name[NAMELENGTH-1] = '\0';
-
-    gretl_chopstr(rdir.series_name);
+    if (fread(&rdir.back_point, sizeof(RECNUM), 1, fp) != 1) {
+	err = 1;
+    } else if (fread(&rdir.forward_point, sizeof(RECNUM), 1, fp) != 1) {
+	err = 1;
+    }
+    if (!err) {
+	fseek(fp, 4L, SEEK_CUR); /* skip two shorts */
+	if (fread(&rdir.first_data, sizeof(RECNUM), 1, fp) != 1) {
+	    err = 1;
+	} else if (fread(rdir.series_name, NAMELENGTH, 1, fp) != 1) {
+	    err = 1;
+	}
+    }
 
+    if (!err) {
+	rdir.series_name[NAMELENGTH-1] = '\0';
+	gretl_strstrip(rdir.series_name);
 #if DB_DEBUG
-    fprintf(stderr, "read_rats_directory: name='%s'\n", rdir.series_name);
+	fprintf(stderr, "read_rats_directory: name='%s'\n", rdir.series_name);
 #endif
+	if (!isprint(rdir.series_name[0])) {
+	    err = 1;
+	}
+    }
 
-    if (!isprint(rdir.series_name[0])) {
+    if (err) {
 	return RATS_PARSE_ERROR;
     }
 
@@ -928,13 +944,18 @@ static RECNUM read_rats_directory (FILE *fp, const char *series_name,
 
     /* skip long, short, long, short */
     fseek(fp, 12, SEEK_CUR);
-    fread(&dinfo.info, sizeof(gint32), 1, fp);
-    fread(&dinfo.digits, sizeof(short), 1, fp);
-    fread(&dinfo.year, sizeof(short), 1, fp);
-    fread(&dinfo.month, sizeof(short), 1, fp);
-    fread(&dinfo.day, sizeof(short), 1, fp);
+    nread = 0;
+    nread += fread(&dinfo.info, sizeof(gint32), 1, fp);
+    nread += fread(&dinfo.digits, sizeof(short), 1, fp);
+    nread += fread(&dinfo.year, sizeof(short), 1, fp);
+    nread += fread(&dinfo.month, sizeof(short), 1, fp);
+    nread += fread(&dinfo.day, sizeof(short), 1, fp);
+    nread += fread(&rdir.datapoints, sizeof(gint32), 1, fp);
+
+    if (nread != 6) {
+	return RATS_PARSE_ERROR;
+    }
 
-    fread(&rdir.datapoints, sizeof(gint32), 1, fp);
     fseek(fp, sizeof(short) * 4L, SEEK_CUR);  /* skip 4 shorts */
 
 #if DB_DEBUG
@@ -944,29 +965,37 @@ static RECNUM read_rats_directory (FILE *fp, const char *series_name,
     fprintf(stderr, "datapoints = %d\n", (int) rdir.datapoints);
 #endif
 
-    fread(&rdir.comment_lines, sizeof(short), 1, fp);
-    fseek(fp, 1L, SEEK_CUR); /* skip one char */
-
-    for (i=0; i<2; i++) {
-	if (i < rdir.comment_lines) {
-	    memset(rdir.comments[i], 0, 80);
-	    fread(rdir.comments[i], 80, 1, fp);
-	    rdir.comments[i][79] = '\0';
-	    gretl_chopstr(rdir.comments[i]);
-	} else {
-	    rdir.comments[i][0] = 0;
-	    fseek(fp, 80, SEEK_CUR);
+    if (fread(&rdir.comment_lines, sizeof(short), 1, fp) != 1) {
+	err = 1;
+    } else {
+	fseek(fp, 1L, SEEK_CUR); /* skip one char */
+	for (i=0; i<2 && !err; i++) {
+	    if (i < rdir.comment_lines) {
+		memset(rdir.comments[i], 0, 80);
+		err = (fread(rdir.comments[i], 80, 1, fp) != 1);
+		if (!err) {
+		    rdir.comments[i][79] = '\0';
+		    gretl_strstrip(rdir.comments[i]);
+		}
+	    } else {
+		rdir.comments[i][0] = 0;
+		fseek(fp, 80, SEEK_CUR);
+	    }
 	}
     }
 
 #if DB_DEBUG
-    fprintf(stderr, "comment_lines = %d\n", (int) rdir.comment_lines);
-    fprintf(stderr, "comment[0] = '%s'\n", rdir.comments[0]);
-    fprintf(stderr, "comment[1] = '%s'\n", rdir.comments[1]);
+    if (!err) {
+	fprintf(stderr, "comment_lines = %d\n", (int) rdir.comment_lines);
+	fprintf(stderr, "comment[0] = '%s'\n", rdir.comments[0]);
+	fprintf(stderr, "comment[1] = '%s'\n", rdir.comments[1]);
+    }
 #endif
 
-    err = dinfo_to_sinfo(&dinfo, sinfo, rdir.series_name, rdir.comments[0],
-			 rdir.datapoints, rdir.first_data);
+    if (!err) {
+	err = dinfo_to_sinfo(&dinfo, sinfo, rdir.series_name, rdir.comments[0],
+			     rdir.datapoints, rdir.first_data);
+    }
 
     ret = (err)? RATS_PARSE_ERROR : rdir.forward_point;
 
@@ -1110,7 +1139,7 @@ static int read_in7_series_info (FILE *fp, dbwrapper *dw)
 			int rem = MAXLABEL - strlen(desc) - 1;
 
 			if (rem > 0) {
-			    gretl_chopstr(line);
+			    gretl_strstrip(line);
 			    strncat(desc, line + 1, rem);
 			}
 			pos = ftell(fp);
@@ -1232,15 +1261,16 @@ dbwrapper *read_pcgive_db (const char *fname, FILE *fp)
 dbwrapper *read_rats_db (const char *fname, FILE *fp) 
 {
     dbwrapper *dw;
-    long forward;
+    long forward = 0;
     int i, err = 0;
 
     gretl_error_clear();
     
     /* get into position */
     fseek(fp, 30L, SEEK_SET); /* skip unneeded fields */
-    fread(&forward, sizeof forward, 1, fp);
-    fseek(fp, 4L, SEEK_CUR);
+    if (fread(&forward, sizeof forward, 1, fp) == 1) {
+	fseek(fp, 4L, SEEK_CUR);
+    }
 
     /* basic check */
     if (forward <= 0) { 
@@ -1304,9 +1334,10 @@ static int get_rats_series (int offset, SERIESINFO *sinfo, FILE *fp,
 {
     RATSData rdata;
     double x;
-    int miss = 0;
     int v = sinfo->v;
     int i, t, T;
+    int miss = 0;
+    int err = 0;
  
     fprintf(stderr, "get_rats_series: starting from offset %d\n", offset);
 
@@ -1322,7 +1353,10 @@ static int get_rats_series (int offset, SERIESINFO *sinfo, FILE *fp,
     while (rdata.forward_point) {
 	fseek(fp, (rdata.forward_point - 1) * 256L, SEEK_SET);
 	/* the RATSData struct is actually 256 bytes.  Yay! */
-	fread(&rdata, sizeof rdata, 1, fp);
+	if (fread(&rdata, sizeof rdata, 1, fp) != 1) {
+	    err = E_DATA;
+	    break;
+	}
 	for (i=0; i<31 && t<T; i++) {
 	    x = rdata.data[i];
 #if G_BYTE_ORDER == G_BIG_ENDIAN
@@ -1336,7 +1370,11 @@ static int get_rats_series (int offset, SERIESINFO *sinfo, FILE *fp,
 	}
     }
 
-    return miss;
+    if (miss && !err) {
+	err = DB_MISSING_DATA;
+    }
+
+    return err;
 }
 
 /**
@@ -1356,30 +1394,23 @@ int get_rats_db_data (const char *fname, SERIESINFO *sinfo,
 		      double **Z)
 {
     FILE *fp;
-    int ret = 0;
+    int err = 0;
 
     fp = gretl_fopen(fname, "rb");
     if (fp == NULL) {
-	return E_FOPEN;
-    }
-    
-#if DB_DEBUG
-    fprintf(stderr, "get_rats_db_data: offset = %d\n", sinfo->offset);
-#endif
-    
-    if (get_rats_series(sinfo->offset, sinfo, fp, Z)) {
-	ret = DB_MISSING_DATA; /* FIXME? */
+	err = E_FOPEN;
+    } else {
+	err = get_rats_series(sinfo->offset, sinfo, fp, Z);
+	fclose(fp);
     }
 
-    fclose(fp);
-
-    return ret;
+    return err;
 }
 
 static int get_rats_series_info (const char *series_name, SERIESINFO *sinfo)
 {
     FILE *fp;
-    long forward;
+    long forward = 0;
     int err = 0;
 
     gretl_error_clear();
@@ -1395,8 +1426,9 @@ static int get_rats_series_info (const char *series_name, SERIESINFO *sinfo)
 
     /* get into position */
     fseek(fp, 30L, SEEK_SET); 
-    fread(&forward, sizeof forward, 1, fp);
-    fseek(fp, 4L, SEEK_CUR);
+    if (fread(&forward, sizeof forward, 1, fp) == 1) {
+	fseek(fp, 4L, SEEK_CUR);
+    }
 
     /* basic check */
     if (forward <= 0) {
@@ -1677,7 +1709,7 @@ int set_db_name (const char *fname, int filetype, PRN *prn)
 	    fp = gretl_fopen(saved_db_name, "rb");
 	}
 
-#ifdef OSX_BUILD
+#ifdef OS_OSX
 	if (fp == NULL) {
 	    gchar *tmp = g_build_path(SLASHSTR, gretl_app_support_dir(), "db",
 				      fname, NULL);
@@ -3344,10 +3376,10 @@ static int get_n_ok_months (const DATASET *dset,
     int any_eop, any_sop, all_same;
     int skip = 0, pad = 0, nm = -1;
 
-    if (sscanf(dset->stobs, "%d/%d/%d", &y1, &m1, &d1) != 3) {
+    if (sscanf(dset->stobs, YMD_READ_FMT, &y1, &m1, &d1) != 3) {
 	return -1;
     }
-    if (sscanf(dset->endobs, "%d/%d/%d", &y2, &m2, &d2) != 3) {
+    if (sscanf(dset->endobs, YMD_READ_FMT, &y2, &m2, &d2) != 3) {
 	return -1;
     }
 
@@ -3440,7 +3472,7 @@ weeks_to_months_exec (double **mZ, const DATASET *dset,
     for (s=0; s<dset->n; s++) {
 	/* loop across the weekly obs in this month */
 	ntodate(obsstr, s, dset);
-	sscanf(obsstr, "%d/%d/%d", &yr, &mon, &day);
+	sscanf(obsstr, YMD_READ_FMT, &yr, &mon, &day);
 	if (monbak > 0 && mon != monbak) {
 	    /* new month: finalize the previous one */
 	    for (i=1; i<dset->v; i++) {
@@ -3512,7 +3544,7 @@ weeks_to_months_check (const DATASET *dset, int *startyr, int *endyr,
 
     for (t=0; t<dset->n; t++) {
 	ntodate(obsstr, t, dset);
-	if (sscanf(obsstr, "%d/%d/%d", &yr, &mon, &day) != 3) {
+	if (sscanf(obsstr, YMD_READ_FMT, &yr, &mon, &day) != 3) {
 	    err = 1;
 	    break;
 	}
@@ -3638,7 +3670,7 @@ static int daily_dataset_to_weekly (DATASET *dset, int repday)
 		n_ok++;
 	    }
 	    if (n == 0) {
-		sscanf(obs, "%d/%d/%d", &y1, &m1, &d1);
+		sscanf(obs, YMD_READ_FMT, &y1, &m1, &d1);
 	    }
 	    n++;
 	}
@@ -3686,7 +3718,7 @@ static int daily_dataset_to_weekly (DATASET *dset, int repday)
 	dset->n = n;
 	dset->pd = 52;
 	
-	sprintf(dset->stobs, "%04d/%02d/%02d", y1, m1, d1);
+	sprintf(dset->stobs, YMD_WRITE_Y4_FMT, y1, m1, d1);
 	dset->sd0 = get_date_x(dset->pd, dset->stobs);
 	dset->t1 = 0;
 	dset->t2 = dset->n - 1;
