@@ -164,4 +164,16 @@ int gretl_loess (const double *y, const double *x, int poly_order,
 		 double bandwidth, gretlopt opt, DATASET *dset, 
 		 double *m);
 
+double series_get_nobs (int t1, int t2, const double *x);
+
+double series_sum_all (int t1, int t2, const double *x);
+
+gretl_matrix *aggregate_by (const double *x, 
+			    const double *y,
+			    const int *xlist,
+			    const int *ylist,
+			    const char *fncall,
+			    const DATASET *dset,
+			    int *err);
+
 #endif /* GENFUNCS_H */
