@@ -85,6 +85,9 @@ struct str_table dvars[] = {
     { R_STOPWATCH, "$stopwatch" },
     { R_NSCAN,     "$nscan" },
     { R_PUNIT,     "$unit" },
+    { R_OBSMAJ,    "$obsmajor" },
+    { R_OBSMIN,    "$obsminor" },
+    { R_OBSMIC,    "$obsmicro" },
     { R_WINDOWS,   "$windows" },
     { R_VERSION,   "$version" },
     { R_ERRNO,     "$error" },
@@ -338,6 +341,7 @@ struct str_table funcs[] = {
     { F_READFILE, "readfile" },
     { F_BACKTICK, "grab" },
     { F_STRSTR,   "strstr" },
+    { F_STRSTRIP, "strstrip" },
     { F_STRNCMP,  "strncmp" },
     { F_STRLEN,   "strlen" },
     { F_SSCANF,   "sscanf" },
@@ -391,6 +395,9 @@ struct str_table funcs[] = {
     { F_HALTON,   "halton" },
     { F_ISNAN,    "isnan" },
     { F_TYPESTR,  "typestr" },
+    { F_QUADTAB,  "quadtable" },
+    { F_AGGRBY,   "aggregate" },
+    { F_REMOVE,   "remove" },
     { 0,          NULL }
 };
 
@@ -733,7 +740,7 @@ void context_error (int c, parser *p)
 
 static char *get_quoted_string (parser *p)
 {
-    int n = parser_gretl_charpos(p, '"');
+    int n = parser_char_index(p, '"');
     char *s = NULL;
 
     if (n >= 0) {
@@ -780,8 +787,11 @@ static int might_be_date_string (const char *s, int n)
     } else if (sscanf(s, "%d:%d", &y, &m) == 2) {
 	/* quarterly, monthly date */
 	return 1;
+    } else if (sscanf(s, "%d-%d-%d", &y, &m, &d) == 3) {
+	/* daily date? */
+	return 1;
     } else if (sscanf(s, "%d/%d/%d", &y, &m, &d) == 3) {
-	/* daily date */
+	/* daily date? */
 	return 1;
     }
 
@@ -993,6 +1003,23 @@ static void look_up_word (const char *s, parser *p)
     }
 }
 
+static void maybe_treat_as_postfix (parser *p)
+{
+    if (p->sym == UNUM) {
+	const char *ok = ")]}+-*/%,:";
+	int c = parser_next_nonspace_char(p, 1);
+
+	/* Interpret as foo++ or foo-- ? Only if
+	   the following character is suitable.
+	*/
+	if (c == 0 || strchr(ok, c)) {
+	    p->sym = p->ch == '+'? UNUM_P : UNUM_M;
+	    /* swallow the pluses or minuses */
+	    parser_advance(p, 1);
+	}
+    }
+}
+
 #define could_be_matrix(t) (model_data_matrix(t) || \
 			    model_data_matrix_builder(t) || \
 			    t == M_UHAT || t == M_YHAT || \
@@ -1060,10 +1087,16 @@ static void word_check_next_char (parser *p)
     } else if (p->ch == '.' && isalpha(*p->point)) {
 	if (p->sym == ULIST) {
 	    p->sym = LISTVAR;
+	} else if (p->sym == BUNDLE) {
+	    p->sym = BMEMB;
 	} else {
 	    p->err = E_PARSE;
 	}
-    }
+    } else if (p->ch == '+' && *p->point == '+') {
+	maybe_treat_as_postfix(p);
+    } else if (p->ch == '-' && *p->point == '-') {
+	maybe_treat_as_postfix(p);
+    }	
 
     if (p->err) {
 	context_error(p->ch, p);
@@ -1562,6 +1595,8 @@ const char *getsymb (int t, const parser *p)
 	return "LISTVAR";
     } else if (t == BOBJ) {
 	return "BOBJ";
+    } else if (t == BMEMB) {
+	return "BMEMB";
     } else if (t == LISTELEM) {
 	return "LISTELEM";
     } else if (t == VEC) {
