@@ -44,6 +44,9 @@ typedef enum {
     R_SCALAR_MAX, /* separator: scalars vs series */
     R_INDEX,      /* consecutive observations index */
     R_PUNIT,      /* 1-based panel unit index */
+    R_OBSMAJ,     /* major component of observation (e.g. year) */
+    R_OBSMIN,     /* minor component of observation (e.g. quarter, month) */
+    R_OBSMIC,     /* micro component of observation (e.g. day) */
     R_SERIES_MAX, /* separator: series vs matrices */
     R_TEST_STAT,  /* last test statistic(s) (scalar or matrix) */
     R_TEST_PVAL,  /* last test p-value(s) (scalar or matrix) */
@@ -201,7 +204,7 @@ int function_lookup (const char *s);
 
 int const_lookup (const char *s);
 
-double get_const_by_name (const char *name);
+double get_const_by_name (const char *name, int *err);
 
 const char *gretl_function_complete (const char *s);
 
