@@ -88,7 +88,9 @@ enum {
 	      UMAT,	  /* user variable, named matrix */
 	      ULIST,      /* user variable, named list */
 	      UOBJ,	  /* user-defined object (e.g. model) */
-  /* 60 */    NUM,	  /* scalar, evaluated */
+  /* 60 */    UNUM_P,     /* user scalar++ */
+	      UNUM_M,     /* user scalar-- */
+	      NUM,	  /* scalar, evaluated */
 	      VEC,	  /* series, evaluated */
 	      MAT,	  /* matrix, evaluated */
 	      OBS,	  /* observation from a series */
@@ -96,9 +98,9 @@ enum {
               DMSL,	  /* "dollar" matrix plus subspec */
 	      DMSTR,	  /* "dollar" matrix plus string subspec */
 	      MSL2,	  /* unevaluated matrix subspec */
-	      MSPEC,	  /* evaluated matrix subspec */
+  /* 70 */    MSPEC,	  /* evaluated matrix subspec */
 	      SUBSL,	  /* row or column component of MSPEC */
-  /* 70 */    MDEF,	  /* explicit matrix definition {...} */
+	      MDEF,	  /* explicit matrix definition {...} */
               LAG,        /* variable plus lag length */	  
 	      DVAR,	  /* $ "dataset" variable (mostly scalar or series) */
 	      MVAR,	  /* $ model var (scalar, series, or matrix) */
@@ -106,19 +108,20 @@ enum {
               LIST,	  /* list, evaluated */
 	      LISTVAR,    /* variable in list, dot syntax */
 	      LISTELEM,   /* list member, [...] syntax) */
-	      MLISTELEM,  /* accessor list member, [...] syntax) */
+  /* 80 */    MLISTELEM,  /* accessor list member, [...] syntax) */
 	      STR,	  /* string */
-  /* 80 */    BUNDLE,     /* gretl bundle (hash table) */
+	      BUNDLE,     /* gretl bundle (hash table) */
               BOBJ,       /* object inside a bundle */
+	      BMEMB,      /* object in bundle (dot notation) */
 	      FARGS,	  /* set of n function arguments */
+              WLIST,      /* wildcard list spec */
               EMPTY,      /* "null" */
 	      ABSENT,
 	      DTYPE_MAX,  /* SEPARATOR: end of "bare" types */
-              EROOT,	  /* dummy root for (...) expression */
-              UFUN,	  /* user-defined function */
+  /* 90 */    EROOT,	  /* dummy root for (...) expression */
+	      UFUN,	  /* user-defined function */
 	      RFUN,       /* GNU R function */
 	      USTR,       /* string variable */
-  /* 90 */    WLIST,      /* wildcard list spec */
 	      IVEC,       /* array of ints, not a varlist */
               INC,   
               DEC,
@@ -244,7 +247,6 @@ enum {
     F_FFTI,
     F_UPPER,
     F_LOWER,
-    F_MREAD,
     F_POLROOTS,
     F_XPX,
     F_ARGNAME,
@@ -266,6 +268,8 @@ enum {
     F_FREQ,
     F_ISNAN,
     F_TYPESTR,
+    F_STRSTRIP,
+    F_REMOVE,
     F1_MAX,	  /* SEPARATOR: end of single-arg functions */
     F_COR,
     F_COV,
@@ -292,10 +296,8 @@ enum {
     F_HDPROD,     /* horizontal direct product */
     F_CDIV,	  /* complex division */
     F_MXTAB,
-    F_MWRITE,
     F_MRSEL,
     F_MCSEL,
-    F_LLAG,
     F_WMEAN,
     F_WVAR,
     F_WSD,
@@ -329,7 +331,9 @@ enum {
     F_PXSUM,
     F_PSD,
     F_RANDINT,
+    F_MREAD,
     F2_MAX,	  /* SEPARATOR: end of two-arg functions */
+    F_LLAG,
     F_PRINCOMP,
     F_BFGSMAX,
     F_MSHAPE,
@@ -357,6 +361,8 @@ enum {
     F_NADARWAT,
     F_SIMANN,
     F_HALTON,
+    F_MWRITE,
+    F_AGGRBY,
     F3_MAX,       /* SEPARATOR: end of three-arg functions */
     F_BKFILT,
     F_MOLS,
@@ -370,6 +376,7 @@ enum {
     F_NRMAX,
     F_LOESS,
     F_GHK,
+    F_QUADTAB,
     FN_MAX,	  /* SEPARATOR: end of n-arg functions */
 };
 
@@ -400,7 +407,8 @@ enum {
 #define string_arg_func(s) (s == F_ISSERIES || s == F_ISNULL || \
 			    s == F_ISLIST   || s == F_ISSTRING || \
 			    s == F_OBSNUM || s == F_BACKTICK || \
-			    s == F_VARNUM || s == F_ARGNAME)
+			    s == F_VARNUM || s == F_ARGNAME || \
+                            s == F_REMOVE)
 
 /* function with multiple args, character code for first arg */
 #define char0_func(s) (s == F_PVAL || s == F_CDF || s == F_INVCDF || \
@@ -409,7 +417,8 @@ enum {
 
 /* functions taking a string arg in last position */
 #define string_last_func(s) (s == F_FDJAC || s == F_BFGSMAX || \
-                             s == F_NRMAX || s == F_DESEAS)
+                             s == F_NRMAX || s == F_DESEAS || \
+			     s == F_AGGRBY)
 
 /* functions taking string arg in middle position */
 #define string_mid_func(s) (s == F_BFGSMAX || s == F_NRMAX || s == F_SIMANN)
@@ -422,7 +431,8 @@ enum {
 
 #define evalb2(s) (binary_op(s) || func2_symb(s) || s == MSL || \
                    s == MSL2 || s == SUBSL || s == LAG || \
-                   s == OBS || s == BOBJ || s == LISTELEM)
+                   s == OBS || s == BOBJ || s == LISTELEM || \
+                   s == BMEMB)
 
 #define b1sym(s) (unary_op(s) || func1_symb(s) || funcn_symb(s) || \
                   s == G_LPR || s == EROOT)
@@ -527,14 +537,15 @@ enum {
     P_LHLIST  = 1 << 18, /* there was a pre-existing LHS list */
     P_LHSTR   = 1 << 19, /* there was a pre-existing LHS string */
     P_LHMAT   = 1 << 20, /* there was a pre-existing LHS matrix */
-    P_QUIET   = 1 << 21, /* don't print any messages or labels */
-    P_GETSTR  = 1 << 22, /* state: flag acceptance of plain strings */
-    P_LOOP    = 1 << 23, /* executing pre-built genr in loop context */
-    P_SLAVE   = 1 << 24, /* running as "slave" of NLS/MLE/GMM */
-    P_LHPTR   = 1 << 25, /* left-hand side: pointer type wanted */
-    P_MMASK   = 1 << 26, /* genr result is masked matrix */
-    P_SLICING = 1 << 27, /* calculating matrix slice (temporary) */
-    P_LAGPRSE = 1 << 28  /* parsing lag spec (temporary) */
+    P_LHBUN   = 1 << 21, /* there was a pre-existing LHS bundle */
+    P_QUIET   = 1 << 22, /* don't print any messages or labels */
+    P_GETSTR  = 1 << 23, /* state: flag acceptance of plain strings */
+    P_LOOP    = 1 << 24, /* executing pre-built genr in loop context */
+    P_SLAVE   = 1 << 25, /* running as "slave" of NLS/MLE/GMM */
+    P_LHPTR   = 1 << 26, /* left-hand side: pointer type wanted */
+    P_MMASK   = 1 << 27, /* genr result is masked matrix */
+    P_SLICING = 1 << 28, /* calculating matrix slice (temporary) */
+    P_LAGPRSE = 1 << 29  /* parsing lag spec (temporary) */
 };
 
 struct lhinfo {
@@ -586,8 +597,8 @@ struct parser_ {
 int parser_getc (parser *p);
 void parser_ungetc (parser *p);
 void parser_advance (parser *p, int n);
-int parser_gretl_charpos (parser *p, int c);
-int parser_next_nonspace_char (parser *p);
+int parser_char_index (parser *p, int c);
+int parser_next_nonspace_char (parser *p, int skip);
 void parser_print_input (parser *p);
 void lex (parser *s);
 NODE *new_node (int t);
