@@ -79,8 +79,8 @@ static NODE *newref (parser *p, int t)
 	    n->vnum = p->idnum;
 	    n->v.xvec = p->dset->Z[n->vnum];
 	    n->vname = p->idstr;
-	} else if (t == UNUM) {
-	    n->t = NUM;
+	} else if (t == UNUM || t == UNUM_P || t == UNUM_M) {
+	    n->t = t == UNUM ? NUM : t;
 	    n->vname = p->idstr;
 	    n->v.xval = *(double *) p->uval;
 	} else if (t == UMAT) {
@@ -282,6 +282,8 @@ static NODE *base (parser *p, NODE *up)
     case WLIST:
     case BUNDLE:
     case USTR:
+    case UNUM_P:
+    case UNUM_M:
 	t = newref(p, p->sym);
 	lex(p);
 	break;
@@ -347,7 +349,8 @@ static NODE *base (parser *p, NODE *up)
 
 #if SDEBUG
     notify("base", t, p);
-    fprintf(stderr, "on exit from base, p->sym = %d\n", p->sym);
+    fprintf(stderr, "on exit from base, p->sym = %d (p->err = %d)\n", 
+	    p->sym, p->err);
 #endif    
 
     return t;
@@ -635,7 +638,7 @@ static NODE *get_middle_string_arg (parser *p)
 
     if (p->ch == '"') {
 	/* special: arg is wrapped in quotes */
-	close = parser_gretl_charpos(p, '"');
+	close = parser_char_index(p, '"');
 	if (close < 0) {
 	    unmatched_symbol_error('"', p);
 	    return NULL;
@@ -712,6 +715,24 @@ static NODE *get_middle_string_arg (parser *p)
     return (p->err)? NULL : newstr(p->idstr);
 }
 
+static NODE *get_bundle_member_name (parser *p)
+{
+    int i, n = gretl_namechar_spn(p->point);
+
+    p->idstr = gretl_strndup(p->point, n);
+    if (p->idstr == NULL) {
+	p->err = E_ALLOC;
+	return NULL;
+    }     
+
+    for (i=0; i<=n; i++) {
+	parser_getc(p);
+    }
+    lex(p);
+
+    return newstr(p->idstr);
+}
+
 enum {
     RIGHT_STR = 1 << 0,
     MID_STR   = 1 << 1
@@ -983,7 +1004,7 @@ static void get_args (NODE *t, parser *p, int f, int k, int opt, int *next)
 
 static void get_ovar_ref (NODE *t, parser *p)
 {
-    if (p->ch != '.' || parser_gretl_charpos(p, '$') != 0) {
+    if (p->ch != '.' || parser_char_index(p, '$') != 0) {
 	p->err = E_PARSE;
 	return;
     }
@@ -1150,6 +1171,12 @@ static NODE *powterm (parser *p)
 	    t->v.b2.l = newref(p, BUNDLE);
 	    lex(p);
 	    t->v.b2.r = get_final_string_arg(p, sym, 1);
+	}
+    } else if (sym == BMEMB) {
+	t = newb2(sym, NULL, NULL);
+	if (t != NULL) {
+	    t->v.b2.l = newref(p, BUNDLE);
+	    t->v.b2.r = get_bundle_member_name(p);
 	}	
     } else if (sym == OVAR) {
 	t = newb2(sym, NULL, NULL);
