@@ -207,6 +207,26 @@ void gretl_bundle_destroy (gretl_bundle *bundle)
     }
 }
 
+/**
+ * gretl_bundle_void_content:
+ * @bundle: target bundle.
+ *
+ * Frees all contents of @bundle.
+ */
+
+void gretl_bundle_void_content (gretl_bundle *bundle)
+{
+    if (bundle != NULL) {
+	if (bundle->ht != NULL) {
+	    g_hash_table_destroy(bundle->ht);
+	}
+	free(bundle->creator);
+	bundle->ht = g_hash_table_new_full(g_str_hash, g_str_equal, 
+					   g_free, bundled_item_destroy);
+	bundle->creator = NULL;
+    }
+}
+
 /**
  * gretl_bundle_new:
  *
@@ -561,10 +581,16 @@ static int real_gretl_bundle_set_data (gretl_bundle *b, const char *key,
 				       void *ptr, GretlType type,
 				       int size, const char *note)
 {
-    bundled_item *item;
-    int err = 0;
+    bundled_item *item = NULL;
+    int err;
 
-    item = bundled_item_new(type, ptr, size, note, &err);
+    err = strlen(key) >= VNAMELEN ? E_DATA : 0;
+
+    if (err) {
+	gretl_errmsg_sprintf("'%s': invalid key string", key);
+    } else {
+	item = bundled_item_new(type, ptr, size, note, &err);
+    }
 
     if (!err) {
 	gchar *k = g_strdup(key);
