@@ -79,6 +79,7 @@ typedef enum {
     INCLUDE,
     INFO,
     INTREG,
+    JOIN,
     KALMAN,
     KPSS,
     LABELS, 
@@ -93,6 +94,7 @@ typedef enum {
     LOOP,
     MAHAL,
     MAKEPKG,
+    MARKERS,
     MEANTEST,
     MLE,
     MODELTAB,
