@@ -54,13 +54,7 @@ static gchar *gretl_Rmsg;
 static gchar *gretl_Oxprog;
 static gchar *gretl_octave_prog;
 static gchar *gretl_stata_prog;
-
-enum {
-    LANG_R = 1,
-    LANG_OX,
-    LANG_OCTAVE,
-    LANG_STATA
-};
+static gchar *gretl_python_prog;
 
 static void destroy_foreign (void)
 {
@@ -83,6 +77,8 @@ static int set_foreign_lang (const char *lang, PRN *prn)
 	foreign_lang = LANG_OCTAVE;
     } else if (g_ascii_strcasecmp(lang, "stata") == 0) {
 	foreign_lang = LANG_STATA;
+    } else if (g_ascii_strcasecmp(lang, "python") == 0) {
+	foreign_lang = LANG_PYTHON;
     } else {
 	pprintf(prn, "%s: unknown language\n", lang);
 	err = E_DATA;
@@ -124,6 +120,17 @@ static const gchar *gretl_stata_filename (void)
     return gretl_stata_prog;
 }
 
+static const gchar *gretl_python_filename (void)
+{
+    if (gretl_python_prog == NULL) {
+	const char *dotdir = gretl_dotdir();
+
+	gretl_python_prog = g_strdup_printf("%sgretltmp.py", dotdir);
+    }
+
+    return gretl_python_prog;
+}
+
 static void make_gretl_R_names (void)
 {
     static int done;
@@ -200,7 +207,12 @@ static int lib_run_other_sync (gretlopt opt, PRN *prn)
 	fname = gretl_octave_filename();
 	cmd = g_strdup_printf("\"%s\" --silent \"%s\"", path, fname);
     } else if (foreign_lang == LANG_STATA) {
-	cmd = g_strdup_printf("stata /q /e gretltmp.do");
+	path = gretl_stata_path();
+	cmd = g_strdup_printf("%s /q /e gretltmp.do", path);
+    } else if (foreign_lang == LANG_PYTHON) {
+	path = gretl_python_path();
+	fname = gretl_python_filename();
+	cmd = g_strdup_printf("%s \"%s\"", path, fname);
     } else {
 	return 1;
     }
@@ -308,7 +320,7 @@ static int lib_run_R_sync (gretlopt opt, PRN *prn)
 
 static int lib_run_other_sync (gretlopt opt, PRN *prn)
 {
-    char *argv[5];
+    char *argv[6];
     int err;
 
     if (foreign_lang == LANG_OX) {
@@ -320,14 +332,19 @@ static int lib_run_other_sync (gretlopt opt, PRN *prn)
 	argv[1] = "--silent";
 	argv[2] = (char *) gretl_octave_filename();
 	argv[3] = NULL;
+    } else if (foreign_lang == LANG_PYTHON) {
+	argv[0] = (char *) gretl_python_path();
+	argv[1] = (char *) gretl_python_filename();
+	argv[2] = NULL;
     } else if (foreign_lang == LANG_STATA) {
-	argv[0] = "stata";
-	argv[1] = "-b";
-	argv[2] = "-q";
-	argv[3] = "gretltmp.do";
-	argv[4] = NULL;
+	argv[0] = (char *) gretl_stata_path();
+	argv[1] = "-q";
+	argv[2] = "-b";
+	argv[3] = "do";
+	argv[4] = "gretltmp.do";
+	argv[5] = NULL;
 	/* otherwise there's no way to control the location
-	   of the state output (gretltmp.log)
+	   of the stata output (gretltmp.log)
 	*/
 	gretl_chdir(gretl_dotdir());
     }	
@@ -367,6 +384,12 @@ static int write_ox_io_file (void)
 #endif	
 	    fputs("}\n\n", fp);
 
+	    fputs("gretl_export_nodot (const X, const str)\n{\n", fp);
+	    fputs("  decl fp = fopen(str, \"w\");\n", fp);
+	    fputs("  fprint(fp, \"%d \", rows(X), \"%d\", columns(X));\n", fp);
+	    fputs("  fprint(fp, \"%.15g\", X);\n", fp);
+	    fputs("  fclose(fp);\n}\n\n", fp);
+
 	    fputs("gretl_export (const X, const str)\n{\n", fp);
             fputs("  decl dname = gretl_dotdir();\n", fp);
 	    fputs("  decl fp = fopen(dname ~ str, \"w\");\n", fp);
@@ -415,17 +438,25 @@ static int write_octave_io_file (void)
 #endif	
 	    fputs("endfunction\n\n", fp);
 
-	    fputs("function gretl_export(X, str)\n", fp);
-            fputs("  dname = gretl_dotdir();\n", fp);
-	    fputs("  fd = fopen(strcat(dname, str), \"w\");\n", fp);
+	    fputs("function gretl_export(X, str, autodot=1)\n", fp);
+	    fputs("  if (autodot)\n", fp);
+            fputs("    dname = gretl_dotdir();\n", fp);
+	    fputs("    fd = fopen(strcat(dname, str), \"w\");\n", fp);
+	    fputs("  else\n", fp);
+	    fputs("    fd = fopen(str, \"w\");\n", fp);
+	    fputs("  endif\n", fp);
 	    fputs("  fprintf(fd, \"%d %d\\n\", size(X));\n", fp);
 	    fputs("  fprintf(fd, \"%.15g\\n\", X);\n", fp);
 	    fputs("  fclose(fd);\n", fp);
 	    fputs("endfunction\n\n", fp);  
 
-	    fputs("function A = gretl_loadmat(str)\n", fp);
-            fputs("  dname = gretl_dotdir();\n", fp);
-	    fputs("  fd = fopen(strcat(dname, str), \"r\");\n", fp);
+	    fputs("function A = gretl_loadmat(str, autodot=1)\n", fp);
+	    fputs("  if (autodot)\n", fp);
+            fputs("    dname = gretl_dotdir();\n", fp);
+	    fputs("    fd = fopen(strcat(dname, str), \"r\");\n", fp);
+	    fputs("  else\n", fp);
+	    fputs("    fd = fopen(str, \"r\");\n", fp);
+            fputs("  endif\n", fp);
 	    fputs("  [r,c] = fscanf(fd, \"%d %d\", \"C\");\n", fp);
 	    fputs("  A = reshape(fscanf(fd, \"%g\", r*c),c,r)';\n", fp);
 	    fputs("  fclose(fd);\n", fp);
@@ -439,6 +470,60 @@ static int write_octave_io_file (void)
     return 0;
 }
 
+static int write_python_io_file (void)
+{
+    static int written;
+
+    if (!written) {
+	const char *dotdir = gretl_dotdir();
+	gchar *fname;
+	FILE *fp;
+
+	fname = g_strdup_printf("%sgretl_io.py", dotdir);
+	fp = gretl_fopen(fname, "w");
+	g_free(fname);
+
+	if (fp == NULL) {
+	    return E_FOPEN;
+	} else {
+#ifdef G_OS_WIN32
+            gchar *dotcpy = win32_dotpath();
+
+	    fprintf(fp, "gretl_dotdir = \"%s\"\n\n", dotcpy);
+	    g_free(dotcpy);
+#else
+	    fprintf(fp, "gretl_dotdir = \"%s\"\n\n", dotdir);
+#endif	
+
+	    fputs("def gretl_export(X, fname, autodot=1):\n", fp);
+	    fputs("  from numpy import asmatrix\n", fp);
+	    fputs("  M = asmatrix(X)\n", fp);
+	    fputs("  r, c = M.shape\n", fp);
+	    fputs("  if autodot:\n", fp);
+            fputs("    fname = gretl_dotdir + fname\n", fp);
+	    fputs("  f = open(fname, 'w')\n", fp);
+	    fputs("  f.write(repr(r) + '\\t' + repr(c) + '\\n')\n", fp);
+	    fputs("  for i in range(0, r):\n", fp);
+	    fputs("    for j in range(0, c):\n", fp);
+	    fputs("      f.write('%.18e ' % M[i,j])\n", fp);
+            fputs("    f.write('\\n')\n", fp);
+	    fputs("  f.close()\n\n", fp);
+
+	    fputs("def gretl_loadmat(fname, autodot=1):\n", fp);
+	    fputs("  from numpy import loadtxt\n", fp);
+	    fputs("  if autodot:\n", fp);
+	    fputs("    fname = gretl_dotdir + fname\n", fp);
+	    fputs("  M = loadtxt(fname, skiprows=1)\n", fp);
+	    fputs("  return M\n\n", fp);
+
+	    fclose(fp);
+	    written = 1;
+	}
+    }
+
+    return 0;
+}
+
 static int write_stata_io_file (void)
 {
     static int written;
@@ -482,40 +567,98 @@ static int write_stata_io_file (void)
     return 0;
 }
 
-static void add_gretl_include (FILE *fp)
+static void add_gretl_include (int lang, FILE *fp)
 {
+    if (lang == LANG_PYTHON) {
+	fputs("from gretl_io import gretl_dotdir, gretl_loadmat, "
+	      "gretl_export\n", fp);
+	return;
+    }
+
 #ifdef G_OS_WIN32
     gchar *dotcpy = win32_dotpath();
 
-    if (foreign_lang == LANG_OX) {
+    if (lang == LANG_OX) {
 	if (strchr(dotcpy, ' ')) {
 	    fprintf(fp, "#include \"%sgretl_io.ox\"\n", dotcpy);
 	} else {
 	    fprintf(fp, "#include <%sgretl_io.ox>\n", dotcpy);
 	}
-    } else if (foreign_lang == LANG_OCTAVE) {
+    } else if (lang == LANG_OCTAVE) {
 	fprintf(fp, "source(\"%sgretl_io.m\")\n", dotcpy);
-    } else if (foreign_lang == LANG_STATA) {
+    } else if (lang == LANG_STATA) {
 	fprintf(fp, "quietly adopath + \"%s\"\n", dotcpy);
     }
     g_free(dotcpy);
 #else
     const char *dotdir = gretl_dotdir();
 
-    if (foreign_lang == LANG_OX) {
+    if (lang == LANG_OX) {
 	if (strchr(dotdir, ' ')) {
 	    fprintf(fp, "#include \"%sgretl_io.ox\"\n", dotdir);
 	} else {
 	    fprintf(fp, "#include <%sgretl_io.ox>\n", dotdir);
 	}
-    } else if (foreign_lang == LANG_OCTAVE) {
+    } else if (lang == LANG_OCTAVE) {
 	fprintf(fp, "source(\"%sgretl_io.m\")\n", dotdir);
-    } else if (foreign_lang == LANG_STATA) {
+    } else if (lang == LANG_STATA) {
 	fprintf(fp, "quietly adopath + \"%s\"\n", dotdir);
     }
 #endif
 }
 
+static int get_foreign_indent (void)
+{
+    const char *s;
+    int i, n, ret = 100;
+
+    for (i=0; i<foreign_n_lines; i++) {
+	n = 0;
+	s = foreign_lines[i];
+	while (*s == ' ' || *s == '\t') {
+	    n++;
+	    s++;
+	}
+	if (n < ret) {
+	    ret = n;
+	}
+    }
+
+    return ret;
+}
+
+static void put_foreign_lines (FILE *fp)
+{
+    int i, n = get_foreign_indent();
+
+    for (i=0; i<foreign_n_lines; i++) { 
+	fprintf(fp, "%s\n", foreign_lines[i] + n);
+	if (foreign_lang == LANG_OX) {
+	    if (strstr(foreign_lines[i], "oxstd.h")) {
+		add_gretl_include(LANG_OX, fp);
+	    }
+	}
+    }
+}
+
+static void put_foreign_buffer (const char *buf, FILE *fp)
+{
+    char line[1024];
+
+    bufgets_init(buf);
+
+    while (bufgets(line, sizeof line, buf)) {
+	fputs(line, fp);
+	if (foreign_lang == LANG_OX) {
+	    if (strstr(line, "oxstd.h")) {
+		add_gretl_include(LANG_OX, fp);
+	    }
+	}
+    }
+
+    bufgets_finalize(buf);
+}
+
 /**
  * write_gretl_ox_file:
  * @buf: text buffer containing Ox code.
@@ -532,33 +675,57 @@ int write_gretl_ox_file (const char *buf, gretlopt opt, const char **pfname)
 {
     const gchar *fname = gretl_ox_filename();
     FILE *fp = gretl_fopen(fname, "w");
-    int err = write_ox_io_file();
+
+    write_ox_io_file();
 
     if (fp == NULL) {
 	return E_FOPEN;
     } else {
 	if (buf != NULL) {
 	    /* pass on the material supplied in the 'buf' argument */
-	    char line[1024];
-
-	    bufgets_init(buf);
-	    while (bufgets(line, sizeof line, buf)) {
-		fputs(line, fp);
-		if (!err && strstr(line, "oxstd.h")) {
-		    add_gretl_include(fp);
-		}
-	    }
-	    bufgets_finalize(buf);
+	    put_foreign_buffer(buf, fp);
 	} else {
 	    /* put out the stored 'foreign' lines */
-	    int i;
+	    put_foreign_lines(fp);
+	}
+	fclose(fp);
+	if (pfname != NULL) {
+	    *pfname = fname;
+	}
+    }
 
-	    for (i=0; i<foreign_n_lines; i++) { 
-		fprintf(fp, "%s\n", foreign_lines[i]);
-		if (!err && strstr(foreign_lines[i], "oxstd.h")) {
-		    add_gretl_include(fp);
-		}
-	    }
+    return 0;
+}
+
+/**
+ * write_gretl_python_file:
+ * @buf: text buffer containing Python code.
+ * @opt: should contain %OPT_G for use from GUI.
+ * @pfname: location to receive name of file written, or %NULL.
+ *
+ * Writes the content of @buf into a file in the gretl user's
+ * "dotdir".
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int write_gretl_python_file (const char *buf, gretlopt opt, const char **pfname)
+{
+    const gchar *fname = gretl_python_filename();
+    FILE *fp = gretl_fopen(fname, "w");
+
+    write_python_io_file();
+
+    if (fp == NULL) {
+	return E_FOPEN;
+    } else {
+	add_gretl_include(LANG_PYTHON, fp);
+	if (buf != NULL) {
+	    /* pass on the material supplied in the 'buf' argument */
+	    put_foreign_buffer(buf, fp);
+	} else {
+	    /* put out the stored 'foreign' lines */
+	    put_foreign_lines(fp);
 	}
 	fclose(fp);
 	if (pfname != NULL) {
@@ -601,16 +768,16 @@ int write_gretl_stata_file (const char *buf, gretlopt opt,
 {
     const gchar *fname = gretl_stata_filename();
     FILE *fp = gretl_fopen(fname, "w");
-    int err = write_stata_io_file();
+
+    write_stata_io_file();
 
     if (fp == NULL) {
 	return E_FOPEN;
     } else {
-	/* source the I-O functions */
-	if (!err) {
-	    add_gretl_include(fp);
-	}
+	int err;
 
+	/* source the I-O functions */
+	add_gretl_include(LANG_STATA, fp);
 	if (opt & OPT_D) {
 	    /* --send-data */
 	    err = write_data_for_stata(dset, fp);
@@ -619,23 +786,12 @@ int write_gretl_stata_file (const char *buf, gretlopt opt,
 		return err;
 	    }
 	}
-
 	if (buf != NULL) {
 	    /* pass on the material supplied in the 'buf' argument */
-	    char line[1024];
-
-	    bufgets_init(buf);
-	    while (bufgets(line, sizeof line, buf)) {
-		fputs(line, fp);
-	    }
-	    bufgets_finalize(buf);
+	    put_foreign_buffer(buf, fp);
 	} else {
 	    /* put out the stored 'foreign' lines */
-	    int i;
-
-	    for (i=0; i<foreign_n_lines; i++) { 
-		fprintf(fp, "%s\n", foreign_lines[i]);
-	    }
+	    put_foreign_lines(fp);
 	}
 	fclose(fp);
 	if (pfname != NULL) {
@@ -677,16 +833,16 @@ int write_gretl_octave_file (const char *buf, gretlopt opt,
 {
     const gchar *fname = gretl_octave_filename();
     FILE *fp = gretl_fopen(fname, "w");
-    int err = write_octave_io_file();
+
+    write_octave_io_file();
 
     if (fp == NULL) {
 	return E_FOPEN;
     } else {
-	/* source the I-O functions */
-	if (!err) {
-	    add_gretl_include(fp);
-	}
+	int err;
 
+	/* source the I-O functions */
+	add_gretl_include(LANG_OCTAVE, fp);
 	if (opt & OPT_D) {
 	    /* --send-data */
 	    err = write_data_for_octave(dset, fp);
@@ -695,23 +851,12 @@ int write_gretl_octave_file (const char *buf, gretlopt opt,
 		return err;
 	    }
 	}
-
 	if (buf != NULL) {
 	    /* pass on the material supplied in the 'buf' argument */
-	    char line[1024];
-
-	    bufgets_init(buf);
-	    while (bufgets(line, sizeof line, buf)) {
-		fputs(line, fp);
-	    }
-	    bufgets_finalize(buf);
+	    put_foreign_buffer(buf, fp);
 	} else {
 	    /* put out the stored 'foreign' lines */
-	    int i;
-
-	    for (i=0; i<foreign_n_lines; i++) { 
-		fprintf(fp, "%s\n", foreign_lines[i]);
-	    }
+	    put_foreign_lines(fp);
 	}
 	fclose(fp);
 	if (pfname != NULL) {
@@ -780,9 +925,10 @@ static int write_data_for_R (const DATASET *dset,
 
 /* define an R function for passing data back to gretl */
 
-static void write_R_export_func (FILE *fp) 
+static void write_R_io_funcs (FILE *fp) 
 {
     fprintf(fp, "gretl.dotdir <- \"%s\"\n", gretl_dot_dir);
+
     fputs("gretl.export <- function(x) {\n", fp);
     fprintf(fp, "  prefix <- \"%s\"\n", gretl_dot_dir);
     fputs("  sx <- as.character(substitute(x))\n", fp);
@@ -804,6 +950,13 @@ static void write_R_export_func (FILE *fp)
     fputs("  gretlmsg <- paste(\"wrote\", fname, \"\\n\")\n", fp);
     fputs("  cat(gretlmsg)\n", fp);
     fputs("}\n", fp);
+
+    fputs("gretl.loadmat <- function(mname) {\n", fp);
+    fprintf(fp, "  prefix <- \"%s\"\n", gretl_dot_dir);
+    fputs("  fname <- paste(prefix, mname, sep=\"\")\n", fp);
+    fputs("  m <- as.matrix(read.table(fname, skip=1))\n", fp);
+    fputs("  return(m)\n", fp);
+    fputs("}\n", fp);
 }
 
 /* basic content which can either go into gretl.Rprofile or into
@@ -816,11 +969,11 @@ static void put_R_startup_content (FILE *fp)
     fputs("if (vnum > 2.41) library(utils)\n", fp);
     fputs("library(stats)\n", fp);
     fputs("if (vnum <= 1.89) library(ts)\n", fp);
-    write_R_export_func(fp);
+    write_R_io_funcs(fp);
 }
 
 /* Set up a gretl-specific R profile, and put notice of its existence
-   into the environment.  Used when exec'ing the R binary (only) */
+   into the environment. Used when exec'ing the R binary (only) */
 
 static int write_gretl_R_profile (gretlopt opt)
 {
@@ -877,7 +1030,7 @@ static int write_R_source_file (const char *buf,
 				gretlopt opt)
 {
     FILE *fp = gretl_fopen(gretl_Rsrc, "w");
-    int i, err = 0;
+    int err = 0;
 
 #if FDEBUG
     printf("write R source file: starting\n");
@@ -915,8 +1068,8 @@ static int write_R_source_file (const char *buf,
 		fputs("sink(errout, type=\"message\")\n", fp);
 	    }
 	    sunk = 1;
-#ifdef G_OS_WIN32
-	    /* can this go in the "startup content"? */
+#if 0 /* ifdef G_OS_WIN32 */
+	    /* not working, 2012-12-27 */
 	    maybe_print_R_path_addition(fp);
 #endif
 	}
@@ -936,9 +1089,7 @@ static int write_R_source_file (const char *buf,
 	    fputs(buf, fp);
 	} else if (!(opt & OPT_G)) {
 	    /* non-GUI */
-	    for (i=0; i<foreign_n_lines; i++) { 
-		fprintf(fp, "%s\n", foreign_lines[i]);
-	    }
+	    put_foreign_lines(fp);
 	}
 
 	if (sunk) {
@@ -1037,6 +1188,13 @@ static void delete_gretl_stata_file (void)
     }
 }
 
+static void delete_gretl_python_file (void)
+{
+    if (gretl_python_prog != NULL) {
+	gretl_remove(gretl_python_prog);
+    }
+}
+
 /* The following code block is used if we're implementing
    gretl's R support by dlopening the R shared library
    (as opposed to executing the R binary).
@@ -1210,6 +1368,33 @@ void gretl_R_reset_error (void)
     Rlib_err = 0;
 }
 
+#ifdef WIN32
+
+static void set_path_for_Rlib (const char *Rhome)
+{
+    char *path = getenv("PATH");
+    gchar *Rpath;
+
+    Rpath = g_strdup_printf("%s\\bin\\i386", Rhome);
+    fprintf(stderr, "Rpath = '%s'\n", Rpath);
+
+    if (path != NULL && strstr(path, Rpath) != NULL) {
+	; /* nothing to be done */
+    } else {
+	g_free(Rpath);
+	Rpath = g_strdup_printf("%s;%s\\bin\\i386", path, Rhome);
+	gretl_setenv("PATH", Rpath);
+	g_free(Rpath);
+	Rpath = NULL;
+    }
+
+    if (Rpath != NULL) {
+	g_free(Rpath);
+    }
+}
+
+#endif
+
 /* Initialize the R library for use with gretl.  Note that we only
    need do this once per gretl session.  We need to check that the
    environment is set to R's liking first, otherwise initialization
@@ -1246,6 +1431,8 @@ static int gretl_Rlib_init (void)
 	fprintf(stderr, "To use Rlib, the variable R_HOME must be set\n");
 	err = E_EXTERNAL;
 	goto bailout;
+    } else {
+	set_path_for_Rlib(Rhome);
     }
 #endif
 
@@ -1289,7 +1476,7 @@ static int gretl_Rlib_init (void)
 }
 
 /* run R's source() function on an R command file written by
-   gretl -- shared library version */
+   gretl, shared library version */
 
 static int lib_run_Rlib_sync (gretlopt opt, PRN *prn) 
 {
@@ -1647,6 +1834,10 @@ int foreign_append_line (const char *line, gretlopt opt, PRN *prn)
 	return 0;
     }
 
+#if 0
+    fprintf(stderr, "foreign_append_line: '%s'\n", line);
+#endif
+
     if (!foreign_started) {
 	/* starting from scratch */
 	err = foreign_block_init(line, opt, prn);
@@ -1743,6 +1934,14 @@ int foreign_execute (const DATASET *dset,
 	    delete_gretl_stata_file();
 	} else {
 	    err = lib_run_other_sync(foreign_opt, prn);
+	}
+    } else if (foreign_lang == LANG_PYTHON) {
+	err = write_gretl_python_file(NULL, foreign_opt, 
+				      NULL);
+	if (err) {
+	    delete_gretl_python_file();
+	} else {
+	    err = lib_run_other_sync(foreign_opt, prn);
 	}	
     } else {
 	/* "can't happen" */
