@@ -839,7 +839,7 @@ char *gretl_list_to_string (const int *list)
  * @err: location to receive error code.
  * 
  * Prints the names of the members of @list of integers into 
- * a newly allocated string, separated by single spaces.
+ * a newly allocated string, separated by commas.
  *
  * Returns: The string representation of the list on success,
  * or NULL on failure.
@@ -893,6 +893,58 @@ char *gretl_list_get_names (const int *list, const DATASET *dset,
     return buf;
 }
 
+/**
+ * gretl_list_get_names_array:
+ * @list: array of integers.
+ * @dset: dataset information.
+ * @err: location to receive error code.
+ * 
+ * Returns: An array of strings holding the names of the
+ * members of @list, or NULL on failure.
+ */
+
+char **gretl_list_get_names_array (const int *list, 
+				   const DATASET *dset,
+				   int *err)
+{
+    char **S = NULL;
+    int i, vi, n;
+
+    if (list == NULL) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    if (list[0] == 0) {
+	return NULL;
+    }
+
+    n = list[0];
+
+    S = strings_array_new(n);
+    if (S == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    for (i=0; i<n; i++) {
+	vi = list[i+1];
+	if (vi < 0 || vi >= dset->v) {
+	    S[i] = gretl_strdup("unknown");
+	} else {
+	    S[i] = gretl_strdup(dset->varname[vi]);
+	}
+	if (S[i] == NULL) {
+	    *err = E_ALLOC;
+	    strings_array_free(S, n);
+	    S = NULL;
+	    break;
+	}
+    }
+
+    return S;
+}
+
 /**
  * gretl_list_to_lags_string:
  * @list: array of integers.
@@ -1316,6 +1368,184 @@ int *gretl_list_intersection (const int *l1, const int *l2, int *err)
     return ret;
 }
 
+static void name_xprod_term (char *vname, int vi, int vj, 
+			     int di, const DATASET *dset)
+{
+    const char *si = dset->varname[vi];
+    const char *sj = dset->varname[vj];
+    int ilen = strlen(si);
+    int jlen = strlen(sj);
+    int totlen = ilen + jlen + 2;
+    char numstr[32];
+
+    sprintf(numstr, "%d", di);
+    totlen += strlen(numstr);
+
+    if (totlen > VNAMELEN - 1) {
+	int decr = totlen - (VNAMELEN - 1);
+
+	while (decr > 0) {
+	    if (ilen > jlen) {
+		ilen--;
+	    } else {
+		jlen--;
+	    }
+	    decr--;
+	}
+    }
+
+    sprintf(vname, "%.*s_%.*s_%s", ilen, si, jlen, sj, numstr);
+}
+
+static void set_xprod_label (int v, int vi, int vj, 
+			     double val, DATASET *dset)
+{
+    const char *si = dset->varname[vi];
+    const char *sj = dset->varname[vj];
+    char label[MAXLABEL];
+
+    sprintf(label, "interaction of %s and (%s == %g)", si, sj, val);
+    series_record_label(dset, v, label);
+}
+
+static int nonneg_integer_series (const DATASET *dset, int v)
+{
+    double xt;
+    int t;
+
+    for (t=dset->t1; t<=dset->t2; t++) {
+	xt = dset->Z[v][t];
+	if (!na(xt) && (xt != floor(xt) || xt < 0)) {
+	    return 0;
+	}
+    }
+
+    return 1;
+}
+
+/**
+ * gretl_list_product:
+ * @X: list of integers (representing discrete variables).
+ * @Y: list of integers.
+ * @dset: pointer to dataset.
+ * @err: location to receive error code.
+ *
+ * Creates a list holding the Cartesian product of @X and @Y.
+ *
+ * Returns: new list on success, NULL on error.
+ */
+
+int *gretl_list_product (const int *X, const int *Y, 
+			 DATASET *dset, int *err)
+{
+    int *ret = NULL;
+    gretl_matrix *xvals;
+    char vname[VNAMELEN];
+    const double *x, *y;
+    int *x_is_int = NULL;
+    int newv, n_old = 0;
+    int n, vi, vj;
+    int i, j, k, t;
+
+    if (X == NULL || Y == NULL) {
+	*err = E_DATA;
+	return NULL;
+    }	
+
+    if (X[0] == 0 || Y[0] == 0) {
+	ret = gretl_null_list();
+	if (ret == NULL) {
+	    *err = E_ALLOC;
+	}
+	return ret;
+    }
+
+    x_is_int = gretl_list_new(X[0]);
+    if (x_is_int == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    /* check the X list for discreteness */
+
+    for (j=1; j<=X[0] && !*err; j++) {
+	vj = X[j];
+	if (nonneg_integer_series(dset, vj)) {
+	    x_is_int[j] = 1;
+	} else if (!series_is_discrete(dset, vj)) {
+	    gretl_errmsg_sprintf(_("The variable '%s' is not discrete"),
+				 dset->varname[vj]);
+	    *err = E_DATA;
+	}
+    }
+
+    if (*err) {
+	free(x_is_int);
+	return NULL;
+    }
+
+    n = sample_size(dset);
+    newv = dset->v;
+
+    for (j=1; j<=X[0] && !*err; j++) {
+	vj = X[j];
+	x = dset->Z[vj];
+	xvals = gretl_matrix_values(x + dset->t1, n, OPT_S, err);
+	if (!*err) {
+	    *err = dataset_add_series(dset, Y[0] * xvals->rows);
+	    if (!*err) {
+		for (i=1; i<=Y[0] && !*err; i++) {
+		    vi = Y[i];
+		    y = dset->Z[vi];
+		    for (k=0; k<xvals->rows && !*err; k++) {
+			int v, oldv, iik;
+			double xik;
+
+			xik = gretl_vector_get(xvals, k);
+			iik = x_is_int[j] ? (int) xik : (k + 1);
+			name_xprod_term(vname, vi, vj, iik, dset);
+			oldv = current_series_index(dset, vname);
+			if (oldv > 0) {
+			    /* reuse existing series of the same name */
+			    v = oldv;
+			    n_old++;
+			} else {
+			    /* make a new series */
+			    v = newv++;
+			}
+			for (t=dset->t1; t<=dset->t2; t++) {
+			    if (na(x[t]) || xna(xik)) {
+				dset->Z[v][t] = NADBL;
+			    } else {
+				dset->Z[v][t] = (x[t] == xik)? y[t] : 0;
+			    }
+			}
+			gretl_list_append_term(&ret, v);
+			if (ret == NULL) {
+			    *err = E_ALLOC;
+			} else {
+			    if (v != oldv) {
+				strcpy(dset->varname[v], vname);
+			    }
+			    set_xprod_label(v, vi, vj, xik, dset);
+			}
+		    }
+		}
+	    }
+	    gretl_matrix_free(xvals);
+	}
+    }
+
+    free(x_is_int);
+
+    if (n_old > 0) {
+	/* we added more series than were actually needed */
+	dataset_drop_last_variables(dset, n_old);
+    }
+
+    return ret;
+}
+
 /**
  * gretl_list_omit_last:
  * @orig: an array of integers, the first element of which holds
