@@ -69,6 +69,8 @@ struct INTERNAL_PATHS {
     char rlibpath[MAXLEN];
     char oxlpath[MAXLEN];
     char octpath[MAXLEN];
+    char statapath[MAXLEN];
+    char pypath[MAXLEN];
     char dbhost[32];
     char pngfont[128];
     unsigned char status;
@@ -838,8 +840,9 @@ int gretl_deltree (const char *path)
 	}
 	if (!err) {
 	    closedir(dir);
-	    chdir("..");
-	    err = gretl_remove(path);
+	    if (chdir("..") == 0) {
+		err = gretl_remove(path);
+	    }
 	}
     }
 
@@ -1362,7 +1365,7 @@ char **get_plausible_search_dirs (int type, int *n_dirs)
     build_path(dirname, gretl_home(), subdir, NULL);
     err = strings_array_add(&dirs, n_dirs, dirname);
 
-#ifdef OSX_BUILD
+#ifdef OS_OSX
     if (!err) {
 	/* the user's ~/Library */
 	build_path(dirname, gretl_app_support_dir(), subdir, NULL);
@@ -1578,7 +1581,7 @@ char *gretl_addpath (char *fname, int script)
 
 	fname = tmp;
 	strcpy(fname, orig);
-#ifdef OSX_BUILD
+#ifdef OS_OSX
 	gpath = gretl_app_support_dir();
 #else
 	gpath = gretl_dotdir();
@@ -2254,6 +2257,16 @@ const char *gretl_octave_path (void)
     return paths.octpath;
 }
 
+const char *gretl_stata_path (void)
+{
+    return paths.statapath;
+}
+
+const char *gretl_python_path (void)
+{
+    return paths.pypath;
+}
+
 const char *gretl_current_dir (void)
 {
     return current_dir;
@@ -2559,23 +2572,23 @@ void set_gretl_plugin_path (const char *path)
 
 static int initialize_dotdir (void)
 {
-    char *home;
+    char *dirname;
     int err = 0;
 
     *paths.dotdir = '\0';
 
 #ifdef WIN32
-    home = appdata_path();
-    if (home != NULL) {
-	sprintf(paths.dotdir, "%s\\gretl\\", home);
-	free(home);
+    dirname = appdata_path();
+    if (dirname != NULL) {
+	sprintf(paths.dotdir, "%s\\gretl\\", dirname);
+	free(dirname);
     } else {
 	sprintf(paths.dotdir, "%s\\user\\", paths.gretldir);
     }
 #else
-    home = getenv("HOME");
-    if (home != NULL) {
-	sprintf(paths.dotdir, "%s/.gretl/", home);
+    dirname = getenv("HOME");
+    if (dirname != NULL) {
+	sprintf(paths.dotdir, "%s/.gretl/", dirname);
     } 
 #endif
 
@@ -2626,7 +2639,8 @@ static int maybe_transcribe_path (char *targ, char *src,
 
    gretldir
    gnuplot (but not on MS Windows)
-   tramo, x12a, rbinpath, rlibpath, oxlpath, octpath, dbhost
+   tramo, x12a, rbinpath, rlibpath, oxlpath, octpath, statapath,
+     pypath, dbhost
 
    * paths.workdir is updated via the separate working directory
      dialog
@@ -2659,6 +2673,8 @@ int gretl_update_paths (ConfigPaths *cpaths, gretlopt opt)
     ndelta += maybe_transcribe_path(paths.rbinpath, cpaths->rbinpath, 0);
     ndelta += maybe_transcribe_path(paths.oxlpath, cpaths->oxlpath, 0);
     ndelta += maybe_transcribe_path(paths.octpath, cpaths->octpath, 0);
+    ndelta += maybe_transcribe_path(paths.statapath, cpaths->statapath, 0);
+    ndelta += maybe_transcribe_path(paths.pypath, cpaths->pypath, 0);
 
 #ifdef USE_RLIB
     if (maybe_transcribe_path(paths.rlibpath, cpaths->rlibpath, 0)) {
@@ -2719,6 +2735,10 @@ static void load_default_path (char *targ)
 	sprintf(targ, "%s\\OxMetrics6\\Ox\\bin\\oxl.exe", progfiles);
     } else if (targ == paths.octpath) {
 	strcpy(targ, "C:\\Octave\\3.2.3_gcc-4.0.0\\bin\\octave.exe");
+    } else if (targ == paths.statapath) {
+	sprintf(targ, "%s\\Stata\\stata.exe", progfiles);
+    } else if (targ == paths.pypath) {
+	strcpy(targ, "python.exe"); /* ?? */
     } else if (targ == paths.pngfont) {
 	strcpy(targ, "verdana 8");
     }
@@ -2754,6 +2774,22 @@ static void load_default_workdir (char *targ)
 
 static void load_default_path (char *targ)
 {
+#ifdef OS_OSX
+    const char *app_paths[] = {
+	"/Applications/OxMetrics6/ox/bin/oxl",
+	"/Applications/Octave.app/Contents/Resources/bin/octave",
+	"/Applications/Stata/Stata.app/Contents/MacOS/Stata",
+	"python"
+    };
+#else
+    const char *app_paths[] = {
+	"oxl",
+	"octave",
+	"stata",
+	"python"
+    };
+#endif  
+
     if (targ == paths.workdir) {
 	load_default_workdir(targ);
     } else if (targ == paths.dbhost) {
@@ -2765,13 +2801,13 @@ static void load_default_path (char *targ)
 	strcpy(targ, "x12a");
 #else
 	*targ = '\0';
-#endif /* HAVE_X12A */
+#endif
     } else if (targ == paths.tramo) {
 #ifdef HAVE_TRAMO
 	strcpy(targ, "tramo");
 #else
 	*targ = '\0';
-#endif /* HAVE_TRAMO */
+#endif
     } else if (targ == paths.rbinpath) {
 	strcpy(paths.rbinpath, "R");
     } else if (targ == paths.rlibpath) {
@@ -2779,22 +2815,18 @@ static void load_default_path (char *targ)
 	strcpy(paths.rlibpath, RLIBPATH);
 #else
 	*paths.rlibpath = '\0';
-#endif /* RLIBPATH */
+#endif
     } else if (targ == paths.oxlpath) {
-#ifdef OSX_BUILD
-	strcpy(paths.oxlpath, "/Applications/OxMetrics6/ox/bin/oxl");
-#else
-	strcpy(paths.oxlpath, "oxl");
-#endif /* OSX_BUILD */
+	strcpy(paths.oxlpath, app_paths[0]);
     } else if (targ == paths.octpath) {
-#ifdef OSX_BUILD
-	strcpy(paths.octpath, 
-	       "/Applications/Octave.app/Contents/Resources/bin/octave");
-#else
-	strcpy(paths.octpath, "octave");
-#endif /* OSX_BUILD */
+	strcpy(paths.octpath, app_paths[1]);
+    } else if (targ == paths.statapath) {
+	strcpy(paths.statapath, app_paths[2]);
+    } else if (targ == paths.pypath) {
+	strcpy(paths.pypath, app_paths[3]);
     } else if (targ == paths.pngfont) {
-#ifdef OSX_BUILD
+#ifdef OS_OSX
+	/* FIXME native quartz */
 	strcpy(targ, "Sans 9");
 #else
 	if (chinese_locale()) {
@@ -2802,7 +2834,7 @@ static void load_default_path (char *targ)
 	} else {
 	    strcpy(targ, "Vera 9");
 	}
-#endif /* OSX_BUILD */	
+#endif
     }
 }
 
@@ -2859,6 +2891,8 @@ static void copy_paths_with_fallback (ConfigPaths *cpaths)
     path_init(paths.rlibpath, cpaths->rlibpath, 0);
     path_init(paths.oxlpath, cpaths->oxlpath, 0);
     path_init(paths.octpath, cpaths->octpath, 0);
+    path_init(paths.statapath, cpaths->statapath, 0);
+    path_init(paths.pypath, cpaths->pypath, 0);
 
     /* graphing font */
     path_init(paths.pngfont, cpaths->pngfont, 0);
@@ -2884,10 +2918,18 @@ int gretl_set_paths (ConfigPaths *cpaths, gretlopt opt)
     *paths.plotfile = '\0';
 
     initialize_gretldir(cpaths->gretldir, opt);
-    err0 = initialize_dotdir();
+
+    if (!(opt & OPT_D)) {
+	/* OPT_D says dotdir = workdir */
+	err0 = initialize_dotdir();
+    }
 
     copy_paths_with_fallback(cpaths);
 
+    if (opt & OPT_D) {
+	strcpy(paths.dotdir, paths.workdir);
+    }
+
     if (strcmp(paths.dotdir, paths.workdir)) { 
 	err1 = validate_writedir(paths.workdir);
 	if (err1) {
@@ -2904,7 +2946,7 @@ int gretl_set_paths (ConfigPaths *cpaths, gretlopt opt)
 	}
     }
 
-#if defined(WIN32) || defined(OSX_BUILD) 
+#if defined(WIN32) || defined(OS_OSX) 
     shelldir_init(paths.workdir);
 #else
     /* if on Linux, respect the "real" CWD */
@@ -2938,8 +2980,7 @@ const char *gretl_maybe_switch_dir (const char *fname)
     if (fname[0] == '~' && fname[1] == '/') {
 	char *home = getenv("HOME");
 	
-	if (home != NULL) {
-	    chdir(home);
+	if (home != NULL && chdir(home) == 0) {
 	    fname += 2;
 	}
     } else if (!g_path_is_absolute(fname)) {
@@ -3169,12 +3210,21 @@ static int rc_bool (const char *s)
 
 void get_gretl_rc_path (char *rcfile)
 {
-    char *home = getenv("HOME");
+    char *path = getenv("GRETL_CONFIG_FILE");
 
-    if (home != NULL) {
-	sprintf(rcfile, "%s/.gretl2rc", home);
+    if (path != NULL) {
+	*rcfile = '\0';
+	strncat(rcfile, path, FILENAME_MAX - 1);
+#if 0
+	fprintf(stderr, "rcfile from env: '%s'\n", rcfile);
+#endif
     } else {
-	strcpy(rcfile, ".gretl2rc");
+	path = getenv("HOME");
+	if (path != NULL) {
+	    sprintf(rcfile, "%s/.gretl2rc", path);
+	} else {
+	    strcpy(rcfile, ".gretl2rc");
+	}
     }
 }
 
@@ -3191,6 +3241,7 @@ int cli_read_rc (void)
     char rcfile[FILENAME_MAX];
     char line[MAXLEN], key[32], val[MAXLEN];
     char dbproxy[PROXLEN] = {0};
+    gretlopt paths_opt = OPT_NONE;
     int usecwd = 0;
     int use_proxy = 0;
     int err = 0;
@@ -3215,13 +3266,17 @@ int cli_read_rc (void)
 	    *val = '\0';
 	    /* get the string that follows " = " */ 
 	    strncat(val, line + strlen(key) + 3, MAXLEN - 1);
-	    gretl_chopstr(val); 
+	    gretl_strstrip(val); 
 	    if (!strcmp(key, "gretldir")) {
 		strncat(cpaths.gretldir, val, MAXLEN - 1);
 	    } else if (!strcmp(key, "gnuplot")) {
 		strncat(cpaths.gnuplot, val, MAXLEN - 1);
 	    } else if (!strcmp(key, "userdir")) {
 		strncat(cpaths.workdir, val, MAXLEN - 1);
+	    } else if (!strcmp(key, "no_dotdir")) {
+		if (rc_bool(val)) {
+		    paths_opt = OPT_D;
+		}
 	    } else if (!strcmp(key, "shellok")) {
 		libset_set_bool(SHELL_OK, rc_bool(val));
 	    } else if (!strcmp(key, "usecwd")) {
@@ -3245,6 +3300,10 @@ int cli_read_rc (void)
 		strncat(cpaths.oxlpath, val, MAXLEN - 1);
 	    } else if (!strcmp(key, "octave")) {
 		strncat(cpaths.octpath, val, MAXLEN - 1);
+	    } else if (!strcmp(key, "stata")) {
+		strncat(cpaths.statapath, val, MAXLEN - 1);
+	    } else if (!strcmp(key, "python")) {
+		strncat(cpaths.pypath, val, MAXLEN - 1);
 	    } else if (!strcmp(key, "Png_font")) {
 		strncat(cpaths.pngfont, val, 128 - 1);
 	    } else if (!strcmp(key, "Gp_colors")) {
@@ -3277,19 +3336,23 @@ int cli_read_rc (void)
  bailout:
 
     if (err) {
-	gretl_set_paths(&cpaths, OPT_NONE);
+	gretl_set_paths(&cpaths, paths_opt);
     } else {
-	err = gretl_set_paths(&cpaths, OPT_NONE);
+	err = gretl_set_paths(&cpaths, paths_opt);
     }
 
     gretl_www_init(cpaths.dbhost, dbproxy, use_proxy);
 
+#if 0
+    show_paths();
+#endif
+
     return err;
 }
 
 #endif /* !WIN32 */
 
-#ifdef OSX_BUILD
+#ifdef OS_OSX
 
 const char *gretl_app_support_dir (void)
 {
