@@ -62,6 +62,8 @@ struct ConfigPaths_ {
     char rlibpath[MAXLEN];
     char oxlpath[MAXLEN];
     char octpath[MAXLEN];
+    char statapath[MAXLEN];
+    char pypath[MAXLEN];
     char dbhost[64];
     char pngfont[128];
 };
@@ -188,6 +190,10 @@ const char *gretl_oxl_path (void);
 
 const char *gretl_octave_path (void);
 
+const char *gretl_stata_path (void);
+
+const char *gretl_python_path (void);
+
 const char *gretl_current_dir (void);
 
 void gretl_set_current_dir (const char *s);
@@ -206,7 +212,7 @@ int cli_read_rc (void);
 
 #endif
 
-#ifdef OSX_BUILD
+#ifdef OS_OSX
 
 const char *gretl_app_support_dir (void);
 
