@@ -881,6 +881,11 @@ catch_setobs_errors (const char *stobs, int pd, int n, int min, gretlopt opt)
     return err;
 }
 
+static int likely_calendar_obs_string (const char *s)
+{
+    return strchr(s, '-') != NULL || strchr(s, '/') != NULL;
+}
+
 #define recognized_ts_frequency(f) (f == 4 || f == 12 || f == 24)
 
 /**
@@ -943,13 +948,12 @@ int set_obs (const char *line, DATASET *dset, gretlopt opt)
     }
 
     /* truncate stobs if not a calendar date */
-    if (strchr(stobs, '/') != NULL) {
+    if (likely_calendar_obs_string(stobs)) {
 	dated = 1;
     } else {
 	stobs[8] = '\0';
     }
 
-    /* does the supplied frequency make sense? */
     if (pd < 1 || (dset->n > 0 && pd > dset->n && opt != OPT_T)) {
 	gretl_errmsg_sprintf(_("frequency (%d) does not make seem to make sense"), pd);
 	return 1;
@@ -1101,7 +1105,7 @@ double gretl_double_from_string (const char *s, int *err)
     }
 
     if (gretl_is_scalar(s)) {
-	x = gretl_scalar_get_value(s);
+	x = gretl_scalar_get_value(s, NULL);
     } else {
 	*err = E_DATA;
 	x = NADBL;
@@ -1147,7 +1151,7 @@ int gretl_int_from_string (const char *s, int *err)
     if (*test == '\0') {
 	return n;
     } else if (gretl_is_scalar(s)) {
-	x = gretl_scalar_get_value(s);
+	x = gretl_scalar_get_value(s, NULL);
 	if (na(x)) {
 	    *err = E_MISSDATA;
 	} else if (fabs(x) > INT_MAX) {
@@ -1240,16 +1244,14 @@ GretlType gretl_type_from_name (const char *s, const DATASET *dset)
 double *copyvec (const double *src, int n)
 {
     double *targ = NULL;
-    int i;
+    size_t sz = n * sizeof *targ;
 
     if (n > 0 && src != NULL) {
-	targ = malloc(n * sizeof *targ);
+	targ = malloc(sz);
     }
 
     if (targ != NULL) {
-	for (i=0; i<n; i++) {
-	    targ[i] = src[i];
-	}
+	memcpy(targ, src, sz);
     }
 
     return targ;
@@ -1564,16 +1566,22 @@ int gretl_copy_file (const char *src, const char *dest)
     return 0;
 }  
 
-/* handle the case of "delete bundle[key]" */
+/* handle the case of "delete b[key]" or "delete b.key" */
 
 static int maybe_delete_bundle_value (const char *s, PRN *prn)
 {
     char bname[VNAMELEN];
     char key[VNAMELEN];
     char fmt[16];
+    int brackets = 0;
     int err = 0;
 
-    sprintf(fmt, "%%%d[^[][%%%d[^]]", VNAMELEN-1, VNAMELEN-1);
+    if (strchr(s, '[')) {
+	sprintf(fmt, "%%%d[^[][%%%d[^]]", VNAMELEN-1, VNAMELEN-1);
+	brackets = 1;
+    } else {
+	sprintf(fmt, "%%%d[^.].%%%ds", VNAMELEN-1, VNAMELEN-1);
+    }
 
     if (sscanf(s, fmt, bname, key) == 2) {
 	gretl_bundle *bundle;
@@ -1582,7 +1590,7 @@ static int maybe_delete_bundle_value (const char *s, PRN *prn)
 	bundle = get_bundle_by_name(bname);
 	if (bundle == NULL) {
 	    err = E_UNKVAR;
-	} else {
+	} else if (brackets) {
 	    if (*key == '"') {
 		s = gretl_unquote(key, &err);
 	    } else if (gretl_is_string(key)) {
@@ -1590,6 +1598,8 @@ static int maybe_delete_bundle_value (const char *s, PRN *prn)
 	    } else {
 		err = E_UNKVAR;
 	    }
+	} else {
+	    s = key;
 	}
 
 	if (!err) {
