@@ -51,8 +51,6 @@ int win_run_sync (char *cmdline, const char *currdir);
 
 int gretl_spawn (char *cmdline);
 
-int gretl_spawn_with_fixup (char *prog, char *arg);
-
 int gretl_shell (const char *arg, PRN *prn);
 
 int gretl_win32_grab_output (const char *cmdline, char **sout);
@@ -79,6 +77,9 @@ void *win32_memalign (size_t size, size_t alignment);
 
 void win32_aligned_free (void *mem);
 
+char *strptime (const char *buf, const char *format, 
+		struct tm *timeptr);
+
 #endif /* WIN32 */
 
 #endif /* GRETL_WIN32_H */
