@@ -259,7 +259,7 @@ int llc_test_driver (const char *parm, const int *list,
     } else if (integer_string(parm)) {
 	p0 = atoi(parm);
     } else if (gretl_is_scalar(parm)) {
-	p0 = gretl_scalar_get_value(parm);
+	p0 = gretl_scalar_get_value(parm, NULL);
     } else {
 	err = E_DATA;
     }
@@ -296,7 +296,7 @@ MODEL quantreg_driver (const char *parm, const int *list,
     }
 
     if (err) {
-	gretl_model_init(&mod);
+	gretl_model_init(&mod, dset);
 	mod.errcode = err;
     } else {
 	mod = quantreg(tau, list, dset, opt, prn);
@@ -318,6 +318,8 @@ MODEL logit_probit (int *list, DATASET *dset, int ci,
 
     if (ci == LOGIT && (opt & OPT_M)) {
 	return multinomial_logit(list, dset, opt, prn);
+    } else if (ci == PROBIT && (opt & OPT_E)) {
+	return reprobit_model(list, dset, opt, prn);
     } else if (gretl_isdummy(dset->t1, dset->t2, dset->Z[yv])) {
 	if (ci == LOGIT) {
 	    return binary_logit(list, dset, opt, prn);
@@ -374,7 +376,7 @@ MODEL tobit_driver (const int *list, DATASET *dset,
     }
 
     if (err) {
-	gretl_model_init(&model);
+	gretl_model_init(&model, dset);
 	model.errcode = err;
 	return model;
     }
