@@ -59,6 +59,11 @@ extern "C" {
 # endif
 #endif
 
+#define YMD_READ_FMT     "%d-%d-%d" 
+#define YMD_WRITE_FMT    "%d-%02d-%02d"
+#define YMD_WRITE_Y2_FMT "%02d-%02d-%02d"
+#define YMD_WRITE_Y4_FMT "%04d-%02d-%02d"
+
 #ifdef ENABLE_NLS
 # include "libintl.h"
 # include "locale.h"
@@ -278,6 +283,7 @@ typedef struct DATASET_ {
     char *restriction;  /* record of sub-sampling restriction */
     char *padmask;      /* record of padding to re-balance panel data */
     int auxiliary;      /* = 0 for regular dataset, 1 for aux dataset */
+    double *pantime;    /* panel-data time values (e.g. years) */
 } DATASET;
 
 typedef struct VMatrix_ {
