@@ -40,6 +40,8 @@ enum {
     AUTO_LAG_NEWEYWEST
 };
 
+/* state flags */
+
 enum {
     STATE_USE_CWD         = 1 << 0,  /* store: use current dir as default */
     STATE_ECHO_ON         = 1 << 1,  /* echoing commands or not */
@@ -61,7 +63,8 @@ enum {
     STATE_LOOP_PROG       = 1 << 17, /* progressive loop is in progress */
     STATE_BFGS_RSTEP      = 1 << 18, /* use Richardson method in BFGS numerical
 					gradient */
-    STATE_DPDSTYLE_ON     = 1 << 19  /* emulate dpd in dynamic panel data models */
+    STATE_DPDSTYLE_ON     = 1 << 19, /* emulate dpd in dynamic panel data models */
+    STATE_OPENMP_ON       = 1 << 20  /* using openmp */
 };    
 
 /* for values that really want a non-negative integer */
@@ -113,6 +116,7 @@ struct set_vars_ {
 #define WARNINGS "warnings"
 #define GRETL_DEBUG "debug"
 #define BLAS_NMK_MIN "blas_nmk_min"
+#define MP_NMK_MIN "mp_nmk_min"
 
 #define libset_boolvar(s) (!strcmp(s, ECHO) || \
                            !strcmp(s, MESSAGES) || \
@@ -132,7 +136,8 @@ struct set_vars_ {
 			   !strcmp(s, R_FUNCTIONS) || \
 			   !strcmp(s, R_LIB) || \
 			   !strcmp(s, BFGS_RSTEP) || \
-			   !strcmp(s, DPDSTYLE))
+			   !strcmp(s, DPDSTYLE) || \
+			   !strcmp(s, USE_OPENMP))
 
 #define libset_double(s) (!strcmp(s, CONV_HUGE) || \
 			  !strcmp(s, BFGS_TOLER) || \
@@ -156,7 +161,8 @@ struct set_vars_ {
 		       !strcmp(s, VECM_NORM) || \
 		       !strcmp(s, GRETL_OPTIM) || \
 		       !strcmp(s, GRETL_DEBUG) || \
-		       !strcmp(s, BLAS_NMK_MIN))
+		       !strcmp(s, BLAS_NMK_MIN) || \
+		       !strcmp(s, MP_NMK_MIN))
 
 /* global state */
 set_vars *state;
@@ -164,9 +170,10 @@ static int gretl_debug;
 static int user_mp_bits;
 static int R_functions;
 static int R_lib = 1;
-static int csv_digits;
+static int csv_digits = UNSET_INT;
 static char data_delim = ',';
 static char data_export_decpoint = '.';
+static int mp_nmk_min = 65535;
 
 static int boolvar_get_flag (const char *s);
 static const char *hac_lag_string (void);
@@ -406,6 +413,70 @@ static void state_vars_copy (set_vars *sv)
     robust_opts_copy(&sv->ropts);
 }
 
+int libset_use_openmp (int n)
+{
+#if defined(_OPENMP)
+    if (state == NULL || !(state->flags & STATE_OPENMP_ON)) {
+	return 0;
+    } else if (mp_nmk_min >= 0 && n > mp_nmk_min) {
+	return 1;
+    }
+#endif
+
+    return 0;
+}
+
+#if defined(_OPENMP)
+# if defined(WIN32)
+# include <windows.h>
+# elif defined(OS_OSX)
+# include <sys/param.h>
+# include <sys/sysctl.h>
+# endif
+
+static int openmp_by_default (void)
+{
+    int num_cores = 1, ret = 0;
+# if defined(WIN32)
+    SYSTEM_INFO sysinfo;
+
+    GetSystemInfo(&sysinfo);
+    num_cores = sysinfo.dwNumberOfProcessors;
+# elif defined(OS_OSX)
+    int mib[2] = {CTL_HW, HW_NCPU}; /* or CTL_KERN, KERN_MAXPROC ? */
+    size_t len = sizeof num_cores;
+
+    if (sysctl(mib, 2, &num_cores, &len, NULL, 0) == -1) {
+	perror("could not determine number of cpus available");
+    }    
+# else
+    num_cores = sysconf(_SC_NPROCESSORS_ONLN);
+# endif
+    ret = num_cores > 1;
+    if (ret) {
+	/* one can use the environment to turn this off */
+	char *envstr = getenv("GRETL_USE_OPENMP");
+
+	if (envstr != NULL && !strcmp(envstr, "0")) {
+	    ret = 0;
+	}
+    }
+
+#if 0   
+    if (ret) {
+	fprintf(stderr, "num_cores = %d, using OpenMP by default\n",
+		num_cores);
+    } else {
+	fprintf(stderr, "num_cores = %d, not using OpenMP by default\n",
+		num_cores);
+    }
+#endif	
+
+    return ret;
+}
+
+#endif /* _OPENMP */
+
 static void state_vars_init (set_vars *sv)
 {
 #if PDEBUG
@@ -413,12 +484,17 @@ static void state_vars_init (set_vars *sv)
 #endif
     sv->flags = STATE_ECHO_ON | STATE_MSGS_ON | STATE_WARN_ON | 
 	STATE_HALT_ON_ERR | STATE_SKIP_MISSING;
+#if defined(_OPENMP)
+    if (openmp_by_default()) {
+	sv->flags |= STATE_OPENMP_ON;
+    }
+#endif
     sv->seed = 0;
     sv->conv_huge = 1.0e100;
     sv->horizon = UNSET_INT;
     sv->bootrep = 1000;
     sv->nls_toler = NADBL;
-    sv->loop_maxiter = 250;
+    sv->loop_maxiter = 1000;
     sv->rq_maxiter = 1000;
     sv->gmm_maxiter = 250;
     sv->vecm_norm = NORM_PHILLIPS;
@@ -741,7 +817,7 @@ static int libset_get_scalar (const char *var, const char *arg,
     }
 
     if (gretl_is_scalar(arg)) {
-	x = gretl_scalar_get_value(arg);
+	x = gretl_scalar_get_value(arg, NULL);
     } else {
 	gretl_errmsg_sprintf("'%s': not a scalar", arg);
 	return E_UNKVAR;
@@ -857,7 +933,7 @@ static int parse_libset_int_code (const char *key,
     }	
 
     if (err) {
-	gretl_errmsg_sprintf("%s: invalid value '%s'\n", key, val);
+	gretl_errmsg_sprintf(_("%s: invalid value '%s'"), key, val);
     }
 
     return err;
@@ -996,6 +1072,47 @@ static int set_initvals (const char *s, PRN *prn)
     return err;
 }
 
+static int set_panel_time (const char *s, DATASET *dset)
+{
+    char vname[VNAMELEN];
+    int err = 0;
+
+    if (!dataset_is_panel(dset)) {
+	return E_PDWRONG;
+    }
+
+    /* skip past "set panel_time" */
+    s += 14;
+
+    if (gretl_scan_varname(s, vname) != 1 || !strcmp(vname, "none")) {
+	dataset_set_panel_time(dset, NULL);
+    } else {
+	gretl_matrix *m = get_matrix_by_name(vname);
+	double *tvals = NULL;
+
+	if (m != NULL) {
+	    if (m->rows < dset->pd) {
+		err = E_PDWRONG;
+	    } else {
+		tvals = m->val;
+	    }
+	} else {
+	    int v = current_series_index(dset, vname);
+
+	    if (v < 0) {
+		err = E_DATA;
+	    } else {
+		tvals = dset->Z[v];
+	    }
+	}
+	if (!err) {
+	    err = dataset_set_panel_time(dset, tvals);
+	}
+    }
+
+    return err;
+}
+
 static int set_matmask (const char *s, const DATASET *dset,
 			PRN *prn)
 {
@@ -1296,8 +1413,9 @@ static int print_settings (PRN *prn, gretlopt opt)
 	    pprintf(prn, "set csv_delim %s\n", arg_from_delim(data_delim));
 	}
 	pprintf(prn, "set csv_na %s\n", get_csv_na_string());
-    }	
+    }
 
+    libset_print_int(CSV_DIGITS, prn, opt);	    
     libset_print_bool(ECHO, prn, opt);
     libset_print_bool(FORCE_DECP, prn, opt);
     libset_print_bool(HALT_ON_ERR, prn, opt);
@@ -1484,6 +1602,18 @@ static int write_or_read_settings (gretlopt opt, PRN *prn)
     return err;
 }
 
+static int check_set_bool (const char *setobj, const char *setarg)
+{
+    if (boolean_on(setarg)) {
+	return libset_set_bool(setobj, 1);
+    } else if (boolean_off(setarg)) {
+	return libset_set_bool(setobj, 0);
+    } else {
+	gretl_errmsg_sprintf(_("%s: invalid value '%s'"), setobj, setarg);
+	return E_PARSE;
+    }
+}
+
 int execute_set_line (const char *line, DATASET *dset, 
 		      gretlopt opt, PRN *prn)
 {
@@ -1505,8 +1635,8 @@ int execute_set_line (const char *line, DATASET *dset,
 	return print_settings(prn, OPT_D);
     }
 
-    /* specials which need the whole line */
     if (nw > 1) {
+	/* specials which need the whole line */
 	if (!strcmp(setobj, "initvals")) {
 	    return set_initvals(line, prn);
 	} else if (!strcmp(setobj, "matrix_mask")) {
@@ -1515,7 +1645,9 @@ int execute_set_line (const char *line, DATASET *dset,
 	    return set_shelldir(line);
 	} else if (!strcmp(setobj, "workdir")) {
 	    return set_workdir(line);
-	} 
+	} else if (!strcmp(setobj, "panel_time")) {
+	    return set_panel_time(line, dset);
+	}
     }
 
     if (nw == 1) {
@@ -1532,7 +1664,7 @@ int execute_set_line (const char *line, DATASET *dset,
 	if (!strcmp(setobj, "csv_na")) {
 	    set_csv_na_string(setarg);
 	    return 0;
-	} else if (!strcmp(setobj, "csv_digits")) {
+	} else if (!strcmp(setobj, CSV_DIGITS)) {
 	    set_csv_digits(setarg);
 	    return 0;
 	}	    
@@ -1543,10 +1675,14 @@ int execute_set_line (const char *line, DATASET *dset,
 	    if (!strcmp(setobj, SHELL_OK)) {
 		pprintf(prn, "You can only set this variable "
 			"via the gretl GUI\n");
-	    } else if (boolean_on(setarg)) {
-		err = libset_set_bool(setobj, 1);
-	    } else if (boolean_off(setarg)) {
-		err = libset_set_bool(setobj, 0);
+	    } else if (!strcmp(setobj, USE_OPENMP)) {
+#if defined(_OPENMP)
+		err = check_set_bool(setobj, setarg);
+#else
+		pprintf(prn, "Warning: openmp not supported\n");
+#endif
+	    } else {
+		err = check_set_bool(setobj, setarg);
 	    }
 	} else if (libset_double(setobj)) {
 	    if (default_ok(setobj) && default_str(setarg)) {
@@ -1825,10 +1961,6 @@ static int intvar_min_max (const char *s, int *min, int *max,
 
 int libset_set_int (const char *key, int val)
 {
-    int min = 0, max = 0;
-    int *ivar = NULL;
-    int err = 0;
-
     if (check_for_state()) {
 	return 1;
     }
@@ -1836,19 +1968,26 @@ int libset_set_int (const char *key, int val)
     if (!strcmp(key, BLAS_NMK_MIN)) {
 	set_blas_nmk_min(val);
 	return 0;
-    }
+    } else if (!strcmp(key, MP_NMK_MIN)) {
+	mp_nmk_min = val;
+	return 0;
+    } else {
+	int min = 0, max = 0;
+	int *ivar = NULL;
+	int err = 0;
 
-    err = intvar_min_max(key, &min, &max, &ivar);
+	err = intvar_min_max(key, &min, &max, &ivar);
 
-    if (!err) {
-	if (val < min || val >= max || ivar == NULL) {
-	    err = E_DATA;
-	} else {
-	    *ivar = val;
+	if (!err) {
+	    if (val < min || val >= max || ivar == NULL) {
+		err = E_DATA;
+	    } else {
+		*ivar = val;
+	    }
 	}
-    }
 
-    return err;
+	return err;
+    }
 }
 
 static int boolvar_get_flag (const char *s)
@@ -1887,6 +2026,8 @@ static int boolvar_get_flag (const char *s)
 	return STATE_BFGS_RSTEP;
     } else if (!strcmp(s, DPDSTYLE)) {
 	return STATE_DPDSTYLE_ON;
+    } else if (!strcmp(s, USE_OPENMP)) {
+	return STATE_OPENMP_ON;
     } else {
 	fprintf(stderr, "libset_get_bool: unrecognized "
 		"variable '%s'\n", s);	
@@ -2410,7 +2551,7 @@ static void set_csv_digits (const char *s)
 {
     int k = atoi(s);
 
-    if (k >= 0 && k < 26) {
+    if (k > 0 && k < 26) {
 	csv_digits = k;
     }
 }
@@ -2479,5 +2620,3 @@ int libset_read_script (const char *fname)
 {
     return real_libset_read_script(fname, NULL);
 }
-
-
