@@ -80,6 +80,7 @@ typedef enum {
 #define CSV_DIGITS       "csv_digits"
 #define NADARWAT_TRIM    "nadarwat_trim"
 #define CONV_HUGE        "huge"
+#define USE_OPENMP       "openmp"
 
 typedef int (*ITER_PRINT_FUNC) (int, PRN *);
 typedef void (*SHOW_ACTIVITY_FUNC) (void);
@@ -109,6 +110,8 @@ int libset_set_int (const char *key, int val);
 
 int is_libset_var (const char *s);
 
+int libset_use_openmp (int n);
+
 /* GUI setter functions */
 void set_xsect_hccme (const char *s);
 void set_tseries_hccme (const char *s);
