@@ -20,6 +20,9 @@
 #include "libgretl.h"
 #include "usermat.h"
 #include "matrix_extra.h"
+#include "libset.h"
+
+#include <errno.h>
 
 /**
  * SECTION:matrix_extra
@@ -953,6 +956,7 @@ static int skip_matrix_comment (FILE *fp, int *err)
 /**
  * gretl_matrix_read_from_text:
  * @fname: name of text file.
+ * @import: non-zero means we're importing via dotdir.
  * @err: location to receive error code.
  *
  * Reads a matrix from a text file by the name @fname; the column
@@ -964,14 +968,22 @@ static int skip_matrix_comment (FILE *fp, int *err)
  * Returns: The matrix read from file, or NULL.
  */
 
-gretl_matrix *gretl_matrix_read_from_text (const char *fname, int *err)
+gretl_matrix *gretl_matrix_read_from_text (const char *fname, 
+					   int import, int *err)
 {
     int r, c, i, j, n;
     double x;
     gretl_matrix *A = NULL;
     FILE *fp;
 
-    fp = gretl_read_user_file(fname);
+    if (import) {
+	char targ[FILENAME_MAX];
+
+	build_path(targ, gretl_dotdir(), fname, NULL);
+	fp = gretl_fopen(targ, "r");
+    } else {
+	fp = gretl_read_user_file(fname);
+    }
 
     if (fp == NULL) {
 	*err = E_FOPEN;
@@ -1030,6 +1042,7 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname, int *err)
  * gretl_matrix_write_as_text:
  * @A: matrix.
  * @fname: name of file to write.
+ * @export: non-zero means we're exporting via dotdir.
  *
  * Writes the matrix @A to a plain text file by the name @fname; the
  * column separator is the tab. The number of rows and columns are
@@ -1039,7 +1052,8 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname, int *err)
  * Returns: 0 on successful completion, non-zero code on error.
  */
 
-int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname)
+int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname,
+				int export)
 {
     int r = A->rows;
     int c = A->cols;
@@ -1047,9 +1061,15 @@ int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname)
     FILE *fp;
     char d = '\t';
 
-    fname = gretl_maybe_switch_dir(fname);
+    if (export) {
+	char targ[FILENAME_MAX];
 
-    fp = gretl_fopen(fname, "w");
+	build_path(targ, gretl_dotdir(), fname, NULL);
+	fp = gretl_fopen(targ, "w");
+    } else {
+	fname = gretl_maybe_switch_dir(fname);
+	fp = gretl_fopen(fname, "w");
+    }
 
     if (fp == NULL) {
 	return E_FOPEN;
@@ -1081,7 +1101,12 @@ static void make_numstr (char *s, double x)
 {
     if (x == -0.0) {
 	x = 0.0;
-    }   
+    }
+
+    if (isnan(x)) {
+	strcpy(s, "nan");
+	return;
+    }
 
     sprintf(s, "%#.5g", x);
 
@@ -1114,7 +1139,7 @@ static int max_numchars (const gretl_matrix *m)
     int i, n = m->rows * m->cols;
     int c, cmax = 0;
 
-    for (i = 0; i < n && cmax < 6; i++) {
+    for (i=0; i<n && cmax<6; i++) {
 	sprintf(s, "%g", m->val[i]);
 	c = strlen(s);
 	if (c > cmax) {
@@ -1143,39 +1168,53 @@ static int max_label_length (const char **names, int n)
 
 static void 
 real_matrix_print_to_prn (const gretl_matrix *m, const char *msg, 
-			  int plain, const char **colnames, 
-			  PRN *prn)
+			  int plain, const char **colnames,
+			  const DATASET *dset, PRN *prn)
 {
     const char **rownames = NULL;
     char numstr[32];
+    char obs[OBSLEN];
     double x;
     int strwidth = 12;
     int rnamelen = 0;
-    int cmax = 0;
-    int cpad = 2;
+    int dated = 0;
+    int cmax = 0, cpad = 2;
+    int mt1 = 0, mt2 = 0;
     int i, j;
 
     if (prn == NULL) {
 	return;
     }
 
-    if (m == NULL || m->val == NULL) {
+    if (m == NULL) {
 	if (msg != NULL && *msg != '\0') {
 	    pprintf(prn, "%s: matrix is NULL\n", msg);
 	} else {
 	    pputs(prn, "matrix is NULL\n");
 	}
 	return;
+    } else if (m->rows == 0 || m->cols == 0) {
+	if (msg != NULL && *msg != '\0') {
+	    pprintf(prn, "%s: matrix is empty (%d x %d)\n", 
+		    msg, m->rows, m->cols);
+	} else {
+	    pprintf(prn, "matrix is empty (%d x %d)\n",
+		    m->rows, m->cols);
+	}
+	return;
+    }
+
+    dated = gretl_matrix_is_dated(m);
+    if (dated) {
+	mt1 = gretl_matrix_get_t1(m);
+	mt2 = gretl_matrix_get_t2(m);
     }
 
     /* @plain != 0 means skip the header stuff */
 
     if (msg != NULL && *msg != '\0' && !plain) {
 	pprintf(prn, "%s (%d x %d)", msg, m->rows, m->cols);
-	if (gretl_matrix_is_dated(m)) {
-	    int mt1 = gretl_matrix_get_t1(m);
-	    int mt2 = gretl_matrix_get_t2(m);
-
+	if (dated && dset == NULL) {
 	    pprintf(prn, " [t1 = %d, t2 = %d]\n\n", mt1 + 1, mt2 + 1);
 	} else {
 	    pputs(prn, "\n\n");
@@ -1192,12 +1231,15 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
     }
 
     rownames = gretl_matrix_get_rownames(m);
+
     if (rownames != NULL) {
 	rnamelen = max_label_length(rownames, m->rows);
+    } else if (dated && dset != NULL) {
+	rnamelen = max_obs_marker_length(dset);
     }
 
     if (colnames != NULL) {
-	if (rownames != NULL) {
+	if (rnamelen > 0) {
 	    bufspace(rnamelen + 1, prn);
 	}
 	for (j=0; j<m->cols; j++) {
@@ -1207,8 +1249,13 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
     }
 
     for (i=0; i<m->rows; i++) {
-	if (rownames != NULL) {
-	    pprintf(prn, "%*s ", rnamelen, rownames[i]);
+	if (rnamelen > 0) {
+	    if (rownames != NULL) {
+		pprintf(prn, "%*s ", rnamelen, rownames[i]);
+	    } else {
+		ntodate(obs, i + mt1, dset);
+		pprintf(prn, "%*s ", rnamelen, obs);
+	    }
 	}
 	for (j=0; j<m->cols; j++) {
 	    x = gretl_matrix_get(m, i, j);
@@ -1216,7 +1263,11 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 		if (colnames != NULL) {
 		    bufspace(strwidth - cmax - cpad, prn);
 		}
-		sprintf(numstr, "%g", x);
+		if (isnan(x)) {
+		    strcpy(numstr, "nan");
+		} else {
+		    sprintf(numstr, "%g", x);
+		}
 		pprintf(prn, "%*s ", cmax + cpad, numstr);
 	    } else {
 		make_numstr(numstr, x);
@@ -1241,7 +1292,7 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 void 
 gretl_matrix_print_to_prn (const gretl_matrix *m, const char *msg, PRN *prn)
 {
-    real_matrix_print_to_prn(m, msg, 0, NULL, prn);
+    real_matrix_print_to_prn(m, msg, 0, NULL, NULL, prn);
 }
 
 /**
@@ -1258,9 +1309,10 @@ gretl_matrix_print_to_prn (const gretl_matrix *m, const char *msg, PRN *prn)
 void gretl_matrix_print_with_col_heads (const gretl_matrix *m, 
 					const char *title,
 					const char **heads,
+					const DATASET *dset,
 					PRN *prn)
 {
-    real_matrix_print_to_prn(m, title, 0, heads, prn);
+    real_matrix_print_to_prn(m, title, 0, heads, dset, prn);
 }
 
 static void maybe_print_col_heads (const gretl_matrix *m, 
@@ -1340,7 +1392,7 @@ void gretl_matrix_print_with_format (const gretl_matrix *m,
     }
 
     if (gretl_is_null_matrix(m) || fmt == NULL || *fmt == '\0') {
-	real_matrix_print_to_prn(m, NULL, 1, NULL, prn);
+	real_matrix_print_to_prn(m, NULL, 1, NULL, NULL, prn);
     } else {
 	const char **rownames = NULL;
 	int llen = 0, intcast = 0;
@@ -1852,4 +1904,308 @@ int gretl_matrix_mp_ols (const gretl_vector *y, const gretl_matrix *X,
     return err;
 }
 
+/* following: quadrature sources based on John Burkhart's GPL'd
+   code at http://people.sc.fsu.edu/~jburkardt
+*/
 
+#define sign(x) (x < 0.0 ? -1.0 : 1.0)
+
+/* Diagonalize a Jacobi (symmetric tridiagonal) matrix. On entry, @d
+   contains the diagonal and @e the sub-diagonal. On exit, @d contains
+   quadrature nodes or abscissae and @z the square roots of the 
+   corresponding weights; @e is used as workspace.
+
+   This is a C adaptation of subroutine IMTQLX, by Sylvan Elhay,
+   Jaroslav Kautsky and John Burkardt. See
+   http://people.sc.fsu.edu/~jburkardt/f_src/quadrature_weights/
+   specifically, qw_golub_welsch.f90
+*/
+
+static int diag_jacobi (int n, double *d, double *e, double *z)
+{
+    double b, c, f, g;
+    double p, r, s;
+    int j, k, l, m = 0;
+    int maxiter = 30;
+    int i, ii, mml;
+    int err = 0;
+
+    if (n == 1) {
+	return 0;
+    }
+
+    e[n-1] = 0.0;
+
+    errno = 0;
+
+    for (l=1; l<=n; l++) {
+	j = 0;
+	for ( ; ; ) {
+	    for (m=l; m<=n; m++) {
+		if (m == n) {
+		    break;
+		}
+		p = fabs(d[m-1]) + fabs(d[m]);
+		if (fabs(e[m-1]) <= DBL_EPSILON * p) {
+		    break;
+		}
+	    }
+	    p = d[l-1];
+	    if (m == l) {
+		break;
+	    }
+	    if (j >= maxiter) {
+		fprintf(stderr, "diag_jacobi: iteration limit exceeded\n");
+		return E_NOCONV;
+	    }
+	    j++;
+	    g = (d[l] - p) / (2.0 * e[l-1]);
+	    r =  sqrt(g * g + 1.0);
+	    g = d[m-1] - p + e[l-1] / (g + fabs(r) * sign(g));
+	    s = 1.0;
+	    c = 1.0;
+	    p = 0.0;
+	    mml = m - l;
+
+	    for (ii=1; ii<=mml; ii++) {
+		i = m - ii;
+		f = s * e[i-1];
+		b = c * e[i-1];
+		if (fabs(g) <= fabs(f)) {
+		    c = g / f;
+		    r =  sqrt(c * c + 1.0);
+		    e[i] = f * r;
+		    s = 1.0 / r;
+		    c = c * s;
+		} else {
+		    s = f / g;
+		    r =  sqrt(s * s + 1.0);
+		    e[i] = g * r;
+		    c = 1.0 / r;
+		    s = s * c;
+		}
+		g = d[i] - p;
+		r = (d[i-1] - g) * s + 2.0 * c * b;
+		p = s * r;
+		d[i] = g + p;
+		g = c * r - b;
+		f = z[i];
+		z[i] = s * z[i-1] + c * f;
+		z[i-1] = c * z[i-1] - s * f;
+	    }
+
+	    d[l-1] = d[l-1] - p;
+	    e[l-1] = g;
+	    e[m-1] = 0.0;
+	}
+    }
+
+    /* sorting */
+
+    for (ii=2; ii<=m; ii++) {
+	i = ii - 1;
+	k = i;
+	p = d[i-1];
+	for (j=ii; j<=n; j++) {
+	    if (d[j-1] < p) {
+		k = j;
+		p = d[j-1];
+	    }
+	}
+	if (k != i) {
+	    d[k-1] = d[i-1];
+	    d[i-1] = p;
+	    p = z[i-1];
+	    z[i-1] = z[k-1];
+	    z[k-1] = p;
+	}
+    }
+
+    if (errno) {
+	/* some calculation went badly */
+	err = E_NAN;
+	errno = 0;
+    }
+
+    return err;
+}
+
+/* Construct the Jacobi matrix for a quadrature rule: @d
+   holds the diagonal, @e the subdiagonal.
+*/
+
+static double make_jacobi (int n, int kind, double *d, double *e)
+{
+    double z0 = 0.0;
+    int i;
+
+    if (kind == QUAD_LEGENDRE) {
+	double xi, xj;
+
+	z0 = 2.0;
+	for (i=1; i<=n; i++) {
+	    xi = i;
+	    xj = 2.0 * i;
+	    e[i-1] = sqrt(xi * xi / (xj * xj - 1.0));
+	}
+    } else if (kind == QUAD_LAGUERRE) {
+	z0 = 1.0; /* tgamma(1.0) */
+	for (i=1; i<=n; i++) {
+	    d[i-1] = 2.0 * i - 1.0;
+	    e[i-1] = i;
+	}
+    } else if (kind == QUAD_GHERMITE) {
+	z0 = tgamma(0.5);
+	for (i=1; i<=n; i++) {
+	    e[i-1] = sqrt(i / 2.0);
+	}
+    } 
+
+    return z0;
+}
+
+/* compute basic Gauss quadrature formula */
+
+static int gauss_quad_basic (int n, int kind, double *x, double *w)
+{
+    double z0, *tmp;
+    int i, err;
+
+    tmp = malloc(n * sizeof *tmp);
+    if (tmp == NULL) {
+	return E_ALLOC;
+    }
+
+    z0 = make_jacobi(n, kind, x, tmp);
+    w[0] = sqrt(z0);
+    err = diag_jacobi(n, x, tmp, w);
+    
+    if (!err) {
+	for (i=0; i<n; i++) {
+	    w[i] = w[i] * w[i];
+	}
+    }
+
+    free(tmp);
+
+    return err;
+}
+
+/* Scale Legendre quadrature rule to the interval (a, b) */
+
+static int legendre_scale (int n, double *x, double *w,
+			   double a, double b)
+{
+    double shft, slp;
+    int i;
+
+    if (fabs(b - a) <= DBL_EPSILON) {
+	fprintf(stderr, "legendre: |b - a| too small\n");
+	return E_DATA;
+    }
+
+    shft = (a + b) / 2.0;
+    slp = (b - a) / 2.0;
+
+    for (i=0; i<n; i++) {
+	x[i] = shft + slp * x[i];
+	w[i] = w[i] * slp;
+    }
+
+    return 0;
+}
+
+static void hermite_scale (int n, double *x, double *w,
+			   double mu, double sigma)
+{
+    double rtpi = sqrt(M_PI);
+    int i;
+
+    for (i=0; i<n; i++) {
+	x[i] = mu + M_SQRT2 * sigma * x[i];
+	w[i] /= rtpi;
+    }
+}
+
+/**
+ * gretl_quadrule_matrix_new:
+ * @n: the order (i.e. the number of abscissae and weights).
+ * @method: should be one of the values in #QuadMethod.
+ * @a: for method = QUAD_LEGENDRE, the lower limit of integration.
+ * @b: for method = QUAD_LEGENDRE, the upper limit of integration.
+ * @err: location to receive error code.
+ *
+ * Calculates a quadrature "rule" (i.e. a set of abscissae or
+ * nodes and assoicated weights) for use in numerical integration.
+ * The three supported methods are Gauss-Hermite, Gauss-Legendre
+ * and Gauss-Laguerre. The arguments @a and @b are ignored for
+ * methods other than Legendre: in the Gauss-Hermite case the
+ * integral that is approximated runs from minus infinity to
+ * plus infinity, and in the Laguerre case it runs from zero to
+ * plus infinity.
+ * 
+ * Returns: an @n x 2 matrix containing the abscissae in the first
+ * column and the weights in the second, or NULL on failure.
+ */
+
+gretl_matrix *gretl_quadrule_matrix_new (int n, int method, 
+					 double a, double b,
+					 int *err)
+{
+    gretl_matrix *m;
+
+    if (method < QUAD_GHERMITE || method >= QUAD_INVALID) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    if (n < 0) {
+	*err = E_DATA;
+	return NULL;
+    } else if (n == 0) {
+	return gretl_null_matrix_new();
+    }
+
+    m = gretl_zero_matrix_new(n, 2);
+
+    if (m == NULL) {
+	*err = E_ALLOC;
+    } else {
+	double *x = m->val;
+	double *w = x + n;
+
+	*err = gauss_quad_basic(n, method, x, w);
+
+	if (!*err) {
+	    if (method == QUAD_LEGENDRE) {
+		if (na(a) && na(b)) {
+		    ; /* default */
+		} else if (a == -1.0 && b == 1.0) {
+		    ; /* default */
+		} else {
+		    *err = legendre_scale(n, x, w, a, b);
+		}
+	    } else if (method == QUAD_GHERMITE) {
+#if 1 /* FIXME: change this */
+		hermite_scale(n, x, w, 0.0, 1.0);
+#else
+		if (!na(a) && !na(b)) {
+		    hermite_scale(n, x, w, a, b);
+		}
+#endif
+	    }
+	}
+    }
+
+    if (*err && m != NULL) {
+	gretl_matrix_free(m);
+	m = NULL;
+    }
+
+    return m;
+}
+
+gretl_matrix *gretl_gauss_hermite_matrix_new (int n, int *err)
+{
+    return gretl_quadrule_matrix_new(n, QUAD_GHERMITE, 0, 1, err);
+}
