@@ -565,7 +565,7 @@ static int nlspec_push_param (nlspec *s, const char *name,
     s->nparam = np + 1;
 
     if (type == GRETL_TYPE_DOUBLE) {
-	err = push_scalar_coeff(s, gretl_scalar_get_value(name));
+	err = push_scalar_coeff(s, gretl_scalar_get_value(name, NULL));
     } else {
 	gretl_matrix *m = get_matrix_by_name(name);
 	int k = gretl_vector_get_length(m);
@@ -953,7 +953,7 @@ static double get_mle_ll (const double *b, void *p)
     }
 
     if (s->lhtype == GRETL_TYPE_DOUBLE) {
-	s->crit = gretl_scalar_get_value(s->lhname);
+	s->crit = gretl_scalar_get_value(s->lhname, NULL);
 	return s->crit;
     }
 
@@ -1114,7 +1114,7 @@ static int get_nls_derivs (int T, int offset, double *g, double **G,
 	    }
 	} else if (scalar_deriv(spec, j)) {
 	    /* transcribe from scalar var to array g */
-	    x = gretl_scalar_get_value(spec->params[j].dname);
+	    x = gretl_scalar_get_value(spec->params[j].dname, NULL);
 	    for (t=0; t<T; t++) {
 		gi[t] = (spec->ci == MLE)? x : -x;
 	    }
@@ -1219,7 +1219,7 @@ static int get_mle_gradient (double *b, double *g, int n,
 		i++;
 	    }
 	} else if (scalar_deriv(spec, j)) {
-	    x = gretl_scalar_get_value(spec->params[j].dname);
+	    x = gretl_scalar_get_value(spec->params[j].dname, NULL);
 	    if (na(x)) {
 		fprintf(stderr, "NA in gradient calculation\n");
 		err = 1;
@@ -1404,7 +1404,7 @@ static gretl_matrix *ml_gradient_matrix (nlspec *spec, int *err)
 		    j++;
 		}
 	    } else if (scalar_deriv(spec, i)) {
-		x = gretl_scalar_get_value(spec->params[i].dname);
+		x = gretl_scalar_get_value(spec->params[i].dname, NULL);
 		for (t=0; t<T; t++) {
 		    gretl_matrix_set(G, t, j, x);
 		}
@@ -1794,7 +1794,7 @@ static MODEL GNR (nlspec *spec, DATASET *dset, PRN *prn)
        (derivatives) */
     gdset = create_auxiliary_dataset(spec->ncoeff + 2, dset->n, 0);
     if (gdset == NULL) {
-	gretl_model_init(&gnr);
+	gretl_model_init(&gnr, NULL);
 	gnr.errcode = E_ALLOC;
 	return gnr;
     }
@@ -1809,7 +1809,7 @@ static MODEL GNR (nlspec *spec, DATASET *dset, PRN *prn)
 
     if (glist == NULL) {
 	destroy_dataset(gdset);
-	gretl_model_init(&gnr);
+	gretl_model_init(&gnr, NULL);
 	gnr.errcode = E_ALLOC;
 	return gnr;
     }
@@ -3030,7 +3030,7 @@ static MODEL real_nl_model (nlspec *spec, DATASET *dset,
     int i, t, origv;
     int err = 0;
 
-    gretl_model_init(&nlmod);
+    gretl_model_init(&nlmod, dset);
 
     if (dset == NULL || dset->v == 0) {
 	gretl_errmsg_set(_("No dataset is in place"));
@@ -3527,8 +3527,8 @@ MODEL ivreg_via_gmm (const int *list, DATASET *dset, gretlopt opt)
     int rv = 0;
     int err = 0;
 
-    gretl_model_init(&olsmod);
-    gretl_model_init(&model);
+    gretl_model_init(&olsmod, dset);
+    gretl_model_init(&model, dset);
 
     err = ivreg_process_lists(list, &mlist, &ilist);
 
@@ -3630,10 +3630,9 @@ static int set_nlspec_from_model (const MODEL *pmod,
 
 static int push_scalar_parm (const char *name, double **px, int *n)
 {
-    double x = gretl_scalar_get_value(name);
+    double x = gretl_scalar_get_value(name, NULL);
     int err = 0;
 
-    x = gretl_scalar_get_value(name);
     if (na(x)) {
 	err = E_DATA;
     } else {
