@@ -24,6 +24,13 @@
 
 #define OPTDEBUG 0
 
+typedef enum {
+    OPT_NO_PARM = 0,
+    OPT_ACCEPTS_PARM,
+    OPT_NEEDS_PARM,
+    OPT_AMBIGUOUS
+} OptStatus;
+
 #define vcv_opt_ok(c) (MODEL_COMMAND(c) || c == ADD || c == OMIT)
 
 #define quiet_opt_ok(c) (MODEL_COMMAND(c) ||	\
@@ -158,6 +165,7 @@ struct gretl_option gretl_opts[] = {
     { DELEET,   OPT_D, "db", 0 },
     { DELEET,   OPT_F, "force", 0 },
     { DELEET,   OPT_L, "list", 0 },
+    { DELEET,   OPT_T, "type", 2 },
     { DIFFTEST, OPT_G, "sign", 0 },
     { DIFFTEST, OPT_R, "rank-sum", 0 },
     { DIFFTEST, OPT_I, "signed-rank", 0 },
@@ -266,6 +274,8 @@ struct gretl_option gretl_opts[] = {
     { JOIN,     OPT_A, "aggr", 2 },
     { JOIN,     OPT_D, "data", 2 },
     { JOIN,     OPT_R, "rev", 2 },
+    { JOIN,     OPT_T, "time", 2 },
+    { JOIN,     OPT_K, "tkey", 2 },
     { JOIN,     OPT_V, "verbose", 0 },
     { KALMAN,   OPT_C, "cross", 0 },
     { KALMAN,   OPT_D, "diffuse", 0 },
@@ -318,6 +328,8 @@ struct gretl_option gretl_opts[] = {
     { MODPRINT, OPT_O, "complete", 0 },
     { MODPRINT, OPT_R, "rtf", 0 },
     { MODPRINT, OPT_T, "tex", 0 },
+    { MODELTAB, OPT_C, "complete", 0 },
+    { MODELTAB, OPT_O, "output", 2 },
     { MPOLS,    OPT_O, "vcv", 0 },
     { MPOLS,    OPT_S, "simple-print", 0 },
     { NEGBIN,   OPT_G, "opg", 0 },
@@ -349,6 +361,8 @@ struct gretl_option gretl_opts[] = {
     { OMIT,     OPT_X, "chi-square", 0 },
     { OMIT,     OPT_I, "silent", 0 },
     { OMIT,     OPT_W, "test-only", 0 },
+    { OMIT,     OPT_T, "trend", 0 },      /* omit auto-trend: VAR only */
+    { OMIT,     OPT_E, "seasonals", 0 },  /* omit auto-seasonals: VAR only */
     { OPEN,     OPT_B, "progress-bar", 0 },
     { OPEN,     OPT_D, "drop-empty", 0 },
     { OPEN,     OPT_F, "fixed-cols", 2 },    
@@ -397,6 +411,8 @@ struct gretl_option gretl_opts[] = {
     { PROBIT,   OPT_R, "robust", 0 },
     { PROBIT,   OPT_C, "cluster", 2 },
     { PROBIT,   OPT_V, "verbose", 0 },
+    { PROBIT,   OPT_E, "random-effects", 0 },
+    { PROBIT,   OPT_G, "quadpoints", 2 },
     { QQPLOT,   OPT_R, "raw", 0 },
     { QQPLOT,   OPT_Z, "z-scores", 0 },      
     { QUANTREG, OPT_I, "intervals", 1 },
@@ -693,28 +709,6 @@ static int opt_is_valid (gretlopt opt, int ci, char c)
     return 0;
 }
 
-gretlopt valid_short_opt (int ci, char c)
-{
-    gretlopt opt = 0;
-    int i, ok;
-
-    for (i=0; flag_matches[i].c != '\0'; i++) {
-	if (c == flag_matches[i].c) {
-	    opt = flag_matches[i].o;
-	    break;
-	}
-    }
-
-    if (opt) {
-	ok = opt_is_valid(opt, ci, c);
-	if (!ok) {
-	    opt = 0;
-	}
-    }
-	
-    return opt;
-}
-
 /* See if at point @p (at which we've found '-') in string @s we
    might be at the start of an option flag: the previous character
    must be a space, and we must not be inside a quoted string.
@@ -944,7 +938,7 @@ double get_optval_double (int ci, gretlopt opt)
 	if (numeric_string(op->val)) {
 	    ret = dot_atof(op->val);
 	} else {
-	    ret = gretl_scalar_get_value(op->val);
+	    ret = gretl_scalar_get_value(op->val, NULL);
 	}
     }
 
@@ -974,11 +968,9 @@ int get_optval_int (int ci, gretlopt opt, int *err)
 	if (integer_string(op->val)) {
 	    ret = atoi(op->val);
 	} else {
-	    double x = gretl_scalar_get_value(op->val);
+	    double x = gretl_scalar_get_value(op->val, err);
 
-	    if (na(x)) {
-		*err = E_DATA;
-	    } else {
+	    if (!*err) {
 		ret = (int) x;
 	    }
 	}
@@ -1032,54 +1024,58 @@ int set_optval_string (int ci, gretlopt opt, const char *s)
     return real_push_option_param(ci, opt, gretl_strdup(s), 1);
 }
 
-gretlopt valid_long_opt (int ci, const char *lopt, OptStatus *status)
+static gretlopt valid_long_opt (int ci, const char *s, OptStatus *status)
 {
-    int opt = OPT_NONE;
+    gretlopt opt = OPT_NONE;
     int i;
 
     *status = 0;
 
-    if (*lopt == '\0') {
+    if (*s == '\0') {
 	return 0;
     }
 
-    if (vcv_opt_ok(ci) && !strcmp(lopt, "vcv")) {
+    if (vcv_opt_ok(ci) && !strcmp(s, "vcv")) {
 	return OPT_O;
     }
 
-    if (quiet_opt_ok(ci) && !strcmp(lopt, "quiet")) {
+    if (quiet_opt_ok(ci) && !strcmp(s, "quiet")) {
 	return OPT_Q;
     }
 
     /* start by looking for an exact match */
     for (i=0; gretl_opts[i].o != 0; i++) {
 	if (ci == gretl_opts[i].ci && 
-	    !strcmp(lopt, gretl_opts[i].longopt)) {
+	    !strcmp(s, gretl_opts[i].longopt)) {
 	    opt = gretl_opts[i].o;
 	    *status = gretl_opts[i].parminfo;
 	    break;
 	}
     }
 
-    /* if this failed, try for an abbreviation or extension */
+    /* if this failed, try for a unique abbreviation */
     if (opt == OPT_NONE) {
-	int len, len1, len2;
+	int optlen, slen = strlen(s);
+	int nmatch = 0;
 
-	len1 = strlen(lopt);
 	for (i=0; gretl_opts[i].o != 0; i++) {
-	    len2 = strlen(gretl_opts[i].longopt);
-	    len = (len2 > len1)? len1 : len2;
-	    if (ci == gretl_opts[i].ci && 
-		!strncmp(lopt, gretl_opts[i].longopt, len)) {
-		opt = gretl_opts[i].o;
-		*status = gretl_opts[i].parminfo;
-		break;
+	    if (ci == gretl_opts[i].ci) {
+		optlen = strlen(gretl_opts[i].longopt);
+		if (optlen > slen && !strncmp(s, gretl_opts[i].longopt, slen)) {
+		    opt = gretl_opts[i].o;
+		    *status = gretl_opts[i].parminfo;
+		    nmatch++;
+		}
 	    }
-	} 
+	}
+	if (nmatch > 1) {
+	    *status = OPT_AMBIGUOUS;
+	    return OPT_NONE;
+	}
     }  
 
     /* backward compatibility */
-    if (opt == OPT_NONE && !strcmp(lopt, "wald")) {
+    if (opt == OPT_NONE && !strcmp(s, "wald")) {
 	opt = OPT_W;
 	*status = 0;
     }
@@ -1118,6 +1114,10 @@ static int handle_optval (char *s, int ci, gretlopt opt, int status)
 	err = E_PARSE;
     }
 
+#if OPTDEBUG
+    fprintf(stderr, "handle_optval: got val = '%s'\n", val);
+#endif
+
     if (val == NULL && !err) {
 	/* allocation must have failed */
 	err = E_ALLOC;
@@ -1150,7 +1150,7 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
     char *s = line;
     char longopt[32];
     OptStatus status = 0;
-    gretlopt match, ret = 0L;
+    gretlopt match, ret = OPT_NONE;
 
     while ((s = strstr(s, "--")) != NULL) {
 	match = 0;
@@ -1166,12 +1166,18 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
 	    if (match > 0) {
 		/* recognized an acceptable option flag */
 		ret |= match;
-	    } else {
+	    } else if (status == OPT_AMBIGUOUS) {
+		/* abbreviation matches more than one option */
+		gretl_errmsg_sprintf(_("Ambiguous option '--%s'"), longopt);
+		fprintf(stderr, " line='%s', ci = %d\n", line, ci);
+		*err = 1;
+		return OPT_NONE;
+	    } else {		
 		/* not a valid flag, or not applicable in context */
 		gretl_errmsg_sprintf(_("Invalid option '--%s'"), longopt);
 		fprintf(stderr, " line='%s', ci = %d\n", line, ci);
 		*err = 1;
-		return 0L;
+		return OPT_NONE;
 	    } 
 	}
 
@@ -1191,7 +1197,7 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
 		*err = E_PARSE;
 	    }
 	    if (*err) {
-		return 0L;
+		return OPT_NONE;
 	    }	    
 	} else {
 	    s += 2;
@@ -1301,8 +1307,8 @@ gretlopt get_gretl_options (char *line, int *err)
 	return 0;
     }
 
-#if 0
-    fprintf(stderr, "get_gretl_options: '%s'\n", line);
+#if OPTDEBUG > 1
+    fprintf(stderr, "get_gretl_options, starting: '%s'\n", line);
 #endif
 
     get_cmdword(line, cmdword);
@@ -1368,6 +1374,10 @@ gretlopt get_gretl_options (char *line, int *err)
     /* strip trailing whitespace after processing */
     tail_strip(line);
 
+#if OPTDEBUG > 1
+    fprintf(stderr, "get_gretl_options, done: '%s'\n", line);
+#endif
+
     if (err != NULL) {
 	*err = myerr;
     }
@@ -1375,13 +1385,15 @@ gretlopt get_gretl_options (char *line, int *err)
     return oflags;
 }
 
-static int quote_option_parm (int ci, gretlopt opt, 
-			      const char *s)
+static int option_parm_needs_quoting (const char *s)
 {
-    if (ci == TABPRINT && opt == OPT_T) {
-	if (strchr(s, '%')) {
+    const char *qchars = "=%, ";
+
+    while (*s) {
+	if (strspn(s, qchars)) {
 	    return 1;
 	}
+	s++;
     }
 
     return 0;
@@ -1445,9 +1457,7 @@ const char *print_flags (gretlopt oflags, int ci)
 	    if (gretl_opts[i].parminfo) {
 		parm = get_optval_string(ci, opt);
 		if (parm != NULL && *parm != '\0') {
-		    if (strchr(parm, ' ')) {
-			pprintf(flagprn, "=\"%s\"", parm);
-		    } else if (quote_option_parm(ci, opt, parm)) {
+		    if (option_parm_needs_quoting(parm)) {
 			pprintf(flagprn, "=\"%s\"", parm);
 		    } else {
 			pprintf(flagprn, "=%s", parm);
