@@ -168,9 +168,15 @@ void gui_script_logo (PRN *prn)
 static void 
 print_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
 {
-    pprintf(prn, " %*s ", VNAMELEN - 1, cf->names[i]);
+    int n = strlen(cf->names[i]);
 
-    bufspace(3, prn);
+    if (n > 16) {
+	pprintf(prn, "%.15s~ ", cf->names[i]);
+	bufspace(3, prn);
+    } else {
+	pprintf(prn, "%14s ", cf->names[i]);
+	bufspace(5, prn);
+    }
 
     if (isnan(cf->coeff[i])) {
 	pprintf(prn, "%*s", UTF_WIDTH(_("undefined"), 16), _("undefined"));
@@ -178,8 +184,6 @@ print_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
 	gretl_print_value(cf->coeff[i], prn);
     }
 
-    bufspace(2, prn);
-
     if (isnan(cf->maxerr[i])) {
 	pprintf(prn, "%*s", UTF_WIDTH(_("undefined"), 10), _("undefined"));
     } else {
@@ -1817,7 +1821,8 @@ static void print_listed_objects (const char *s,
 	} else if ((b = get_bundle_by_name(name)) != NULL) {
 	    gretl_bundle_print(b, prn);
 	} else if ((p = get_string_by_name(name)) != NULL) {
-	    pprintf(prn, "%s\n", p);	
+	    pputs(prn, p);
+	    pputc(prn, '\n');
 	} 
 	free(name);
     }
