@@ -27,7 +27,7 @@
 #include <errno.h>
 #include <libset.h>
 
-#if defined(_OPENMP) && defined(USE_OPENMP)
+#if defined(_OPENMP) && !defined(OS_OSX)
 # include <omp.h>
 #endif
 
@@ -603,7 +603,7 @@ double student_cdf_inverse (double df, double a)
  * on failure.
  */
 
-double chisq_cdf (int df, double x)
+double chisq_cdf (double df, double x)
 {
     double p = NADBL;
 
@@ -627,7 +627,7 @@ double chisq_cdf (int df, double x)
  * on failure.
  */
 
-double chisq_cdf_comp (int df, double x)
+double chisq_cdf_comp (double df, double x)
 {
     double p = NADBL;
 
@@ -1597,7 +1597,11 @@ static double GHK_1 (const gretl_matrix *C,
  * Returns: an n x 1 vector of probabilities.
  */
 
-#if defined(_OPENMP) && defined(USE_OPENMP)
+/* Note: using openmp on this function requires thread-local
+   storage of static variables in gretl_matrix.c, and at
+   present we can't arrange that on OS X */
+
+#if defined(_OPENMP) && !defined(OS_OSX)
 
 gretl_matrix *gretl_GHK (const gretl_matrix *C,
 			 const gretl_matrix *A,
