@@ -968,15 +968,15 @@ char *shift_string_left (char *str, size_t move)
 }
 
 /**
- * gretl_chopstr:
+ * gretl_strstrip:
  * @str: the string to process.
  *
- * Removes both leading and trailing space from a string.
+ * Removes leading and trailing white space from a string.
  *
  * Returns: the possibly modified string.
  */
 
-char *gretl_chopstr (char *str)
+char *gretl_strstrip (char *str)
 {
     int i, n = strspn(str, " \t");
 
@@ -997,6 +997,41 @@ char *gretl_chopstr (char *str)
     return str;
 }
 
+/**
+ * gretl_strstrip_copy:
+ * @str: the string to process.
+ *
+ * Returns: a copy of @str, from which both leading and 
+ * trailing white space have been removed.
+ */
+
+char *gretl_strstrip_copy (const char *str, int *err)
+{
+    char *ret = NULL;
+    int i, n;
+
+    while (isspace(*str)) {
+	str++;
+    }
+
+    n = strlen(str);
+
+    for (i=n-1; i>=0; i--) {
+	if (isspace(str[i]) || str[i] == '\r') {
+	    n--;
+	} else {
+	    break;
+	}
+    }
+
+    ret = gretl_strndup(str, n);
+    if (ret == NULL) {
+	*err = E_ALLOC;
+    }
+
+    return ret;
+}
+
 /**
  * switch_ext:
  * @targ: the target or output string (must be pre-allocated).
@@ -1976,35 +2011,36 @@ void unescape_url (char *url)
 
 char *make_varname_unique (char *vname, int v, DATASET *dset) 
 {
-    const char *add = "abcdefghijklmnopqrstuvwxyz";
     size_t n = strlen(vname);
-    int i, j, conflict;
+    size_t nmax = VNAMELEN - 8;
+    char tmp[8];
+    int i, k, conflict;
 
-    if (n > 15) {
-	n = 15;
+    if (n > nmax) {
+	n = nmax;
     }
 
- tryagain:
-
-    for (j=0; j<26; j++) {
+    for (k=1; k<999999; k++) {
 	conflict = 0;
-	for (i=1; i<dset->v && !conflict; i++) {
+	for (i=1; i<dset->v; i++) {
 	    if (i != v && !strcmp(vname, dset->varname[i])) {
 		conflict = 1;
+		break;
 	    }
 	}
 	if (conflict) {
-	    vname[n] = add[j];
-	    vname[n+1] = '\0';
+	    sprintf(tmp, "_%d", k);
+	    vname[n] = '\0';
+	    strncat(vname, tmp, strlen(tmp));
 	} else {
+	    /* name is unique */
 	    break;
 	}
     }
-
-    if (conflict && n < 18) {
-	n++;
-	goto tryagain;
-    }
+    
+    if (conflict) {
+	fprintf(stderr, "make_varname_unique: unresolved conflict!\n");
+    }    
 
     return vname;
 }
@@ -2016,7 +2052,7 @@ int fix_varname_duplicates (DATASET *dset)
 
     for (i=1; i<dset->v; i++) {
 	for (j=i+1; j<dset->v; j++) {
-	    if (!strcmp(dset->varname[i], dset->varname[j])) {
+	    if (strcmp(dset->varname[i], dset->varname[j]) == 0) {
 		fprintf(stderr, "'%s' duplicated variable name\n", dset->varname[i]);
 		dups = 1;
 		make_varname_unique(dset->varname[j], j, dset);
