@@ -1013,6 +1013,12 @@ copy_data_to_subsample (DATASET *subset, const DATASET *dset,
 	}
     }
 
+    /* copy panel time data? */
+    if (dset->pantime != NULL && dataset_is_panel(subset) &&
+	subset->pd == dset->pd) {
+	subset->pantime = copyvec(dset->pantime, dset->pd);
+    }
+
     strcpy(subset->stobs, "1");
     sprintf(subset->endobs, "%d", subset->n);
 }
@@ -1668,7 +1674,7 @@ static int smpl_get_int (const char *s, DATASET *dset, int *err)
     if (integer_string(s)) {
 	k = atoi(s);
     } else if (gretl_is_scalar(s)) {
-	k = gretl_scalar_get_value(s);
+	k = gretl_scalar_get_value(s, NULL);
     } else {
 	k = (int) generate_scalar(s, dset, err);
     }
@@ -1982,7 +1988,12 @@ int add_dataset_to_model (MODEL *pmod, const DATASET *dset,
 	/* FIXME? */
 	destroy_dataset(pmod->dataset);
 	pmod->dataset = NULL;
-    }    
+    }
+
+#if SUBDEBUG
+    fprintf(stderr, "add_dataset_to_model: fullset=%p, pmod->submask=%p\n",
+	    (void *) fullset, (void *) pmod->submask);
+#endif
 
     if (fullset != NULL) {
 	sync_datainfo_members(dset);
@@ -1995,9 +2006,6 @@ int add_dataset_to_model (MODEL *pmod, const DATASET *dset,
 	/* no subsample info: pmod was estimated on the full dataset,
 	   so we'll reconstruct the full dataset */
 	sn = srcset->n;
-#if SUBDEBUG
-	fprintf(stderr, "pmod->submask = NULL, set sn = %d\n", sn);
-#endif
     } else {
 	/* pmod was estimated on a subsample, which has to 
 	   be reconstructed */
