@@ -214,10 +214,13 @@ static int real_user_var_add (const char *name,
     user_var *u = user_var_new(name, type, value);
     int err = 0;
 
-    /* use OPT_P for a private variable */
+    /* We use OPT_P for a private variable, OPT_A
+       when adding as a function argument, OPT_S
+       when adding as a "shell" variable.
+    */
 
 #if UVDEBUG
-    fprintf(stderr, "user_var_add: '%s'\n", name);
+    fprintf(stderr, "real_user_var_add: '%s'\n", name);
 #endif
 
     if (u == NULL) {
@@ -230,6 +233,9 @@ static int real_user_var_add (const char *name,
 	    } else if (opt & OPT_S) {
 		u->flags = UV_SHELL;
 	    }
+	    if (opt & OPT_A) {
+		u->level += 1;
+	    }
 	    uvars[n_vars] = u;
 	    set_nvars(n_vars + 1, "user_var_add");
 	}
@@ -701,9 +707,9 @@ int user_var_add_or_replace (const char *name,
     if (u != NULL && u->type != type) {
 	err = E_TYPES;
     } else if (u != NULL) {
-	err = user_var_replace_value(u, value);
+ 	err = user_var_replace_value(u, value);
     } else {
-	err = user_var_add(name, type, value);
+	err = real_user_var_add(name, type, value, OPT_NONE);
     }
 
     return err;
@@ -808,7 +814,7 @@ void set_scalar_edit_callback (void (*callback))
 
 int create_user_var (const char *name, GretlType type)
 {
-    return user_var_add(name, type, NULL);
+    return real_user_var_add(name, type, NULL, OPT_NONE);
 }
 
 /**
@@ -832,15 +838,7 @@ int create_user_var (const char *name, GretlType type)
 int arg_add_as_shell (const char *name, GretlType type,
 		      void *value)
 {
-    int err = real_user_var_add(name, type, value, OPT_S);
-
-    if (!err) {
-	user_var *u = uvars[n_vars-1];
-
-	u->level += 1;
-    }
-
-    return err;
+    return real_user_var_add(name, type, value, OPT_S | OPT_A);
 }
 
 /**
@@ -868,10 +866,9 @@ int copy_matrix_as (const gretl_matrix *m, const char *newname,
     if (m2 == NULL) {
 	err = E_ALLOC;
     } else {
-	err = user_var_add(newname, GRETL_TYPE_MATRIX, m2);
-	if (!err && fnarg) {
-	    uvars[n_vars-1]->level += 1;
-	}
+	gretlopt opt = fnarg ? OPT_A : OPT_NONE;
+
+	err = real_user_var_add(newname, GRETL_TYPE_MATRIX, m2, opt);
     }
 
     return err;
@@ -925,10 +922,7 @@ int copy_as_arg (const char *param_name, GretlType type, void *value)
     }	
 
     if (!err) {
- 	err = user_var_add(param_name, type, copyval);
-	if (!err) {
-	    uvars[n_vars-1]->level += 1;
-	}
+ 	err = real_user_var_add(param_name, type, copyval, OPT_A);
     }
 
     return err;
@@ -994,17 +988,20 @@ static int real_destroy_user_vars_at_level (int level, int type,
 	    break;
 	}
 	if (type > 0 && uvars[i]->type != type) {
+	    /* preserving this variable */
 	    nv++;
 	    continue;
 	}
 	if (uvar_levels_match(uvars[i], level)) {
 	    user_var_destroy(uvars[i]);
+	    /* shuffle the remainder down one place */
 	    for (j=i; j<n_vars-1; j++) {
 		uvars[j] = uvars[j+1];
 	    }
 	    uvars[n_vars-1] = NULL;
 	    i--;
 	} else {
+	    /* preserving */
 	    nv++;
 	}
     }
@@ -1014,6 +1011,45 @@ static int real_destroy_user_vars_at_level (int level, int type,
     return err;
 }
 
+static int destroy_user_vars_via_callback (int type)
+{
+    user_var **delvars = NULL;
+    int i, j, n = 0;
+    int err = 0;
+
+    for (i=0; i<n_vars; i++) {
+	if (uvars[i]->level == 0 && uvars[i]->type == type) {
+	    n++;
+	}
+    }
+
+    if (n == 0) {
+	return 0;
+    }
+
+    delvars = malloc(n * sizeof *delvars);
+    if (delvars == NULL) {
+	return E_ALLOC;
+    }
+
+    j = 0;
+    for (i=0; i<n_vars; i++) {
+	if (uvars[i]->level == 0 && uvars[i]->type == type) {
+	    delvars[j++] = uvars[i];
+	}
+    }    
+
+    for (j=0; j<n && !err; j++) {
+	err = (*user_var_callback)(delvars[j]->name, 
+				   delvars[j]->type, 
+				   UVAR_DELETE);
+    }
+
+    free(delvars);
+
+    return err;
+}
+
 /**
  * destroy_user_vars_at_level:
  * @level: stack level of function execution.
@@ -1043,6 +1079,35 @@ int destroy_private_matrices (void)
 					   0);    
 }
 
+int delete_user_vars_of_type (GretlType type, PRN *prn)
+{
+    int err = 0;
+
+    if (type == GRETL_TYPE_MATRIX ||
+	type == GRETL_TYPE_BUNDLE ||
+	type == GRETL_TYPE_STRING ||
+	type == GRETL_TYPE_DOUBLE ||
+	type == GRETL_TYPE_LIST) {
+	int level = gretl_function_depth();
+
+	if (level == 0 && user_var_callback != NULL &&
+	    (type == GRETL_TYPE_MATRIX || type == GRETL_TYPE_BUNDLE)) {
+	    err = destroy_user_vars_via_callback(type);
+	} else {	
+	    err = real_destroy_user_vars_at_level(level, type, 0);
+	}
+
+	if (!err && gretl_messages_on()) {
+	    pprintf(prn, "Deleted all variables of type %s\n",
+		    gretl_arg_type_name(type));
+	}
+    } else {
+	err = E_DATATYPE;
+    }
+
+    return err;
+}
+
 /**
  * destroy_private_scalars:
  *
@@ -1219,7 +1284,8 @@ int gretl_scalar_add (const char *name, double val)
 	    err = E_ALLOC;
 	} else {
 	    *px = val;
-	    err = user_var_add(name, GRETL_TYPE_DOUBLE, px);
+	    err = real_user_var_add(name, GRETL_TYPE_DOUBLE, 
+				    px, OPT_NONE);
 	}
 
 	if (!err && level == 0 && scalar_edit_callback != NULL) {
@@ -1239,38 +1305,44 @@ int add_auxiliary_scalar (const char *name, double val)
 	err = E_ALLOC;
     } else {
 	*px = val;
-	err = user_var_add(name, GRETL_TYPE_DOUBLE, px);
+	err = real_user_var_add(name, GRETL_TYPE_DOUBLE, 
+				px, OPT_NONE);
     }
 
     return err;
 }
 
-void gretl_scalar_set_value (const char *name, double val)
+int gretl_scalar_set_value (const char *name, double val)
 {
     user_var *u;
+    int err = 0;
 
     u = get_user_var_of_type_by_name(name, GRETL_TYPE_DOUBLE);
 
-    if (u != NULL) {
+    if (u == NULL) {
+	err = E_DATA;
+    } else {
 	*(double *) u->ptr = val;
 
 	if (scalar_edit_callback != NULL) {
 	    scalar_edit_callback();
 	}
     }
+
+    return err;
 }
 
-double gretl_scalar_get_value (const char *name)
+double gretl_scalar_get_value (const char *name, int *err)
 {
     user_var *u;
     double ret = NADBL;
 
     u = get_user_var_of_type_by_name(name, GRETL_TYPE_DOUBLE);
-    
+
     if (u != NULL) {
 	ret = *(double *) u->ptr;
     } else {
-	ret = get_const_by_name(name);
+	ret = get_const_by_name(name, err);
     }
 
     return ret;
@@ -1313,34 +1385,6 @@ char *get_string_by_name (const char *name)
     }
 }
 
-/**
- * add_string_as:
- * @s: string value to be added.
- * @name: the name of the string variable to add.
- *
- * Adds @s to the saved array of string variables 
- * under the name @name.
- *
- * Returns: 0 on success, non-zero on failure.
- */
-
-int add_string_as (const char *s, const char *name)
-{
-    char *scpy = gretl_strdup(s);
-    int err;
-
-    if (scpy == NULL) {
-	err = E_ALLOC;
-    } else {
-	err = user_var_add(name, GRETL_TYPE_STRING, scpy);
-	if (!err) {
-	    uvars[n_vars-1]->level += 1;
-	}
-    }
-
-    return err;
-}
-
 /**
  * gretl_is_string:
  * @name: name to test.
