@@ -1509,6 +1509,21 @@ int gretl_VAR_get_t2 (const GRETL_VAR *var)
     return (var == NULL)? 0 : var->t2;
 }
 
+int gretl_var_get_sample (const GRETL_VAR *var, int *t1, int *t2)
+{
+    if (var != NULL && var->models != NULL && var->models[0] != NULL) {
+	MODEL *pmod = var->models[0];
+
+	if (pmod->smpl.t1 >= 0 && pmod->smpl.t2 > pmod->smpl.t1) {
+	    *t1 = pmod->smpl.t1;
+	    *t2 = pmod->smpl.t2;
+	    return 0;
+	}
+    }
+
+    return E_DATA;
+}
+
 const MODEL *gretl_VAR_get_model (const GRETL_VAR *var, int i)
 {
     if (var != NULL && i < var->neqns) {
@@ -2523,7 +2538,7 @@ GRETL_VAR *gretl_VAR (int order, int *list, const DATASET *dset,
 		*err = gretl_VAR_do_error_decomp(var->S, var->C, NULL);
 	    }
 
-	    if (!*err) {
+	    if (!*err && prn != NULL) {
 		gretl_VAR_print(var, dset, opt, prn);
 	    }
 	}
@@ -2716,38 +2731,42 @@ static int johansen_degenerate_stage_1 (GRETL_VAR *v,
     const double **Z = (const double **) dset->Z;
     gretl_matrix *R0 = v->jinfo->R0;
     gretl_matrix *R1 = v->jinfo->R1;
-    int i, j, vi, s, t;
+    int i, vi, s, t, j = 0;
 
+#if 0
     fprintf(stderr, "degenerate stage 1\n");
+#endif
 
     for (i=0; i<v->neqns; i++) {
 	vi = v->ylist[i+1];
 	s = 0;
 	for (t=v->t1; t<=v->t2; t++) {
-	    gretl_matrix_set(R0, s, i, Z[vi][t] - Z[vi][t-1]);
-	    gretl_matrix_set(R1, s, i, Z[vi][t-1]);
+	    gretl_matrix_set(R0, s, j, Z[vi][t] - Z[vi][t-1]);
+	    gretl_matrix_set(R1, s, j, Z[vi][t-1]);
 	    s++;
 	}
+	j++;
     }
 
     if (auto_restr(v)) {
 	int trend = (jcode(v) == J_REST_TREND);
 
 	for (t=0; t<v->T; t++) {
-	    gretl_matrix_set(R1, t, v->neqns, (trend)? v->t1 + t : 1);
+	    gretl_matrix_set(R1, t, j, (trend)? v->t1 + t : 1);
 	}
+	j++;
     }
 
     if (v->rlist != NULL) {
 	for (i=0; i<v->rlist[0]; i++) {
 	    vi = v->rlist[i+1];
-	    j = v->neqns + i + 1;
 	    s = 0;
 	    for (t=v->t1; t<=v->t2; t++) {
 		gretl_matrix_set(R1, s++, j, Z[vi][t]); /* was t-1 */
 	    }
+	    j++;
 	}
-    } 
+    }
 
     johansen_fill_S_matrices(v);
 
