@@ -24,6 +24,7 @@
 #include "matrix_extra.h"
 #include "libset.h"
 #include "qr_estimate.h"
+#include "varprint.h"
 
 int gretl_VAR_normality_test (const GRETL_VAR *var, PRN *prn)
 {
@@ -728,11 +729,13 @@ static int *build_VAR_list (const GRETL_VAR *var, int *exolist, int *err)
 static int gretl_VAR_real_omit_test (const GRETL_VAR *orig,
 				     const GRETL_VAR *new,
 				     const DATASET *dset,
+				     gretlopt opt,
 				     PRN *prn)
 {
-    int *omitlist;
+    int *omitlist = NULL;
     double LR, pval;
-    int i, df, err = 0;
+    int df, nr = 0;
+    int err = 0;
 
 #if VO_DEBUG
     fprintf(stderr, "gretl_VAR_real_omit_test: about to diff lists\n");
@@ -740,109 +743,133 @@ static int gretl_VAR_real_omit_test (const GRETL_VAR *orig,
     printlist(new->xlist, "new xlist");
 #endif
 
-    if (new->xlist == NULL) {
-	omitlist = gretl_list_copy(orig->xlist);
-    } else {
-	omitlist = gretl_list_diff_new(orig->xlist, new->xlist, 1);
+    if (orig->xlist != NULL) {
+	if (new->xlist == NULL) {
+	    omitlist = gretl_list_copy(orig->xlist);
+	} else {
+	    omitlist = gretl_list_diff_new(orig->xlist, new->xlist, 1);
+	}
+	if (omitlist == NULL) {
+	    return E_ALLOC;
+	}
+	nr = omitlist[0];
     }
-
-    if (omitlist == NULL) {
-	return E_ALLOC;
+    if (opt & OPT_E) {
+	/* omitting seasonals */
+	nr += dset->pd + 1;
     }
+    if (opt & OPT_T) {
+	/* omitting trend */
+	nr++;
+    }    
 
     LR = orig->T * (new->ldet - orig->ldet);
-    df = orig->neqns * omitlist[0];
+    df = orig->neqns * nr;
     pval = chisq_cdf_comp(df, LR);
-    
-    pputs(prn, _("\n  Null hypothesis: the regression parameters are "
-		 "zero for the variables\n\n"));
-    for (i=1; i<=omitlist[0]; i++) {
-	pprintf(prn, "    %s\n", dset->varname[omitlist[i]]);	
-    }
 
-    pprintf(prn, "\n  %s: %s(%d) = %g, ", _("Test statistic"), 
+    record_test_result(LR, pval, _("omit"));
+
+    pprintf(prn, "%s:\n", _("Test on the original VAR"));
+
+    print_add_omit_null(omitlist, dset, opt | OPT_S, prn);
+
+    pprintf(prn, "  %s: %s(%d) = %g, ", _("LR test"), 
 	    _("Chi-square"), df, LR);
-    pprintf(prn, _("with p-value = %g\n\n"), pval);
+    pprintf(prn, _("with p-value = %g\n"), pval);
 
     free(omitlist);
 
     return err;
 }
 
+static int VAR_omit_check (GRETL_VAR *var, const int *omitlist,
+			   gretlopt opt)
+{
+    int nx = omitlist == NULL ? 0 : omitlist[0];
+    int err = 0;
+
+    if (nx == 0 && !(opt & (OPT_T | OPT_E))) {
+	/* nothing to be omitted */
+	err = E_NOOMIT;
+    } else if ((opt & OPT_T) && !(var->detflags & DET_TREND)) {
+	/* can't have the --trend option with no auto-trend */
+	err = E_BADOPT;
+    } else if ((opt & OPT_E) && !(var->detflags & DET_SEAS)) {
+	/* can't have the --seasonals option with no auto-seasonals */
+	err = E_BADOPT;
+    }    
+
+    return err;
+}
+
 /**
  * gretl_VAR_omit_test:
- * @omitvars: list of variables to omit from original model.
- * @orig: pointer to original VAR.
- * @dset: datset struct.
+ * @var: pointer to original VAR. 
+ * @omitlist: list of variables to omit from original model.
+ * @dset: dataset struct.
+ * @opt: option flags.
  * @prn: gretl printing struct.
  * @err: location to receive error code.
  *
  * Re-estimates a given VAR after removing the variables
- * specified in @omitvars, and reports per-equation F-tests
+ * specified in @omitlist, and reports per-equation F-tests
  * and system-wide LR tests for the null hypothesis that
  * the omitted variables have zero parameters.
  * 
- * Returns: restricted VAR on sucess, %NULL on error.
+ * Returns: restricted VAR on success, %NULL on error.
  */
 
-GRETL_VAR *gretl_VAR_omit_test (const int *omitvars, const GRETL_VAR *orig, 
-				DATASET *dset, PRN *prn, int *err)
+GRETL_VAR *gretl_VAR_omit_test (GRETL_VAR *var, const int *omitlist, 
+				DATASET *dset, gretlopt opt,
+				PRN *prn, int *err)
 {
-    GRETL_VAR *var = NULL;
-    gretlopt opt = OPT_NONE;
+    GRETL_VAR *vnew = NULL;
+    gretlopt varopt = OPT_NONE;
     int smpl_t1 = dset->t1;
     int smpl_t2 = dset->t2;
     int *tmplist = NULL;
     int *varlist = NULL;
     int c1 = 0;
 
-    *err = 0;
-
-    if (orig == NULL || orig->xlist == NULL) {
+    if (var == NULL) {
 	*err = E_DATA;
 	return NULL;
     }
 
-    if (omitvars == NULL || omitvars[0] == 0) {
-	*err = E_PARSE;
+    *err = VAR_omit_check(var, omitlist, opt);
+    if (*err) {
 	return NULL;
     }
 
-#if VO_DEBUG
-    printlist(orig->xlist, "original xlist");
-#endif
-
-    if (orig->ifc) {
-	c1 = !gretl_list_const_pos(omitvars, 1, dset);
-    } 
-
-    /* create reduced exogenous vars list for test VAR */
-    tmplist = gretl_list_omit(orig->xlist, omitvars, 1, err);
-    if (tmplist == NULL) {
-	goto bailout;
+    if (var->ifc) {
+	c1 = !gretl_list_const_pos(omitlist, 1, dset);
     }
 
-#if VO_DEBUG
-    fprintf(stderr, "c1 = %d\n", c1);
-    printlist(tmplist, "exog vars list for test VAR");
-#endif
+    if (omitlist != NULL && omitlist[0] > 0) {
+	/* create reduced exogenous vars list for test VAR */
+	tmplist = gretl_list_omit(var->xlist, omitlist, 1, err);
+	if (tmplist == NULL) {
+	    goto bailout;
+	}
+    } else if (var->xlist != NULL) {
+	tmplist = gretl_list_copy(var->xlist);
+	if (tmplist == NULL) {
+	    goto bailout;
+	}	
+    }
 
     /* create full input VAR list for test VAR */
-    varlist = build_VAR_list(orig, tmplist, err);
+    varlist = build_VAR_list(var, tmplist, err);
     if (varlist == NULL) {
 	goto bailout;
     }
 
-#if VO_DEBUG
-    printlist(varlist, "full list for test VAR");
-#endif
-
-    if (orig->detflags & DET_SEAS) {
-	opt |= OPT_D;
+    if ((var->detflags & DET_SEAS) && !(opt & OPT_E)) {
+	varopt |= OPT_D;
     }
 
-    if (orig->detflags & DET_TREND) {
-	opt |= OPT_T;
+    if ((var->detflags & DET_TREND) && !(opt & OPT_T)) {
+	varopt |= OPT_T;
     }
 
     /* If the original VAR did not include a constant, we need to
@@ -850,20 +877,23 @@ GRETL_VAR *gretl_VAR_omit_test (const int *omitvars, const GRETL_VAR *orig,
        We also need to pass OPT_N in case the constant was
        present originally but is now to be omitted.
     */
-    if (orig->ifc == 0 || c1 == 0) {
-	opt |= OPT_N;
+    if (var->ifc == 0 || c1 == 0) {
+	varopt |= OPT_N;
     }
 
     /* impose as sample range the estimation range of the 
        original VAR */
-    dset->t1 = orig->t1;
-    dset->t2 = orig->t2;
+    dset->t1 = var->t1;
+    dset->t2 = var->t2;
 
-    var = gretl_VAR(orig->order, varlist, dset, opt, prn, err);
+    vnew = gretl_VAR(var->order, varlist, dset, varopt, NULL, err);
 
-    /* now, if var is non-NULL, do the actual test(s) */
-    if (var != NULL) {
-	*err = gretl_VAR_real_omit_test(orig, var, dset, prn);
+    if (vnew != NULL) {
+	/* do the actual test(s) */
+	*err = gretl_VAR_real_omit_test(var, vnew, dset, opt, prn);
+	if (!*err && prn != NULL) {
+	    gretl_VAR_print(vnew, dset, OPT_NONE, prn);
+	}	
     }
 
     /* put back into dset what was there on input */
@@ -875,6 +905,175 @@ GRETL_VAR *gretl_VAR_omit_test (const int *omitvars, const GRETL_VAR *orig,
     free(tmplist);
     free(varlist);
 
-    return var;
+    return vnew;
 }
 
+static int set_wald_R_ones (gretl_matrix *R, int k, int r0, int neq,
+			    int stride)
+{
+    if (k < 0) {
+	return E_DATA;
+    } else {
+	int i, rmax = r0 + neq;
+
+	for (i=r0; i<rmax; i++) {
+	    gretl_matrix_set(R, i, k, 1.0);
+	    k += stride;
+	}
+	return 0;
+    }
+}
+
+/**
+ * gretl_VAR_wald_omit_test:
+ * @var: pointer to original. 
+ * @omitlist: list of variables to omit from original model.
+ * @dset: dataset struct.
+ * @opt: option flags.
+ * @prn: gretl printing struct.
+ *
+ * Runs a Wald test for the joint omission of the exogenous
+ * variables specified in @omitlist.
+ * 
+ * Returns: 0 on successful completion, non-zero error code
+ * otherwise.
+ */
+
+int gretl_VAR_wald_omit_test (GRETL_VAR *var, const int *omitlist, 
+			      DATASET *dset, gretlopt opt,
+			      PRN *prn)
+{
+    gretl_matrix *B = NULL;
+    gretl_matrix *S = NULL;
+    gretl_matrix *V = NULL;
+    gretl_matrix *R = NULL;
+    gretl_matrix *RB = NULL;
+    gretl_matrix *RVR = NULL;
+    double x, test = NADBL;
+    int i, j, neq, eqk, row0;
+    int pos, nr = 0, nx_omit = 0;
+    int err = 0;
+
+    if (var == NULL || var->B == NULL || 
+	var->S == NULL || var->XTX == NULL) {
+	return E_DATA;
+    }
+
+    err = VAR_omit_check(var, omitlist, opt);
+    if (err) {
+	return err;
+    }
+
+    if (omitlist != NULL) {
+	nx_omit = omitlist[0];
+    }
+
+    B = gretl_matrix_vectorize_new(var->B);
+    if (B == NULL) {
+	return E_ALLOC;
+    }
+
+    neq = var->neqns;
+
+    if (nx_omit > 0) {
+	/* omitting user-specified exogenous vars */
+	nr += nx_omit * neq;
+    }
+    if (opt & OPT_T) {
+	/* omit trend */
+	nr += neq;
+    }
+    if (opt & OPT_E) {
+	/* omit seasonals */
+	nr += (dset->pd - 1) * neq;
+    }
+
+    S = gretl_zero_matrix_new(neq, neq);
+    R = gretl_zero_matrix_new(nr, B->rows);
+    RB = gretl_matrix_alloc(nr, 1);
+    RVR = gretl_matrix_alloc(nr, nr);
+
+    if (S == NULL || R == NULL || RB == NULL || RVR == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    for (i=0; i<neq; i++) {
+	x = gretl_matrix_get(var->S, i, i);
+	gretl_matrix_set(S, i, i, x);
+    }
+
+    V = gretl_matrix_kronecker_product_new(S, var->XTX, &err);
+    if (err) {
+	goto bailout;
+    }
+
+    eqk = var->B->rows;
+    pos = var->ifc + neq * var->order;
+    row0 = 0;
+
+    if (nx_omit > 0) {
+	/* @omitlist holds ID numbers of exog variables */ 
+	int vi;
+
+	for (i=1; i<=omitlist[0]; i++) {
+	    vi = omitlist[i];
+	    if (vi == 0) {
+		j = 0;
+	    } else {
+		j = in_gretl_list(var->xlist, vi);
+		j += pos - 1;
+	    }
+	    set_wald_R_ones(R, j, row0, neq, eqk);
+	    row0 += neq;
+	}	
+    }
+    if (opt & OPT_E) {
+	/* omit auto-seasonals */
+	for (i=1; i<dset->pd; i++) {
+	    j = pos + i - 1;
+	    set_wald_R_ones(R, j, row0, neq, eqk);
+	    row0 += neq;
+	}
+    }   
+    if (opt & OPT_T) {
+	/* omit auto-trend (comes as last param) */
+	j = eqk - 1;
+	set_wald_R_ones(R, j, row0, neq, eqk);
+	row0 += neq;
+    }
+
+    gretl_matrix_multiply(R, B, RB);
+    gretl_matrix_qform(R, GRETL_MOD_NONE, V,
+		       RVR, GRETL_MOD_NONE);
+    err = gretl_invert_symmetric_matrix(RVR);
+
+    if (!err) {
+	test = gretl_scalar_qform(RB, RVR, &err);
+	if (!err) {
+	    double pval = chisq_cdf_comp(nr, test);
+
+	    record_test_result(test, pval, _("omit"));
+
+	    if (!(opt & OPT_I)) {
+		/* not silent */
+		pprintf(prn, "%s:\n", _("Test on VAR"));
+		print_add_omit_null(omitlist, dset, opt | OPT_S, prn);
+		pprintf(prn, "  %s: %s(%d) = %g, %s %g\n\n",  _("Wald test"),
+			_("Chi-square"), nr, test,
+			_("p-value"), pval);
+	    }
+	}
+    }
+
+ bailout:
+
+    gretl_matrix_free(B);
+    gretl_matrix_free(S);
+    gretl_matrix_free(V);
+    gretl_matrix_free(R);
+    gretl_matrix_free(RB);
+    gretl_matrix_free(RVR);
+
+    return err;
+}
