@@ -4,7 +4,7 @@
    Based on stataread.c from the GNU R "foreign" package with the 
    following original info:
 
-     * $Id: stata_import.c,v 1.48 2012/01/06 14:52:20 allin Exp $
+     * $Id: stata_import.c,v 1.54 2012/10/14 22:35:39 allin Exp $
   
      (c) 1999, 2000, 2001, 2002 Thomas Lumley. 
      2000 Saikat DebRoy
@@ -324,7 +324,7 @@ dta_make_string_table (int *types, int nvar, int ncols)
 	}
     }
 
-    st = string_table_new_from_cols_list(list);
+    st = gretl_string_table_new(list);
 
     free(list);
 
@@ -459,7 +459,7 @@ static int dta_value_labels_setup (PRN **pprn, gretl_string_table **pst)
 
     if (*pprn != NULL) {
 	if (*pst == NULL) {
-	    *pst = gretl_string_table_new(&err);
+	    *pst = gretl_string_table_new(NULL);
 	    if (*pst == NULL) {
 		gretl_print_destroy(*pprn);
 		*pprn = NULL;
@@ -550,7 +550,7 @@ static int read_dta_data (FILE *fp, DATASET *dset,
     int *types = NULL;
     int *lvars = NULL;
     char **lnames = NULL;
-    char strbuf[129];
+    char strbuf[256];
     char *txt = NULL; 
     int *off = NULL;
     int st_err = 0;
@@ -649,12 +649,12 @@ static int read_dta_data (FILE *fp, DATASET *dset,
 	if (*label != '\0') {
 	    pprintf(vprn, "variable %d: label = '%s'\n", i+1, label);
 	    if (g_utf8_validate(label, -1, NULL)) {
-		strncat(VARLABEL(dset, i+1), label, MAXLABEL - 1);
+		series_set_label(dset, i+1, label);
 	    } else {
 		gchar *tr = recode_stata_string(label);
 
 		if (tr != NULL) {
-		    gretl_utf8_strncat(VARLABEL(dset, i+1), tr, MAXLABEL - 1);
+		    series_set_label(dset, i+1, tr);
 		    g_free(tr);
 		}
 	    } 
@@ -707,23 +707,26 @@ static int read_dta_data (FILE *fp, DATASET *dset,
 		dset->Z[v][t] = (ix == NA_INT)? NADBL : ix;
 	    } else {
 		clen = types[i] - soffset;
+		if (clen > 255) {
+		    clen = 255;
+		} 
 		stata_read_string(fp, clen, strbuf, &err);
 		strbuf[clen] = 0;
 #if 0
 		fprintf(stderr, "Z[%d][%d] = '%s'\n", v, t, strbuf);
 #endif
-		if (*strbuf != '\0' && strcmp(strbuf, ".") && *pst != NULL) {
+		if (!err && *strbuf != '\0' && strcmp(strbuf, ".") && *pst != NULL) {
 		    ix = gretl_string_table_index(*pst, strbuf, v, 0, prn);
 		    if (ix > 0) {
 			dset->Z[v][t] = ix;
 			if (t == 0) {
-			    set_var_discrete(dset, v, 1);
+			    series_set_discrete(dset, v, 1);
 			}
 		    }	
 		}
 	    }
 
-	    if (i == tnum && t == 0) {
+	    if (i == tnum && t == 0 && !err) {
 		set_time_info((int) dset->Z[v][t], pd, dset);
 	    }
 	}
@@ -833,7 +836,7 @@ static int read_dta_data (FILE *fp, DATASET *dset,
     free(types);
 
     if (lvars != NULL) {
-	free_strings_array(lnames, lvars[0]);
+	strings_array_free(lnames, lvars[0]);
 	free(lvars);
     }
 
@@ -946,7 +949,7 @@ int dta_get_data (const char *fname, DATASET *dset,
 	int nvtarg = newset->v - 1;
 
 	if (nvread < nvtarg) {
-	    dataset_drop_last_variables(nvtarg - nvread, newset);
+	    dataset_drop_last_variables(newset, nvtarg - nvread);
 	}
 	
 	if (fix_varname_duplicates(newset)) {
